// -------------------------------------------------------------------------
//
// Copyright or © or Copr. CNRS, (November 16, 2004)
//
// This software is a library providing tool for bioinformatics.
//
// This software is governed by the CeCILL  license under French law and
// abiding by the rules of distribution of free software.  You can  use,
// modify and/ or redistribute the software under the terms of the CeCILL
// license as circulated by CEA, CNRS and INRIA at the following URL
// "http://www.cecill.info".
//
// As a counterpart to the access to the source code and  rights to copy,
// modify and redistribute granted by the license, users are provided only
// with a limited warranty  and the software's author,  the holder of the
// economic rights,  and the successive licensors  have only  limited
// liability.
//
// In this respect, the user's attention is drawn to the risks associated
// with loading,  using,  modifying and/or developing or reproducing the
// software by the user in light of its specific status of free software,
// that may mean  that it is complicated to manipulate,  and  that  also
// therefore means  that it is reserved for developers  and  experienced
// professionals having in-depth computer knowledge. Users are therefore
// encouraged to load and test the software's suitability as regards their
// requirements in conditions enabling the security of their systems and/or
// data to be ensured and,  more generally, to use and operate it in the
// same conditions as regards security.
//
// The fact that you are presently reading this means that you have had
// knowledge of the CeCILL license and that you accept its terms.
//
// -------------------------------------------------------------------------

/**
 * <p>Description : Baobab core classes.</p>
 * <p>Copyright : Copyright (c) Julien Dutheil</p>
 * @author Julien Dutheil
 */

package TreeEditor;

import bio.dendogram.*;
import bio.dendogram.xml.*;
import java.awt.Color;

public class ColoredTree extends XTree {

  /**
   * Constructors:
   */

  public ColoredTree() { super(); }
  public ColoredTree(String name) { super(name); }
  public ColoredTree(String name, ColoredNode root) { super(name, root); }
  public ColoredTree(ColoredNode root) { super(root); }
  public ColoredTree(Tree tree) { super(tree); }

  public Object clone() {
    //WARNING!!! This method clones all internal nodes too.
    Tree result = new ColoredTree((ColoredNode)cloneSubtree(root));
    result.setName(name);
    return result;
  }

  public Node createNode() { return new ColoredNode(super.createNode()); }
  public Node createNode(String nodeName, boolean checkNames) throws NodeException {
    return new ColoredNode(super.createNode(nodeName, checkNames));
  }
  public Node createNode(Node node) {
    Node newNode = new ColoredNode(node);
    newNode.setId(++nextId);
    return newNode;
  }

  /**
   * Here are the ColoredTree object (new defined) specific methods:
   */


  public static void setColorsOfAllSubtrees(ColoredNode node, Color color, int attribut) {
    node.setColor(color, attribut);
    if(!node.isLeaf()) {
      for(int i = 0; i < node.getNumberOfSons(); i++) {
        setColorsOfAllSubtrees(((ColoredNode)node.getSon(i)), color, attribut);
      }
    }
  }
  public void setColorsOfTree(Color color, int attribut) {
   setColorsOfAllSubtrees((ColoredNode)root, color, attribut);
  }

  public static void setColorsOfAllSubtrees(ColoredNode node, Color[] colors) {
    node.setColors((Color[])colors.clone());
    if(!node.isLeaf()) {
      for(int i = 0; i < node.getNumberOfSons(); i++) {
        ((ColoredNode)node.getSon(i)).setColors(colors);
      }
    }
  }
  public void setColorsToTree(Color[] colors) {
   setColorsOfAllSubtrees((ColoredNode)root, colors);
  }

}
