// -------------------------------------------------------------------------
//
// Copyright or © or Copr. CNRS, (November 16, 2004)
//
// This software is a library providing tool for bioinformatics.
//
// This software is governed by the CeCILL  license under French law and
// abiding by the rules of distribution of free software.  You can  use,
// modify and/ or redistribute the software under the terms of the CeCILL
// license as circulated by CEA, CNRS and INRIA at the following URL
// "http://www.cecill.info".
//
// As a counterpart to the access to the source code and  rights to copy,
// modify and redistribute granted by the license, users are provided only
// with a limited warranty  and the software's author,  the holder of the
// economic rights,  and the successive licensors  have only  limited
// liability.
//
// In this respect, the user's attention is drawn to the risks associated
// with loading,  using,  modifying and/or developing or reproducing the
// software by the user in light of its specific status of free software,
// that may mean  that it is complicated to manipulate,  and  that  also
// therefore means  that it is reserved for developers  and  experienced
// professionals having in-depth computer knowledge. Users are therefore
// encouraged to load and test the software's suitability as regards their
// requirements in conditions enabling the security of their systems and/or
// data to be ensured and,  more generally, to use and operate it in the
// same conditions as regards security.
//
// The fact that you are presently reading this means that you have had
// knowledge of the CeCILL license and that you accept its terms.
//
// -------------------------------------------------------------------------

/**
 * Titre : Dendogram<p>
 * Description : Set of Tools for dealing with phylogenetic trees (dendograms).<p>
 * Copyright : 2002<p>
 * @author Julien Dutheil
 * @version 1.0
 */
package bio.dendogram.GUI;

import java.util.*;
import java.io.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

public class PrintWriterDialog extends JDialog implements ActionListener {

  public PrintWriterDialog(File output) {
    this.output = output;
    message = ResourceBundle.getBundle("bio.dendogram.GUI.bundle", getLocale());
    setTitle(message.getString("FileExistsDialogTitle"));
    Box box = Box.createVerticalBox();
    overwrite = new JButton(message.getString("FileAlreadyExistsOverwrite"));
    append = new JButton(message.getString("FileAlreadyExistsAppend"));
    cancel = new JButton(message.getString("FileAlreadyExistsCancel"));
    pack();
  }

  public PrintWriter getPrintWriter() {
    return pw;
  }

  public void actionPerformed(ActionEvent e) {
    if(e.getSource() == overwrite) {
      try {
        pw = new PrintWriter(new FileWriter(output));
      } catch(IOException ex) {
        ex.printStackTrace();
      }
    }
    if(e.getSource() == append) {
      BufferedReader input;
      try {
        input = new BufferedReader(new FileReader(output));
        ArrayList al = new ArrayList();
        String line = input.readLine();
        do {
          al.add(line);
          line = input.readLine();
        }
        while(line != null);
        input.close();
        pw = new PrintWriter(new FileWriter(output));
        for(int i = 0 ; i < al.size(); i++) pw.println((String)al.get(i));
      } catch(IOException ex) {
        ex.printStackTrace();
      }
    }
    if(e.getSource() == cancel) pw = null;
    setVisible(false);
  }

  ResourceBundle message;
  JButton overwrite, append, cancel;
  File output;
  PrintWriter pw;
  public static final int OVERWRITE = 1;
  public static final int APPEND = 2;
  public static final int CANCEL = 0;
}
