// -------------------------------------------------------------------------
//
// Copyright or © or Copr. CNRS, (November 16, 2004)
//
// This software is a library providing tool for bioinformatics.
//
// This software is governed by the CeCILL  license under French law and
// abiding by the rules of distribution of free software.  You can  use,
// modify and/ or redistribute the software under the terms of the CeCILL
// license as circulated by CEA, CNRS and INRIA at the following URL
// "http://www.cecill.info".
//
// As a counterpart to the access to the source code and  rights to copy,
// modify and redistribute granted by the license, users are provided only
// with a limited warranty  and the software's author,  the holder of the
// economic rights,  and the successive licensors  have only  limited
// liability.
//
// In this respect, the user's attention is drawn to the risks associated
// with loading,  using,  modifying and/or developing or reproducing the
// software by the user in light of its specific status of free software,
// that may mean  that it is complicated to manipulate,  and  that  also
// therefore means  that it is reserved for developers  and  experienced
// professionals having in-depth computer knowledge. Users are therefore
// encouraged to load and test the software's suitability as regards their
// requirements in conditions enabling the security of their systems and/or
// data to be ensured and,  more generally, to use and operate it in the
// same conditions as regards security.
//
// The fact that you are presently reading this means that you have had
// knowledge of the CeCILL license and that you accept its terms.
//
// -------------------------------------------------------------------------

/**
 * Titre : Dendogram<p>
 * Description : Set of Tools for dealing with phylogenetic trees (dendograms).<p>
 * Copyright : 2002<p>
 * @author null
 * @version 1.0
 */
package bio.dendogram.GUI;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class TreeDrawingControls {

  public TreeDrawingControls(TreeDrawing treeDrawing) {
    this.treeDrawing = treeDrawing;
    initActions();
    initButtons();
  }

  private void initActions() {
    drawLengths = new AbstractAction("Draw Lengths", new ImageIcon(getClass().getResource("DrawLengths.gif"))) {
      public void actionPerformed(ActionEvent e) {
        treeDrawing.setDrawLengths(drawLengthsButton.isSelected());
        treeDrawing.repaint();
      }
    };
    drawBootstraps = new AbstractAction("Draw Bootstraps", new ImageIcon(getClass().getResource("DrawBootstraps.gif"))) {
      public void actionPerformed(ActionEvent e) {
        treeDrawing.setDrawBootstraps(drawBootstrapsButton.isSelected());
        treeDrawing.repaint();
      }
    };
    drawIds = new AbstractAction("Draw Ids", new ImageIcon(getClass().getResource("DrawIds.gif"))) {
      public void actionPerformed(ActionEvent e) {
        treeDrawing.setDrawIds(drawIdsButton.isSelected());
        treeDrawing.repaint();
      }
    };
//    drawGroups = new AbstractAction("Draw Groups", new ImageIcon(getClass().getResource("DrawGroups.gif"))) {
//      public void actionPerformed(ActionEvent e) {
//        treeDrawing.setDrawGroups(drawGroupsButton.isSelected());
//        treeDrawing.repaint();
//      }
//    };
    drawClickableNodes = new AbstractAction("Draw Clickable Nodes", new ImageIcon(getClass().getResource("DrawClickableNodes.gif"))) {
      public void actionPerformed(ActionEvent e) {
        treeDrawing.setDrawClickableNodes(drawClickableNodesButton.isSelected());
        treeDrawing.repaint();
      }
    };
    drawNumbersAtFirstBranches = new AbstractAction("Draw numbers at first branches", new ImageIcon(getClass().getResource("DrawNumbersAtFirstBranches.gif"))) {
      public void actionPerformed(ActionEvent e) {
        treeDrawing.setDrawBranchesNumbers(drawNumbersAtFirstBranchesButton.isSelected());
        treeDrawing.repaint();
      }
    };
    drawLeavesNames = new AbstractAction("Draw leaves names", new ImageIcon(getClass().getResource("DrawLeavesNames.gif"))) {
      public void actionPerformed(ActionEvent e) {
        treeDrawing.setDrawLeavesNames(!drawLeavesNamesButton.isSelected()); // The button is for hiding the names!
        treeDrawing.repaint();
      }
    };
  }

  private void initButtons() {
    drawLengthsButton = new JToggleButton(drawLengths);
    drawLengthsButton.setText("");
    drawBootstrapsButton = new JToggleButton(drawBootstraps);
    drawBootstrapsButton.setText("");
    drawIdsButton = new JToggleButton(drawIds);
    drawIdsButton.setText("");
//    drawGroupsButton = new JToggleButton(drawGroups);
//    drawGroupsButton.setText("");
    drawClickableNodesButton = new JToggleButton(drawClickableNodes);
    drawClickableNodesButton.setText("");
    drawNumbersAtFirstBranchesButton = new JToggleButton(drawNumbersAtFirstBranches);
    drawNumbersAtFirstBranchesButton.setText("");
    drawLeavesNamesButton = new JToggleButton(drawLeavesNames);
    drawLeavesNamesButton.setText("");
    actualize();
  }

  public void actualize() {
  /**
   * When other objects affect the tree drawing.
   * Perhaps I may manage a change listener or something.
   */
    drawLengthsButton.setSelected(treeDrawing.drawLengths());
    drawBootstrapsButton.setSelected(treeDrawing.drawBootstraps());
    drawIdsButton.setSelected(treeDrawing.drawIds());
//    drawGroupsButton.setSelected(treeDrawing.drawGroups());
    drawClickableNodesButton.setSelected(treeDrawing.drawClickableNodes());
    drawNumbersAtFirstBranchesButton.setSelected(treeDrawing.drawBranchesNumbers());
    drawLeavesNamesButton.setSelected(!treeDrawing.drawLeavesNames()); // The button is for hiding names!
  }

  public Action getDrawLengths()                { return drawLengths; }
  public Action getDrawBootstraps()             { return drawBootstraps; }
  public Action getDrawIds()                    { return drawIds; }
//  public Action getDrawGroups()                 { return drawGroups; }
  public Action getDrawNumbersAtFirstBranches() { return drawNumbersAtFirstBranches; }
  public Action getDrawClickableNodesLengths()  { return drawClickableNodes; }

  public JToggleButton getDrawLengthsButton()                { return drawLengthsButton; }
  public JToggleButton getDrawBootstrapsButton()             { return drawBootstrapsButton; }
  public JToggleButton getDrawIdsButton()                    { return drawIdsButton; }
//  public JToggleButton getDrawGroupsButton()                 { return drawClickableNodesButton; }
  public JToggleButton getDrawNumbersAtFirstBranchesButton() { return drawNumbersAtFirstBranchesButton; }
  public JToggleButton getDrawClickableNodesLengthsButton()  { return drawLengthsButton; }

  private TreeDrawing treeDrawing;
  public Action drawLengths
              , drawBootstraps
              , drawIds
  //            , drawGroups
              , drawClickableNodes
              , drawNumbersAtFirstBranches
              , drawLeavesNames
              ;
   public JToggleButton drawLengthsButton
                      , drawBootstrapsButton
                      , drawIdsButton
//                      , drawGroupsButton
                      , drawClickableNodesButton
                      , drawNumbersAtFirstBranchesButton
                      , drawLeavesNamesButton
                      ;
  }
