package bio.dendogram.io;

/**
 * <p>Title: Dendogram</p>
 * <p>Description: Set of Tools for dealing with phylogenetic trees (dendograms).</p>
 * <p>Copyright: 2002</p>
 * <p>Company: </p>
 * @author not attributable
 * @version 1.0
 */

import bio.dendogram.*;
import java.io.*;
import java.awt.Component;
import java.util.*;
import javax.swing.*;

public abstract class AbstractOTree
    implements OTree {

  public void writeTree(Tree tree, File output) throws IOException {
    PrintWriter pw = new PrintWriter(new FileWriter(output));
    writeTree(tree, pw);
    pw.close();
  }

  public void writeTree(Tree[] trees, File output) throws IOException {
    PrintWriter pw = new PrintWriter(new FileWriter(output));
    writeTree(trees, pw);
    pw.close();
  }

  protected static int options(Component owner) {
    String[] choices = {
        "Overwrite", "Append", "Cancel"};
    return JOptionPane.showOptionDialog(owner
                                        , "What do you want to do?"
                                        , "File already exists."
                                        , JOptionPane.YES_NO_CANCEL_OPTION
                                        , JOptionPane.QUESTION_MESSAGE
                                        , null
                                        , choices
                                        , choices[1]
                                        );
  }

  protected static PrintWriter getPrintWriter(File output, Component owner) throws
      IOException {
    PrintWriter pw = null;
    if (output.exists()) {
      int c = options(owner);
      if (c < 0 || c == 2) {
        throw new IOException();
      }
      if (c == 0) {
        pw = new PrintWriter(new FileWriter(output)); //the output file
      }
      if (c == 1) {
        BufferedReader input = new BufferedReader(new FileReader(output));
        ArrayList file = new ArrayList();
        String line = input.readLine();
        do {
          file.add(line);
          line = input.readLine();
        }
        while (line != null);
        input.close();
        pw = new PrintWriter(new FileWriter(output)); //the output file
        for (int i = 0; i < file.size(); i++) {
          pw.println( (String) file.get(i));
        }
      }
    }
    else {
      pw = new PrintWriter(new FileWriter(output));
    } //the output file
    return pw;
  }

  public void writeTree(Tree[] trees, PrintWriter pw) throws IOException {
    for (int i = 0; i < trees.length; i++) {
      writeTree(trees[i], pw);
      pw.println();
    }
  }

  public void writeTree(Tree tree, File output, Component owner) throws
      IOException {
    PrintWriter pw = getPrintWriter(output, owner); //the output file
    writeTree(tree, pw);
    pw.close();
  }

  public void writeTree(Tree[] trees, File output, Component owner) throws
      IOException {
    PrintWriter pw = getPrintWriter(output, owner); //the output file
    writeTree(trees, pw);
    pw.close();
  }

}
