// -------------------------------------------------------------------------
//
// Copyright or © or Copr. CNRS, (November 16, 2004)
//
// This software is a library providing tool for bioinformatics.
//
// This software is governed by the CeCILL  license under French law and
// abiding by the rules of distribution of free software.  You can  use,
// modify and/ or redistribute the software under the terms of the CeCILL
// license as circulated by CEA, CNRS and INRIA at the following URL
// "http://www.cecill.info".
//
// As a counterpart to the access to the source code and  rights to copy,
// modify and redistribute granted by the license, users are provided only
// with a limited warranty  and the software's author,  the holder of the
// economic rights,  and the successive licensors  have only  limited
// liability.
//
// In this respect, the user's attention is drawn to the risks associated
// with loading,  using,  modifying and/or developing or reproducing the
// software by the user in light of its specific status of free software,
// that may mean  that it is complicated to manipulate,  and  that  also
// therefore means  that it is reserved for developers  and  experienced
// professionals having in-depth computer knowledge. Users are therefore
// encouraged to load and test the software's suitability as regards their
// requirements in conditions enabling the security of their systems and/or
// data to be ensured and,  more generally, to use and operate it in the
// same conditions as regards security.
//
// The fact that you are presently reading this means that you have had
// knowledge of the CeCILL license and that you accept its terms.
//
// -------------------------------------------------------------------------

package bio.dendogram.xml;

/**
 * <p>Title: Bio</p>
 * <p>Description: Tools for bioinformatics</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author Julien Dutheil
 * @version 1.0
 */

import bio.dendogram.io.*;
import bio.dendogram.*;
import gnu.jtools.utils.storage.ReverseMap;
import java.io.*;
import java.awt.*;

public abstract class AbstractXMLOTree extends AbstractOTree implements XMLOTree {

  protected ReverseMap tagDescription;

  public AbstractXMLOTree() {
  }

  public ReverseMap getTagDescription() { return tagDescription; }

  public void setTagDescription(ReverseMap tagDescription) { this.tagDescription = tagDescription; }

  public void writeTree(XTree tree, File output) throws IOException {
    PrintWriter pw = new PrintWriter(new FileWriter(output));
    writeTree(tree, pw);
    pw.close();
  }

  public void writeTree(XTree[] trees, File output) throws IOException {
    PrintWriter pw = new PrintWriter(new FileWriter(output));
    writeTree(trees, pw);
    pw.close();
  }

  public void writeTree(XTree[] trees, PrintWriter pw) throws IOException {
    for(int i = 0; i < trees.length; i++) { writeTree(trees[i], pw); pw.println(); }
  }

  public void writeTree(XTree tree, File output, Component owner) throws IOException {
    PrintWriter pw = getPrintWriter(output, owner); //the output file
    writeTree(tree, pw);
    pw.close();
  }

  public void writeTree(XTree[] trees, File output, Component owner) throws IOException {
    PrintWriter pw = getPrintWriter(output, owner); //the output file
    writeTree(trees, pw);
    pw.close();
  }

  public void writeTree(Tree tree, PrintWriter pw) throws IOException {
    writeTree(new XTree(tree), pw);
  }

}
