// -------------------------------------------------------------------------
//
// Copyright or © or Copr. CNRS, (November 16, 2004)
//
// This software is a library providing tool for bioinformatics.
//
// This software is governed by the CeCILL  license under French law and
// abiding by the rules of distribution of free software.  You can  use,
// modify and/ or redistribute the software under the terms of the CeCILL
// license as circulated by CEA, CNRS and INRIA at the following URL
// "http://www.cecill.info".
//
// As a counterpart to the access to the source code and  rights to copy,
// modify and redistribute granted by the license, users are provided only
// with a limited warranty  and the software's author,  the holder of the
// economic rights,  and the successive licensors  have only  limited
// liability.
//
// In this respect, the user's attention is drawn to the risks associated
// with loading,  using,  modifying and/or developing or reproducing the
// software by the user in light of its specific status of free software,
// that may mean  that it is complicated to manipulate,  and  that  also
// therefore means  that it is reserved for developers  and  experienced
// professionals having in-depth computer knowledge. Users are therefore
// encouraged to load and test the software's suitability as regards their
// requirements in conditions enabling the security of their systems and/or
// data to be ensured and,  more generally, to use and operate it in the
// same conditions as regards security.
//
// The fact that you are presently reading this means that you have had
// knowledge of the CeCILL license and that you accept its terms.
//
// -------------------------------------------------------------------------

package bio.dendogram.xml;

import gnu.jtools.utils.file.FileTools;
import java.io.*;
import java.util.*;
import bio.dendogram.*;

/**
 * <p>Title: Bio</p>
 * <p>Description: Tools for bioinformatics</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author Julien Dutheil
 * @version 1.0
 */

public class PagelFormat extends AbstractXMLOTree {

  public PagelFormat() {
  }

  public void writeTree(Tree tree, PrintWriter output) throws java.io.IOException {
    writeNode(tree.getRootNode(), output);
  }

  public void writeNode(Node node, PrintWriter output) {
    Node father = node.getFather();
    if(father != null) {
      output.println(
          (node.hasName() ? node.getName() : "" + node.getId())
          + "\t" +
          (father.hasName() ? father.getName() : "" + father.getId())
          );
    }
    for(int i = 0; i < node.getNumberOfSons(); i++) writeNode(node.getSon(i), output);
  }

  public void writeTree(XTree tree, PrintWriter output) throws java.io.IOException {
    Set atts = XTreeTools.getAvailableTags(tree, false);
    writeNode((XNode)tree.getRootNode(), output, (String[])atts.toArray(new String[0]));
  }

  public void writeNode(XNode node, PrintWriter output, String[] atts) {
    XNode father = (XNode)node.getFather();
    if(father != null) {
      output.print(
          (node.hasName() ? node.getName() : "" + node.getId())
          + "\t" +
          (father.hasName() ? father.getName() : "" + father.getId())
          );
    } else {
      output.print(
          (node.hasName() ? node.getName() : "" + node.getId())
          + "\tROOT"
          );
    }
    for(int i = 0; i < atts.length; i++) {
      //Print each tag value:
      Object value = node.getAttribute((String)tagDescription.getValue(atts[i]));
      output.print("\t" + (value == null ? "NA" : value.toString()));
    }
    output.println();

    for(int i = 0; i < node.getNumberOfSons(); i++) writeNode((XNode)node.getSon(i), output, atts);
  }

  public static javax.swing.filechooser.FileFilter PAGEL_FILTER = new javax.
      swing.filechooser.FileFilter() {

    public boolean accept(File file) {
      if (file.isDirectory()) {
        return true;
      }
      String extension = FileTools.getExtension(file);
      if (extension != null) {
        if (extension.equalsIgnoreCase("pag")) {
          return true;
        }
        else {
          return false;
        }
      }
      return false;
    }

    // The description of this filter
    public String getDescription() {
      return "Pagel tree files (*.pag)";
    }
  };

}
