package bio.sequences;

/**
 * <p>Title: Bio</p>
 * <p>Description: Tools for bioinformatics</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author Julien Dutheil
 * @version 1.0
 */

/**
 * @brief Exception thrown when two alphabet don't match.
 *
 * Typically, this may occur when you try to add a bad sequence to a container,
 * or concatenate two kinds of sequences, and so on.</p>
 */
public class AlphabetMismatchException extends Exception {

  protected Alphabet alphabet1, alphabet2;

  /**
   * @brief Build a new AlphabetMismatchException object.
   *
   * @param text A message to be passed to the exception hierarchy.
   * @param alpha1 A const pointer toward the first alphabet.
   * @param alpha2 A const pointer toward the second alphabet, i.e. the one which does not match with the first.
   */
  public AlphabetMismatchException(String text, Alphabet alpha1,
                                   Alphabet alpha2) {
    super(text);
    this.alphabet1 = alpha1;
    this.alphabet2 = alpha2;
  }

  /**
   * @brief Get the alphabets that do not match.
   *
   * @return a vector of pointers toward the alphabets.
   */
  public Alphabet[] getAlphabets() {
    Alphabet[] a = new Alphabet[2];
    a[0] = alphabet1;
    a[1] = alphabet2;
    return a;
  }
}
