package bio.sequences.GUI;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import bio.sequences.*;

public class AlignmentEditor extends JSplitPane {

  private SequenceContainer sequences;

  // Graphic components:
  private SequenceEditorPane editor;
  private JScrollBar verticalScrollBar, horizontalScrollBar;
  private int seqLenMax;
  private JPanel rightPanel;
  private int windowHeight, windowWidth;

  public AlignmentEditor() {
    initGUI();
  }

  private void initGUI() {
    editor = new SequenceEditorPane();
    windowWidth = editor.getWindowSize().width;
    windowHeight = editor.getWindowSize().height;
    horizontalScrollBar = new JScrollBar(JScrollBar.HORIZONTAL);
    horizontalScrollBar.addAdjustmentListener(new AdjustmentListener() {
      public void adjustmentValueChanged(AdjustmentEvent ae) {
        editor.setSiteWindowPositions(ae.getValue(), ae.getValue() + windowWidth);
      }
    });
    verticalScrollBar = new JScrollBar(JScrollBar.VERTICAL);
    verticalScrollBar.addAdjustmentListener(new AdjustmentListener() {
      public void adjustmentValueChanged(AdjustmentEvent ae) {
        editor.setSequenceWindowPositions(ae.getValue(), ae.getValue() + windowHeight);
      }
    });
    editor.addComponentListener(new ComponentAdapter() {
      public void componentResized(ComponentEvent ce) {
        setWindowSize(
          Math.round(ce.getComponent().getWidth() / editor.getGridSize().width),
          Math.round(ce.getComponent().getHeight() / editor.getGridSize().height)
        );
      }
    });
    rightPanel = new JPanel(new BorderLayout());
    rightPanel.add(editor, BorderLayout.CENTER);
    rightPanel.add(horizontalScrollBar, BorderLayout.SOUTH);
    rightPanel.add(verticalScrollBar, BorderLayout.WEST);
    this.setLeftComponent(new JScrollPane(editor.getNamesPanel(), JScrollPane.VERTICAL_SCROLLBAR_NEVER, JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS));
    this.setRightComponent(rightPanel);
    this.setDividerLocation(75);
  }

  private void updateEditor() {
    editor.repaint();
    editor.getNamesPanel().repaint();
  }

  public void setSequences(SequenceContainer sequences) {
    this.sequences = sequences;
    editor.loadSequences(sequences);
    seqLenMax = SequenceContainerTools.getMaximumSequenceLength(sequences);
    horizontalScrollBar.setMinimum(0);
    horizontalScrollBar.setValue(0);
    horizontalScrollBar.setVisibleAmount(windowWidth);
    horizontalScrollBar.setMaximum(Math.max(seqLenMax, windowWidth));
    verticalScrollBar.setMinimum(0);
    verticalScrollBar.setValue(0);
    verticalScrollBar.setVisibleAmount(windowHeight);
    verticalScrollBar.setMaximum(Math.max(sequences.getNumberOfSequences(), windowHeight));
    updateEditor();
  }

  public void setWindowSize(int width, int height) {
    windowWidth = width;
    windowHeight = height;
    editor.setWindowSize(width, height);
    horizontalScrollBar.setVisibleAmount(windowWidth);
    horizontalScrollBar.setMaximum(Math.max(seqLenMax, windowWidth));
    verticalScrollBar.setVisibleAmount(windowHeight);
    verticalScrollBar.setMaximum(Math.max(sequences.getNumberOfSequences(), windowHeight));
  }

  public SequenceEditorPane getSequenceEditor() { return editor; }

}
