package bio.sequences;

/**
 * <p>Title: Bio</p>
 * <p>Description: Tools for bioinformatics</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author Julien Dutheil
 * @version 1.0
 */

public class SequenceContainerTools {

  /**
   * @brief Check if all sequences in a SequenceContainer have the same length.
   *
   * @param sequences The container to check.
   * @return True is all sequence have the same length.
   */
  public static boolean sequencesHaveTheSameLength(SequenceContainer sequences) {
    String[] seqNames = sequences.getSequencesNames();
    if (seqNames.length <= 1) {
      return true;
    }
    try {
      int length = sequences.getSequence(seqNames[0]).size();
      for (int i = 1; i < seqNames.length; i++) {
        if (sequences.getSequence(seqNames[i]).size() != length) {
          return false;
        }
      }
    } catch(SequenceNotFoundException snfe) {
      System.out.println("This should never happen!!!");
      snfe.printStackTrace();
    }
    return true;
  }

  public static int getMaximumSequenceLength(SequenceContainer sequences) {
    String[] seqNames = sequences.getSequencesNames();
    int max = 0;
    try {
      for (int i = 0; i < seqNames.length; i++) {
        int length = sequences.getSequence(seqNames[i]).size();
        if (length > max) max = length;
      }
    } catch(SequenceNotFoundException snfe) {
      System.out.println("This should never happen!!!");
      snfe.printStackTrace();
    }
    return max;
  }
}
