package bio.sequences.io;

/**
 * <p>Title: Bio</p>
 * <p>Description: Tools for bioinformatics</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author Julien Dutheil
 * @version 1.0
 */

import bio.sequences.*;
import java.io.*;

/**
 * @brief Low level implementation of the ISequence and OSequence interfaces.
 */
public abstract class AbstractIOSequence implements ISequence, OSequence {

  /**
   * @brief The maximum number of chars to be written on a line.
   */
  protected int _charsByLine;

  /**
   * @brief Build a new Fasta object.
   *
   * @param charsByLine Number of character per line when writing files.
   */
  public AbstractIOSequence(int charsByLine) {
    _charsByLine = charsByLine;
  }

  public AbstractIOSequence() {
    _charsByLine = 100;
  }

  /**
   * @name ISequence methods:
   *
   * @{
   */
  public VectorSequenceContainer read(File file, Alphabet alpha) throws Exception {
    VectorSequenceContainer vsc = new VectorSequenceContainer(alpha);
    read(file, vsc);
    return vsc;
  }

  public void read(File file, VectorSequenceContainer sc) throws Exception {
    throw new NoSuchMethodException("Method read() is not supported for this format.");
  }

  public void write(File file, SequenceContainer sc, boolean overwrite) throws Exception {
    throw new NoSuchMethodException("Method write() is not supported for this format.");
  }


  /** @} */
};
