/*
 * StatsTools package. You should have received this file along with another program.
 * See the program files for description and full license terms.
 * Copyright (C) 2003  Julien Dutheil
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package gnu.jtools.stats.array;

import java.util.*;

public class ArrayStatTools {

  public static int sum(int[] t) {//somme de tous les elements
    int result = 0;
    for(int i = 0; i < t.length; i++) {
      result += t[i];
    }
    return result;
  }
  public static double sum(double[] t) {//somme de tous les elements
    double result = 0;
    for(int i = 0; i < t.length; i++) {
      result += t[i];
    }
    return result;
  }
  public static int[] sumCum(int[] x) {
    int[] sc = new int[x.length];
    for(int i = 0; i < sc.length; i++) {
      sc[i] = x[i];
      if(i > 0) sc[i] += sc[i - 1];
    }
    return sc;
  }

  public static double[] sumCum(double[] x) {
    double[] sc = new double[x.length];
    for(int i = 0; i < sc.length; i++) {
      sc[i] = x[i];
      if(i > 0) sc[i] += sc[i - 1];
    }
    return sc;
  }

  public static int prod(int[] t) {//somme de tous les elements
    int result = 1;
    for(int i = 0; i < t.length; i++) {
      result *= t[i];
    }
    return result;
  }
  public static double prod(double[] t) {//somme de tous les elements
    double result = 1;
    for(int i = 0; i < t.length; i++) {
      result *= t[i];
    }
    return result;
  }
  public static int[] prodCum(int[] x) {
    int[] sc = new int[x.length];
    for(int i = 0; i < sc.length; i++) {
      sc[i] = x[i];
      if(i > 0) sc[i] *= sc[i - 1];
    }
    return sc;
  }

  public static double[] prodCum(double[] x) {
    double[] sc = new double[x.length];
    for(int i = 0; i < sc.length; i++) {
      sc[i] = x[i];
      if(i > 0) sc[i] *= sc[i - 1];
    }
    return sc;
  }


  public static double mean(int[] t) {//moyenne arithmetique
    double result = sum(t) / t.length;
    return result;
  }
  public static double mean(double[] t) {//moyenne arithmetique
    double result = sum(t) / t.length;
    return result;
  }

  public static int[] sqr(int[] t) {
    int[] result = new int[t.length];
    for(int i = 0; i < t.length; i++) {
      result[i] = (int)Math.pow(t[i],2);
    }
    return result;
  }
  public static double[] sqr(double[] t) {
    double[] result = new double[t.length];
    for(int i = 0; i < t.length; i++) {
      result[i] = Math.pow(t[i], 2);
    }
    return result;
  }

  public static double variance(int[] t) {//variance d'echantillon
    return mean(sqr(t)) - Math.pow(mean(t), 2);
  }
  public static double variance(double[] t) {//variance d'echantillon
    return mean(sqr(t)) - Math.pow(mean(t), 2);
  }

  public static int min(int[] t) {
    int result = Integer.MAX_VALUE;
    for(int i = 0; i < t.length; i++) {
      if(t[i] < result) result = t[i];
    }
    return result;
  }
  public static double min(double[] t) {
    double result = Double.POSITIVE_INFINITY;
    for(int i = 0; i < t.length; i++) {
      if(t[i] < result) result = t[i];
    }
    return result;
  }

  public static int max(int[] t) {
    int result = Integer.MIN_VALUE;
    for(int i = 0;i < t.length; i++) {
      if(t[i] > result) result = t[i];
    }
    return result;
  }
  public static double max(double[] t) {
    double result = Double.NEGATIVE_INFINITY;
    for(int i = 0; i < t.length; i++) {
      if(t[i] > result) result = t[i];
    }
    return result;
  }


/*  public static int[] decompose(int[] t, int nbFreq) {
    double mini = min(t);
    double maxi = max(t);
    double x;
    int indice;
    int[] result = new int[nbFreq];

    for(int i = 0; i < t.length; i++) {
      x = t[i];
      indice = (int)(((x-mini)/(maxi-mini))*(nbFreq-1));
      if(indice == nbFreq) result[indice-1]++;
      else result[indice]++;
    }
    return result;
  }*/
  public static double[][] decompose(double[] t, double[] breaks) {
    int indice;
    List[] decomposition = new List[breaks.length - 1];
    //Initialisation:
    for(int i = 0; i < decomposition.length; i++) decomposition[i] = new ArrayList();
    //Completing:
    for(int i = 0; i < t.length; i++) {
      indice = 0;
      while(indice < decomposition.length - 1 && breaks[indice] < t[i]) {
        //System.out.println(i+"\t"+t[i]+"\t"+breaks[indice]);
        indice ++;
      }
      decomposition[indice].add(new Double(t[i]));
    }
    //Convert to array:
    double[][] result = new double[breaks.length - 1][];
    for(int i = 0; i < result.length; i++) {
      result[i] = new double[decomposition[i].size()];
      for(int j = 0; j < result[i].length; j++) {
        result[i][j] = ((Double)decomposition[i].get(j)).doubleValue();
      }
    }
    //System.out.println("Before: " + t.length);
    //System.out.println("After : " + sum(getCounts(result)));
    return result;
  }

  public static int[] getCounts(double[][] decomposedArray) {
    int[] sizes = new int[decomposedArray.length];
    for(int i = 0; i < sizes.length; i++) {
      sizes[i] = decomposedArray[i].length;
    }
    return sizes;
  }

  public static double[] getBreaks(double[] t, int nbFreq) {
    double min = min(t);
    double max = max(t);
    double step = (max - min) / (double)nbFreq;
    double[] breaks = new double[nbFreq + 1];
    breaks[0] = min;
    for(int i = 1; i < breaks.length; i++) breaks[i] = breaks[i-1] + step;
    return breaks;
  }

  public static double[] getMids(double[] breaks) {
    double[] mids = new double[breaks.length - 1];
    for(int i = 0; i < mids.length; i++) mids[i] = (breaks[i] + breaks[i + 1])/2;
    return mids;
  }

  public static double[] getProbabilities(int[] counts) {
    int[] sc = sumCum(counts);
    return ArrayOperators.scalarMultiply(ArrayOperators.castFromArrayOfInt(sc), 1.0 / (double)sum(counts));
  }

  public static int whereIsMin(int[] array) {
    int result = 0;
    int m = min(array);
    for(int i = 0; i < array.length; i++) {
      if(array[i] == m) result=i;
    }
    return result;
  }
  public static int whereIsMin(double[] array) {
    int result = 0 ;
    double m = min(array);
    for(int i = 0; i < array.length; i++) {
      if(array[i] == m) result = i;
    }
    return result;
  }
  public static int whereIsMax(int[] array) {
    int result = 0;
    int m = max(array);
    for(int i = 0; i < array.length; i++) {
      if(array[i] == m) result = i;
    }
    return result;
  }
  public static int whereIsMax(double[] array) {
    int result = 0;
    double m = max(array);
    for(int i = 0; i < array.length; i++) {
      if(array[i] == m) result = i;
    }
    return result;
  }

  /****************************************************************************/

  public static int getNumberOfValuesGreaterOrEqualTo(double[] array, double limit) {
    int count = 0;
    for(int i = 0; i < array.length; i++)
      if(array[i] >= limit) count++;
    return count;
  }

  /****************************************************************************/

}
