package gnu.jtools.stats.table;

/**
 * <p>Title: Stats</p>
 * <p>Description: Statistics tools</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author Julien Dutheil
 * @version 1.0
 */

import java.util.*;

public class Table {

  public static class TableColumn extends ArrayList {

    public TableColumn(Class type, String name) {
      this.type = type;
      this.name = name;
    }

    public Class getType() { return type; }

    public String getName() { return name; }

    public void setName(String name) { this.name = name; }

    public void coerceColumnType(Class type) throws NumberFormatException {
      List newObjects = new ArrayList();
      for(int i = 0; i < size(); i++) {
        Object o;
        if(type.equals(Integer.class)) o = new Integer(get(i).toString());
        else if(type.equals(Double.class)) o = new Double(get(i).toString());
        else if(type.equals(Boolean.class)) o = new Boolean(get(i).toString());
        else if(type.equals(String.class)) o = new String(get(i).toString());
        else {
          System.err.println("Error, unsupported type. String used.");
          o = new String(get(i).toString());
        }
        newObjects.add(o);
      }
      // If this is raised, the cast worked!
      this.type = type;
      for(int i = 0; i < size(); i++) {
        set(i, newObjects.get(i));
      }
    }

    public boolean add(Object o) {
      if(o == null) return super.add(o);
      if(o.getClass().isAssignableFrom(type)) return super.add(o);
      else return false;
    }

    public void add(int index, Object o) {
      if(o.getClass().isAssignableFrom(type)) super.add(index, o);
      else throw new ClassCastException();
    }

    public void createCell() throws InstantiationException, IllegalAccessException {
      super.add(type.newInstance());
    }

    public void createCells(int n) throws InstantiationException, IllegalAccessException {
      for(int i = 0; i < n; i++) super.add(type.newInstance());
    }

    protected Class type;
    protected String name;
  }

  public Table(int numberOfColumns) {
    columns = new ArrayList(numberOfColumns);
    for(int j = 0; j < numberOfColumns; j++) columns.add(new TableColumn(Object.class, null));
    rowNames = null;
  }



  public Table(Class[] columnTypes) {
    int numberOfColumns = columnTypes.length;
    columns = new ArrayList(numberOfColumns);
    for(int j = 0; j < numberOfColumns; j++) columns.add(new TableColumn(columnTypes[j], null));
    rowNames = null;
  }

  // Work on columns:

  public int getNumberOfColumns() { return columns.size(); }

  public List getColumn(int j) {
    return new ArrayList((TableColumn)columns.get(j));
  }

  protected List getColumnPrivate(int j) {
    return (TableColumn)columns.get(j);
  }

  public List getColumn(String columnName) throws TableColumnNotFoundException {
    if(!hasColumnNames()) throw new TableColumnNotFoundException(this, columnName);
    for(int j = 0; j < getNumberOfColumns(); j++) {
      if(getColumnName(j).equals(columnName)) return getColumn(j);
    }
    throw new TableColumnNotFoundException(this, columnName);
  }

  protected List getColumnPrivate(String columnName) throws TableColumnNotFoundException {
    if(!hasColumnNames()) throw new TableColumnNotFoundException(this, columnName);
    for(int j = 0; j < getNumberOfColumns(); j++) {
      if(getColumnName(j).equals(columnName)) return getColumnPrivate(j);
    }
    throw new TableColumnNotFoundException(this, columnName);
  }

  public int getColumnIndex(String columnName) throws TableColumnNotFoundException {
    if(!hasColumnNames()) throw new TableColumnNotFoundException(this, columnName);
    for(int j = 0; j < getNumberOfColumns(); j++) {
      if(getColumnName(j).equals(columnName)) return j;
    }
    throw new TableColumnNotFoundException(this, columnName);
  }

  public boolean hasColumnNames() {
    for(int i = 0; i < getNumberOfColumns(); i++) if(((TableColumn)columns.get(i)).getName() == null) return false;
    return true;
    //return columnNames != null;
  }

  public String[] getColumnNames() {
    String[] columnNames = new String[getNumberOfColumns()];
    for(int i = 0; i < columnNames.length; i++) columnNames[i] = ((TableColumn)columns.get(i)).getName();
    return columnNames; //(String[])columnNames.toArray(new String[0]);
  }

  public String getColumnName(int j) { return ((TableColumn)columns.get(j)).getName(); }

  public Class getColumnClass(int j) { return ((TableColumn)columns.get(j)).getType(); }

  public void coerceColumnType(int columnIndex, Class type) throws NumberFormatException {
    ((TableColumn)columns.get(columnIndex)).coerceColumnType(type);
  }

  public void setColumnNames(String[] names) throws InvalidTableRowSizeException {
    if(names.length != getNumberOfColumns()) throw new InvalidTableRowSizeException(this, names.length);
    for(int i = 0; i < getNumberOfColumns(); i++) ((TableColumn)columns.get(i)).setName(names[i]);
    //columnNames = Arrays.asList(names);
  }

  public void setColumnName(int columnIndex, String name) {
    ((TableColumn)columns.get(columnIndex)).setName(name);
  }

  // Work on rows:

  public int getNumberOfRows() { return ((TableColumn)columns.get(0)).size(); }

  public List getRow(int i) {
    ArrayList al = new ArrayList(getNumberOfColumns());
    for(int j = 0; j < getNumberOfColumns(); j++) al.add(getCell(i, j));
    return al;
  }

  public List getRow(String rowName) throws TableRowNotFoundException {
    if(!hasRowNames()) throw new TableRowNotFoundException(this, rowName);
    for(int i = 0; i < getNumberOfRows(); i++) {
      if(rowNames.get(i).equals(rowName)) return getRow(i);
    }
    throw new TableRowNotFoundException(this, rowName);
  }

  public int getRowIndex(String rowName) throws TableRowNotFoundException {
    if(!hasRowNames()) throw new TableRowNotFoundException(this, rowName);
    for(int i = 0; i < getNumberOfRows(); i++) {
      if(rowNames.get(i).equals(rowName)) return i;
    }
    return -1;
  }


  public boolean hasRowNames() { return rowNames != null; }

  public String[] getRowNames() { return (String[])rowNames.toArray(new String[0]); }

  public String getRowName(int rowIndex) { return (String)rowNames.get(rowIndex); }

  public void setRowName(int rowIndex, String name) { rowNames.set(rowIndex, name); }

  public void setRowNames(String[] names) throws InvalidTableColumnSizeException {
    if(names.length != getNumberOfRows()) throw new InvalidTableColumnSizeException(this, names.length);
    rowNames = Arrays.asList(names);
  }

  public void addRow(List row) throws InvalidTableRowSizeException, ClassCastException {
    //System.out.println("addRow of size " + row.size() + " in a table with " + getNumberOfColumns() + " columns.");
    if(row.size() != getNumberOfColumns()) throw new InvalidTableRowSizeException(this, row.size());
    for(int j = 0; j < row.size(); j++)
     if(!((TableColumn)columns.get(j)).add(row.get(j)))
       throw new ClassCastException();
  }

  public void addRow(int i, List row) throws InvalidTableRowSizeException, ClassCastException {
    if(row.size() != getNumberOfColumns()) throw new InvalidTableRowSizeException(this, row.size());
    for(int j = 0; j < row.size(); j++)
     ((TableColumn)columns.get(j)).add(i, row.get(j));
  }

  public void createRow() throws InstantiationException, IllegalAccessException {
    for(int j = 0; j < getNumberOfColumns(); j++)
     ((TableColumn)columns.get(j)).createCell();
  }

  public void createRows(int n) throws InstantiationException, IllegalAccessException {
    for(int j = 0; j < getNumberOfColumns(); j++)
     ((TableColumn)columns.get(j)).createCells(n);
  }

  // Act on cells:

  public Object getCell(int i, int j) {
    return ((TableColumn)columns.get(j)).get(i);
  }

  public Object getCell(int i, String columnName) throws TableColumnNotFoundException {
    return getColumnPrivate(columnName).get(i);
  }

  public void setCell(int i, int j, Object value) throws ClassCastException {
    ((TableColumn)columns.get(j)).set(i, value);
  }

  public void setCell(int i, String columnName, Object value) throws ClassCastException, TableColumnNotFoundException {
    getColumnPrivate(columnName).set(i, value);
  }
  
  // Fields:

  protected List columns;
  protected List rowNames;
}
