package gnu.jtools.utils.attributes;

/**
 * <p>Titre : Attributes</p>
 * <p>Description : Basic classes to deal with attributes</p>
 * <p>Copyright : Copyright (c) 2002</p>
 * @author Julien Dutheil
 * @version 1.0
 */

import java.util.Map.Entry;
import java.lang.reflect.*;

public class Attribute implements Entry {

  public Attribute(String name, Object value) {
    this.name  = name;
    this.value = value;
    //System.out.println("using default values for " + name + " " + value.toString());
    this.isNameEditable = true;
    this.isValueEditable = true;
    this.type = Object.class;
  }

  public Attribute(String name, Object value, boolean isNameEditable, boolean isValueEditable) {
    this.name  = name;
    this.value = value;
    this.isNameEditable = isNameEditable;
    this.isValueEditable = isValueEditable;
    this.type = Object.class;
  }

  public Attribute(String name, Object value, boolean isNameEditable, boolean isValueEditable, Class type) {
    this.name  = name;
    this.value = value;
    this.isNameEditable = isNameEditable;
    this.isValueEditable = isValueEditable;
    this.type = type;
  }

  public Attribute(String name, Object value, Class type) {
    this.name  = name;
    this.value = value;
    this.isNameEditable = true;
    this.isValueEditable = true;
    this.type = type;
  }

  public Object clone() {
    return new Attribute(new String(name), value, isNameEditable, isValueEditable, type);
  }

  public Object deepClone() throws NoSuchMethodException, InstantiationException, IllegalAccessException, InvocationTargetException {
    Class[] p = new Class[1];
    p[0] = value.getClass();
    Object[] o = new Object[1];
    o[0] = value;
    return new Attribute(new String(name), value.getClass().getConstructor(p).newInstance(o), isNameEditable, isValueEditable, type);
  }

  public Object   getKey() { return name; }
  public String  getName() { return name; }
  public Class   getType() { return type; }
  public Object getValue() { return value; }

  public Object setValue(Object value) throws ClassCastException {
    if(!type.isAssignableFrom(value.getClass())) throw new ClassCastException();
    Object old = this.value;
    this.value = value;
    return old;
  }

  public Object setName(String name) {
    String old = this.name;
    this.name = name;
    return old;
  }

  public boolean equals(Object o) {
    if(o.getClass() != this.getClass()) return false;
    else return name.equals(((Attribute)o).getKey()) && value.equals(((Attribute)o).getValue());
  }

  public boolean isNameEditable() { return isNameEditable; }

  public boolean isValueEditable() { return isValueEditable; }

  public void setNameEditable(boolean b) { isNameEditable = b; }

  public void setValueEditable(boolean b) { isValueEditable = b; }

  private String name;
  private Object value;
  private boolean isNameEditable;
  private boolean isValueEditable;
  private Class type;
}
