package gnu.jtools.utils.attributes;

/**
 * <p>Titre : Attributes</p>
 * <p>Description : Basic classes to deal with attributes</p>
 * <p>Copyright : Copyright (c) 2002</p>
 * @author Julien Dutheil
 * @version 1.0
 */

import java.util.*;

public class AttributesMap extends AbstractMap {

  public AttributesMap() {
    attributes = new HashSet();
  }
  public AttributesMap(AttributesMap am) {
    attributes = new HashSet();
    Iterator i = am.entrySet().iterator();
    while(i.hasNext()) { this.attributes.add(((Attribute)i.next()).clone()); }
  }

  public Set entrySet() { return attributes; }

  public Attribute getAttribute(String name) {
    for(Iterator i = attributes.iterator(); i.hasNext();) {
      Attribute attribute = (Attribute)i.next();
      if(attribute.getName().equals(name)) return attribute;
    }
    return null;
  }

  /**
   * The put method add attributes to the set.
   * If an attribute with the same name already exists in the set,
   * it is exchanged with the new one passed to the method.
   */

  public Object put(Object key, Object value) throws IllegalArgumentException {
    if(key.getClass() != String.class) throw new IllegalArgumentException();
    Attribute old = this.getAttribute((String)key);
    if(old != null) attributes.remove(old);
    attributes.add(new Attribute((String)key, value));
    return old;
  }

  public Object put(String name, Object value) throws IllegalArgumentException {
    Attribute old = this.getAttribute(name);
    if(old != null) attributes.remove(old);
    attributes.add(new Attribute(name, value));
    return old;
  }

  public Object put(String name, Object value, boolean isNameEditable, boolean isValueEditable) throws IllegalArgumentException {
    Attribute old = this.getAttribute(name);
    if(old != null) attributes.remove(old);
    attributes.add(new Attribute(name, value, isNameEditable, isValueEditable));
    return old;
  }

  public Attribute put(Attribute att) {
    Attribute old = this.getAttribute(att.getName());
    if(old != null) attributes.remove(old);
    attributes.add(att);
    return old;
  }

/*  public void set(String name, Object value) {
    getAttribute(name).setValue(value);
  }

  public void set(String name, Object value, boolean isNameEditable, boolean isValueEditable) {
    Attribute a = getAttribute(name);
              a.setValue(value);
              a.setNameEditable(isNameEditable);
              a.setValueEditable(isValueEditable);
  }

  public void set(String name, boolean isNameEditable, boolean isValueEditable) {
    Attribute a = getAttribute(name);
              a.setNameEditable(isNameEditable);
              a.setValueEditable(isValueEditable);
  }*/

  /**
   * Remove:
   */

  public Attribute remove(Attribute attribute) {
    for(Iterator i = this.entrySet().iterator(); i.hasNext();) {
      Attribute result = (Attribute)i.next();
      if(result.getName().equals(attribute.getName())) {
        attributes.remove(result);
        return result;
      }
    }
    return null;
  }

  public Attribute remove(String name) {
    for(Iterator i = this.entrySet().iterator(); i.hasNext();) {
      Attribute result = (Attribute)i.next();
      if(result.getName().equals(name)) {
        attributes.remove(result);
        return result;
      }
    }
    return null;
  }

  public String[] getNames() {
    List keys = new ArrayList();
    Iterator i = attributes.iterator();
    while(i.hasNext()) { keys.add(((Attribute)i.next()).getName()); }
    return (String[])keys.toArray(new String[0]);
  }

  protected Set attributes;
}
