/*
 * CommandLine package. You should have received this file along with another program.
 * See the program files for description and full license terms.
 * Copyright (C) 2003  Julien Dutheil
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package gnu.jtools.utils.commandline;

/**
 * <p>Title: Command Line</p>
 * <p>Description: Tools to deal with command lines</p>
 * <p>Copyright: Copyright (c) Julien Dutheil</p>
 * <p>Company: Myself</p>
 * @author Julien Dutheil
 * @version 1.0
 */

import java.io.*;
import java.util.*;

public class ArgumentsTools {

  /**
   * Deals with arguments.
   * Arguments are of the type 'name=value'.
   * Further arguments may be separated by a semicolon or an end of line.
   */

  public static ArgumentsList parseArguments(String description) {
    ArgumentsList list = new ArgumentsList();
    int end = description.indexOf('#');
    String txt = (end > -1) ? description.substring(0, end) : description;
    StringTokenizer st = new StringTokenizer(txt, ";\n");
    while(st.hasMoreTokens()) list.add(new Argument(st.nextToken()));
    return list;
  }

  /**
   * The following method may be useful to analyse command line
   * parameters passed through the main method:
   */
  public static ArgumentsList parseArguments(String[] description) {
    ArgumentsList list = new ArgumentsList();
    for(int i = 0; i < description.length; i++) list.add(new Argument(description[i]));
    return list;
  }

  public static ArgumentsList parseArguments(BufferedReader br) throws IOException {
    ArgumentsList list = new ArgumentsList();
    String currentLine = br.readLine();
    while(currentLine != null) {
      list.addAll(parseArguments(currentLine));
      currentLine = br.readLine();
    }
    return list;
  }

  public static void writeArgumentsToFile(File outFile, ArgumentsList argsList, String delimiter) throws IOException {
    PrintWriter pw = new PrintWriter(new FileWriter(outFile));
    for(Iterator i = argsList.iterator(); i.hasNext();) {
      Argument arg = (Argument)i.next();
      pw.println(arg.getName() + delimiter + arg.getValue());
    }
    pw.close();
  }
}