/*
 * Format package. You should have received this file along with another program.
 * See the program files for description and full license terms.
 * Copyright (C) 2003  Julien Dutheil
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package gnu.jtools.utils.file;

import gnu.jtools.utils.string.TextTools;
import java.io.*;
import java.util.zip.*;
import java.util.*;

public class FileTools {

  public static String getFileNameWithoutExtension(File fichier) {
    StringTokenizer analyse = new StringTokenizer(fichier.getName(),".");
    return analyse.nextToken();
  }

  public static String getExtension(File file) {
    String result = "";
    String name = file.getName();
    int i = name.lastIndexOf('.');
    if(i > 0 && i < name.length() - 1) result = name.substring(i + 1).toLowerCase();
    //System.out.println(result);
    return result;
  }

  public static boolean isExtensionValid(String extension) {
    String charGenants = ".,;:/\\\n\t^[]()";
    boolean result = true;
    for(int i = 0; i < charGenants.length(); i++) {
      if(extension.indexOf(charGenants.charAt(i)) > -1) {
        result = false;
        break;
      }
    }
    if(extension == "") result = false;
    return result;
  }

  public static boolean isFileNameValid(String filename) {
    String charGenants = ",;/\\\n\t^";
    boolean result = true;
    for(int i = 0; i < charGenants.length(); i++) {
      if(filename.indexOf(charGenants.charAt(i)) > -1) {
        result = false;
        break;
      }
    }
    if(filename == "") result = false;
    return result;
  }

  public static void write(File outPut, String[] listOfLines) throws IOException {
    PrintWriter sortie = new PrintWriter(new FileWriter(outPut)); //le fichier ou on ecrit
    for(int i=0; i<listOfLines.length; i++) sortie.println(listOfLines[i]);
    sortie.close();
  }

  public static String[] readAll(File inFile) throws IOException {
    List result = new ArrayList();
    BufferedReader input = new BufferedReader(new FileReader(inFile));
    String lineRead = input.readLine();
    while(lineRead != null) {
      result.add(lineRead);
      lineRead = input.readLine();
    }
    input.close();
    return (String[])result.toArray(new String[0]);
  }

  public static void copyTo(File input, File output) throws IOException {
    FileInputStream reader = new FileInputStream(input);
    FileOutputStream writer = new FileOutputStream(output);
    int n = reader.available();
    for(int i = 0; i < n; i++) writer.write(reader.read());
    reader.close();
    writer.close();
  }
  
  public static void copyTo(InputStream reader, File output) throws IOException {
    FileOutputStream writer = new FileOutputStream(output);
    int n = reader.available();
    for(int i = 0; i < n; i++) writer.write(reader.read());
  }
  
  public static void unzipTo(ZipInputStream reader, File output) throws IOException {
    ZipEntry entry = reader.getNextEntry();
    while(entry != null) {
      System.out.println(entry.getName());
      if(entry.isDirectory()) new File(output, entry.getName()).mkdir();
      else {
        FileOutputStream writer = new FileOutputStream(new File(output, entry.getName()));
        int b = reader.read();
        while(b != -1) {
          writer.write(b);
          b = reader.read();
        }
      }
      entry = reader.getNextEntry();
    }
  }
  
  public static void copyDirTo(File input, File output) throws IOException {
      if(input.isDirectory()) {
          if(!output.exists()) output.mkdir();
          String [] childrens = input.list();
          for(int i = 0; i < childrens.length; i++) {
              copyDirTo(new File(input, childrens[i]), new File(output, childrens[i]));
          }
      } else {
          copyTo(input, output);
      }
  }

  public static PrintWriter getExistingFile(File outFile) throws IOException {
    PrintWriter output = new PrintWriter(new FileWriter(outFile));
    String[] lines = readAll(outFile);
    for(int i = 0; i < lines.length; i++) output.println(lines[i]);
    return output;
  }

  public static String correctFilename(String filename) {
    StringBuffer bf=new StringBuffer(filename);
    for(int i=0; i<bf.length(); i++) {
      int indice=FORBIDDEN_CHARS_IN_FILENAME.indexOf(bf.charAt(i));
      if(indice>-1) bf.deleteCharAt(indice);
    }
    return bf.toString();
  }

  public static boolean isFinalDirectory(File dir) {
    if(!dir.isDirectory()) return false;
    else {
      File[] children = dir.listFiles();
      for(int i = 0; i < children.length; i++) if(children[i].isDirectory()) return false;
      return true;
    }
  }

  public static String winToNix(String path) { return TextTools.replace(path, "\\", "/"); }

  public static final String FORBIDDEN_CHARS_IN_FILENAME="*!?./\\";

  public static List<String> putFileIntoStringList(File file) throws IOException {
    BufferedReader br = new BufferedReader(new FileReader(file));
    List<String> l = new ArrayList<String>();
    String line = br.readLine();
    while (line != null) {
      l.add(line);
      line = br.readLine();
    }
    return l;
  }

  public static String getNextLine(BufferedReader br) throws IOException {
    String temp = br.readLine();
    while (temp != null && TextTools.isVoid(temp)) {
      temp = br.readLine();
    }
    return temp;
  }

}
