/*
 * DirectoryChooser package. You should have received this file along with another program.
 * See the program files for description and full license terms.
 * Copyright (C) 2003  Julien Dutheil
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**
 * Titre : DirectoryChooser<p>
 * @author Julien DUTHEIL
 */
package gnu.jtools.utils.gui;

import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

public class DirectoryChooser extends JDialog implements TreeSelectionListener, TreeWillExpandListener, ActionListener {

  public DirectoryChooser(File initialPath) {
    //System.out.println("Initial path: " + initialPath.getAbsolutePath());
    setModal(true);
    node = new DefaultMutableTreeNode();
    //Ajout des racines disponibles:
    File root;
    for(int i = 0; i < File.listRoots().length; i++) {
      root = File.listRoots()[i];
      try {node.add(createPseudoLeaf(root));}//If this root exists
      catch(NullPointerException e) { System.out.println("Drive " + root.getAbsolutePath() + " is not ready."); }//Lecteur non fonctionnel.
    }
    if(initialPath != null) { setCurrentDirectory(initialPath); }

    //D�finition des composants graphiques:

      //La zone texte affichant le chemin s�lectionn�:
    displayPath=new JTextField(30);
      displayPath.setEditable(false);
    getContentPane().add(displayPath, BorderLayout.NORTH);//En haut.

      //L'arbre des fichiers inclu dans un ascensceur:
    displayTree=new JTree(node);
      displayTree.addTreeSelectionListener(this);
      displayTree.addTreeWillExpandListener(this);
      displayTree.setRootVisible(false);
      displayTree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
        DefaultTreeCellRenderer dtcr=new DefaultTreeCellRenderer();
          dtcr.setLeafIcon(dtcr.getDefaultClosedIcon());
          dtcr.setOpenIcon(dtcr.getDefaultClosedIcon());
      displayTree.setCellRenderer(dtcr);
    treeLift=new JScrollPane(displayTree);
    getContentPane().add(treeLift, BorderLayout.CENTER);//Au centre.
    if(initialPath!=null) {
      expandCurrentDirectory(initialPath);
    }

    accessory = null;

      //La barre de controle:
    Box hbox=Box.createHorizontalBox();
        ok=new JButton("Ok");
        ok.addActionListener(this);
        ok.setMnemonic('O');
      hbox.add(ok);
        cancel=new JButton("Cancel");
        cancel.addActionListener(this);
        cancel.setMnemonic('C');
      hbox.add(cancel);
    getContentPane().add(hbox, BorderLayout.SOUTH);//En bas.

    //Dimension fen�tre:
    pack();
    Toolkit tk=Toolkit.getDefaultToolkit();
    setBounds((tk.getScreenSize().width-getWidth())/2, (tk.getScreenSize().height-getHeight())/2, getWidth(), getHeight());
  }

  public static int getNumberOfChildrenDirectories(File path) {
    try { return path.listFiles(df).length; }
    catch(NullPointerException e) {
      System.out.println("Can't access this directory: " + path.getAbsolutePath());
      return 0;
    }
  }

  protected DefaultMutableTreeNode createPseudoLeaf(File directory) {
    DefaultMutableTreeNode dirNode;
    getNumberOfChildrenDirectories(directory); //May throw an exception if directory is not ready!
    dirNode = new DefaultMutableTreeNode(new FileName(directory, directory.getName().equals("") ? directory.getAbsolutePath() : directory.getName()), true);
    dirNode.add(new DefaultMutableTreeNode());//Contient un objet fictif pour �tre consid�r� tel un r�pertoire.
    return dirNode;
  }
  protected DefaultMutableTreeNode createNode(File directory) {
    DefaultMutableTreeNode dirNode;
    dirNode = new DefaultMutableTreeNode(new FileName(directory, directory.getName().equals("") ? directory.getAbsolutePath().toLowerCase() : directory.getName()), true);
    return dirNode;
  }

  protected DefaultMutableTreeNode createBranch(File directory) {
    DefaultMutableTreeNode node = createPseudoLeaf(directory);
    try {
      do {
        DefaultMutableTreeNode parentNode = createNode(directory.getParentFile());
        parentNode.add(node);
        node = parentNode;
        directory = directory.getParentFile();
      } while(true);//Stops when an exception occures.
    } catch(NullPointerException ex) {} //Stops here.
    return node;
  }

/*  private DefaultMutableTreeNode createNode(File directory) {
    DefaultMutableTreeNode dirNode;
    int n=getNumberOfChildrenDirectories(directory);
    dirNode=new DefaultMutableTreeNode(directory, true);
    for(int i=0; i<n; i++) {
      dirNode.add(createNode(directory.listFiles(df)[i]));
      }
    return dirNode;
    }

  public void actualize() {
    node.removeAllChildren();
    File root;
    for(int i=0; i<File.listRoots().length; i++) {
      root=File.listRoots()[i];
      try {node.add(createNode(root));}//Si cette racine existe (lecteur op�rationnel).
      catch(NullPointerException e) { System.out.println("Drive "+root.getAbsolutePath()+" is not ready.");}//Lecteur non fonctionnel.
      }
    }
*///Trop long si beaucoup de fichiers!

  public void showSaveDialog(JDialog owner) {
    this.setTitle("Save in a directory...");
    setVisible(true);
  }

  public File getSelectedDirectory() { return currentSelectedDirectory; }

  public void setCurrentDirectory(File currentDirectory) throws NullPointerException {
    currentSelectedDirectory = currentDirectory;
    try { killNodeUpTo(displayTree.getSelectionPath()); }
    catch(NullPointerException ex) {} //No node selected yet.
    DefaultMutableTreeNode newBranch = createBranch(currentDirectory);
    node.add(newBranch);
    for(int i=0; i<node.getChildCount()-1; i++) {
      if(((FileName)((DefaultMutableTreeNode)node.getChildAt(i)).getUserObject()).getFile()
              .equals(
         ((FileName)newBranch.getUserObject()).getFile() )) node.remove(i);
    }
  }
  protected void expandCurrentDirectory(File currentDirectory) {
    TreePath path = getTreePath(node, currentDirectory);
    if(path.equals(null)) throw new NullPointerException();
    else { displayTree.expandPath(path); }
  }

  protected TreePath getTreePath(DefaultMutableTreeNode node, File directory) {
    TreePath result = null;
    File file;
    try { file = ((FileName)node.getUserObject()).getFile();}
    catch(NullPointerException ex) { file = new File(File.separator); }

    if(file.equals(directory)) { result = new TreePath(node.getPath()); }
    else if(!node.getChildAt(0).isLeaf()) {
      for(int i=0; i<node.getChildCount(); i++)
        result = getTreePath((DefaultMutableTreeNode)node.getChildAt(i), directory);
    }
    return result;
  }

  public void killNodeDownTo(TreePath path) {
    DefaultMutableTreeNode node = getNodeAt(path);
    node.removeAllChildren();//Remise � z�ro.
    for(int i=0; i<node.getChildCount(); i++) { killNodeDownTo(path.pathByAddingChild(node.getChildAt(i))); }//maybe useless as a matter of facts:-)
    node.add(new DefaultMutableTreeNode());//Contient un objet fictif pour �tre consid�r� tel un r�pertoire.
  }
  public void killNodeUpTo(TreePath path) {
    DefaultMutableTreeNode node = getNodeAt(path);
    node.removeAllChildren();//Remise � z�ro.
    node.add(new DefaultMutableTreeNode());//Contient un objet fictif pour �tre consid�r� tel un r�pertoire.
    try { killNodeUpTo(path.getParentPath()); }
    catch(NullPointerException ex) {} //Stop here.
  }

  public DefaultMutableTreeNode getNodeAt(TreePath path) {
    //Sauvegarde de la s�lection actuelle:
    TreePath selectionPath=displayTree.getSelectionPath();
    //On s�lectionne le noeud d'int�ret:
    displayTree.setSelectionPath(path);
    //On r�cup�re le noeud:
    DefaultMutableTreeNode node=(DefaultMutableTreeNode)(displayTree.getLastSelectedPathComponent());
    //On r�tablit l'ancienne s�lection:
    displayTree.setSelectionPath(selectionPath);
    //On renvoit le r�sultat:
    return node;
    //NB: je n'ai pas trouv� de m�thode pr�d�finie permettant d'arriver � ce r�sultat :-(
  }

  public void setAccessory(JComponent accessoryComponent) {
    accessory = accessoryComponent;
    this.getContentPane().add(accessory, BorderLayout.EAST);
  }

  //Interface TreeSelectionListener:
  public void valueChanged(TreeSelectionEvent e) {
//    if(displayTree.getLastSelectedPathComponent()==null) return;
    DefaultMutableTreeNode node=(DefaultMutableTreeNode)(displayTree.getLastSelectedPathComponent());

    if(node == null) return;
    else {
      currentDisplayedDirectory = ((FileName)(node.getUserObject())).getFile();
      displayPath.setText(currentDisplayedDirectory.getAbsolutePath());//Affichage du nouveau path.
    }
  }

  //Interface TreeWillExpandListener:
  public void treeWillExpand(TreeExpansionEvent e) throws ExpandVetoException {
    DefaultMutableTreeNode node = (DefaultMutableTreeNode)e.getPath().getLastPathComponent();
    File[] children = ((FileName)(node.getUserObject())).getFile().listFiles(df);
    if(children.length == 0) throw new ExpandVetoException(e);
    else {
      node.removeAllChildren();
      for(int i=0; i<children.length; i++) { node.add(createPseudoLeaf(children[i])); }
    }
  }

  public void treeWillCollapse(TreeExpansionEvent e) { killNodeDownTo(e.getPath()); }

  //Interface ActionListener:
  public void actionPerformed(ActionEvent e) {
    Object source=e.getSource();
    if(source==cancel) setVisible(false);
    if(source==ok) {
      currentSelectedDirectory=currentDisplayedDirectory;
      setVisible(false);
    }
  }

  private JTree displayTree;
  private File currentDisplayedDirectory;//Le r�pertoire en cours de s�lection
  private File currentSelectedDirectory;//Le dernier r�pertoire valid�.
  private DefaultMutableTreeNode node;

  //Composants graphiques:
  private JScrollPane treeLift;
  private JTextField displayPath;
  private JComponent accessory;
  private JButton ok, cancel;

  public static final DirectoryFilter df = new DirectoryFilter();
  public static final int OK_OPTION = 1;
  public static final int CANCEL_OPTION = 0;
}

class FileName {

  public FileName(File file, String fileName) {
    this.file=file;
    this.fileName=fileName;
  }

  public String toString() { return fileName; }
  public File getFile()    { return file;     }

  private File file;
  private String fileName;
}