
/**
 * Titre : Graphic Window<p>
 * Description : Package contenant des classes de graphisme
 * (Definition d'un repere, trace de points, de droites, de fonctions...).<p>
 * Copyright : Copyright (c) DUTHEIL Julien<p>
 * @author DUTHEIL Julien
 * @version 1.0
 */
package gnu.jtools.utils.gui;

public class Range implements Cloneable {

/**
 *
 *  Margins parameters:
 *
 *     |--------------------------|
 *     |       |( 1)              |        (1) insetLength
 *     |    |----------------|    |        (2) offsetLength
 *     |    |                |    |        (3) insetWidth
 *     | (3)|                |(4) |        (4) offsetWidth
 *     |----|                |----|
 *     |    |     GRAPH      |    |
 *     |    |                |    |
 *     |    |                |    |
 *     |    |________________|    |
 *     |       |(2)               |
 *     |__________________________|
 *
 */

  public Range(double xmin, double xmax, double ymin, double ymax) {
    this.xmin = xmin;
    this.xmax = xmax;
    this.ymin = ymin;
    this.ymax = ymax;
    insetWidth = 0;
    insetLength = 0;
    offsetWidth = 0;
    offsetLength = 0;
  }

  public Object clone() {
    return new Range(xmin, xmax, ymin, ymax);
  }

  public void setWidth(int width) { this.width = width; }
  public void setLength(int length) { this.length = length; }
  public void setHeight(int height) { this.length = height; }

  public void setInsetWidth(int insetWidth) { this.insetWidth = insetWidth; }
  public void setInsetLength(int insetLength) { this.insetLength = insetLength; }
  public void setInsetHeight(int insetHeight) { this.insetLength = insetHeight; }

  public void setOffsetWidth(int offsetWidth) { this.offsetWidth = offsetWidth; }
  public void setOffsetLength(int offsetLength) { this.offsetLength = offsetLength; }
  public void setOffsetHeight(int offsetHeight) { this.offsetLength = offsetHeight; }

  public void setWLSet(int insetWidth, int offsetWidth, int insetLength, int offsetLength) {
    this.insetWidth = insetWidth;
    this.offsetWidth = offsetWidth;
    this.insetLength = insetLength;
    this.offsetLength = offsetLength;
  }

  public void setWHSet(int insetWidth, int offsetWidth, int insetHeight, int offsetHeight) {
    this.insetWidth = insetWidth;
    this.offsetWidth = offsetWidth;
    this.insetLength = insetHeight;
    this.offsetLength = offsetHeight;
  }

  public double echx() { return (width - insetWidth - offsetWidth) / (xmax - xmin); }

  public double echy() { return (length - insetLength - offsetLength) / (ymax - ymin); }

  public int x(double abscisse) {
    return (int)((abscisse - xmin) * echx()) + insetWidth;
  }

  public int y(double ordonnee) {
    return length - offsetLength - (int)((ordonnee - ymin) * echy());
  }

  public double xinv(int xpix) {
    return ((xpix - insetWidth) / echx() + xmin);
  }

  public double yinv(int ypix) {
    return ((length - offsetLength - ypix) / echy() + ymin);
  }

  private double xmin, xmax, ymin, ymax;
  private int insetWidth, insetLength, offsetWidth, offsetLength;
  private int width, length;
}