
/**
 * Titre : Graphic Window<p>
 * Description : Package contenant des classes de graphisme
 * (Definition d'un repere, trace de points, de droites, de fonctions...).<p>
 * Copyright : Copyright (c) DUTHEIL Julien<p>
 * @author DUTHEIL Julien
 * @version 1.0
 */
package gnu.jtools.utils.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;

public class ZoomDialog extends JDialog implements ActionListener{

  public ZoomDialog(JFrame i) {
    super(i, true);
    setBounds(i.getX() + (i.getWidth()  - 250) / 2,//centree en largeur
              i.getY() + (i.getHeight() - 150) / 2,//centree en hauteur
              250,
              150);
    ZC = new ArrayList();
    message = ResourceBundle.getBundle("gnu/jtools/utils/gui/bundle", i.getLocale());
    setLocale(i.getLocale());
    setTitle(message.getString("zoomDialogTitle"));

    setaw = new JTextField(5);
    setbw = new JTextField(5);
    setah = new JTextField(5);
    setbh = new JTextField(5);
    ok             =new JButton  (message.getString("ok"));
      ok.addActionListener(this);
    cancel         =new JButton  (message.getString("cancel"));
      cancel.addActionListener(this);
    loadDialogContent=super.getContentPane();
    g=new GridBagLayout();
    loadDialogContent.setLayout(g);
    c=new GridBagConstraints();

      c.gridx=0;      c.gridy=0;
      c.gridwidth=4;  c.gridheight=1;
      c.weightx=0;    c.weighty=10;
    loadDialogContent.add(new JLabel(message.getString("zoomDialogExplain")),c);
      c.gridx=0;      c.gridy=1;
      c.gridwidth=1;  c.gridheight=1;
      c.weightx=0;    c.weighty=0;
    loadDialogContent.add(new JLabel("a:"),c);
      c.gridx=1;      c.gridy=1;
      c.gridwidth=1;  c.gridheight=1;
      c.weightx=0;   c.weighty=0;
    loadDialogContent.add(setaw,c);
      c.gridx=2;      c.gridy=1;
      c.gridwidth=1;  c.gridheight=1;
      c.weightx=0;    c.weighty=0;
    loadDialogContent.add(new JLabel("* "+message.getString("zoomDialogCurrentWidth")+" + b:"),c);
      c.gridx=3;      c.gridy=1;
      c.gridwidth=1;  c.gridheight=1;
      c.weightx=0;    c.weighty=10;
    loadDialogContent.add(setbw,c);
      c.gridx=0;      c.gridy=2;
      c.gridwidth=1;  c.gridheight=1;
      c.weightx=10;    c.weighty=0;
    loadDialogContent.add(new JLabel("a:"),c);
      c.gridx=1;      c.gridy=2;
      c.gridwidth=1;  c.gridheight=1;
      c.weightx=50;   c.weighty=0;
    loadDialogContent.add(setah,c);
      c.gridx=2;      c.gridy=2;
      c.gridwidth=1;  c.gridheight=1;
      c.weightx=20;    c.weighty=0;
    loadDialogContent.add(new JLabel("* "+message.getString("zoomDialogCurrentHeight")+" + b:"),c);
      c.gridx=3;      c.gridy=2;
      c.gridwidth=1;  c.gridheight=1;
      c.weightx=50;    c.weighty=10;
    loadDialogContent.add(setbh,c);
      c.gridx=0;      c.gridy=3;
      c.gridwidth=2;  c.gridheight=1;
      c.weightx=0;   c.weighty=0;
    loadDialogContent.add(ok,c);
      c.gridx=2;      c.gridy=3;
      c.gridwidth=2;  c.gridheight=1;
      c.weightx=0;   c.weighty=10;
    loadDialogContent.add(cancel,c);
    pack();
  }

  public void addZoomControls(ZoomControls zoomControls) {
    ZC.add(zoomControls);
    if(ZC.size() == 1) {//Init textfields
      aw = zoomControls.getCoefs()[0];
      ah = zoomControls.getCoefs()[1];
      bw = zoomControls.getCtes()[0];
      bh = zoomControls.getCtes()[1];
      setaw.setText("" + aw);
      setah.setText("" + ah);
      setbw.setText("" + bw);
      setbh.setText("" + bh);
    }
  }

  public void removeZoomControls(ZoomControls zoomControls) {
    ZC.remove(zoomControls);
  }
  public void removeZoomControls(int index) {
    ZC.remove(index);
  }

  public int getZoomControlsCount() { return ZC.size(); }

  public void setVisible(boolean yn) {
    Rectangle r = this.getOwner().getBounds();
    int x = r.x + (int)((r.width - this.getSize().width)/2);
    int y = r.y + (int)((r.height - this.getSize().height)/2);
    super.setLocation(x, y);
    super.setVisible(yn);
  }

  public void actionPerformed(ActionEvent ev) {
    Object source = ev.getSource();
    if(source == ok) {
      try {
        aw = Double.parseDouble(setaw.getText());
        bw = Integer.parseInt(setbw.getText());
        ah = Double.parseDouble(setah.getText());
        bh = Integer.parseInt(setbh.getText());
        if(aw < 1 || ah < 1) error(message.getString("badNumber"));
        else if(bw < 0 || bh < 0) error(message.getString("badNumber"));
        else {
          for(int i = 0; i < ZC.size(); i++) {
            ((ZoomControls)ZC.get(i)).setCoefs(aw, ah);
            ((ZoomControls)ZC.get(i)).setCtes(bw, bh);
          }
        }
        setVisible(false);
      } catch(NumberFormatException e) {
        error(message.getString("badNumber"));
      }
    }
    if(source == cancel) {
      //on annule.
      setVisible(false);
    }
  }

  public double[] getCurrentCoefs() {
    double[] coefs = new double[2];
             coefs[0] = aw;
             coefs[1] = ah;
    return coefs;
  }

  public int[] getCurrentCtes() {
    int[] ctes = new int[2];
             ctes[0] = bw;
             ctes[1] = bh;
    return ctes;
  }

  public void actualize() {
    setaw.setText("" + aw);
    setbw.setText("" + bw);
    setah.setText("" + ah);
    setbh.setText("" + bh);
  }

  public void error(String text) {//affichage d'un messsage d'erreur.
    JOptionPane.showMessageDialog(this, text, message.getString("failure"), JOptionPane.ERROR_MESSAGE);
  }

  private JButton ok, cancel;
  private JTextField setaw, setbw, setah, setbh;
  private double aw, ah;
  private int bw, bh;
  private ArrayList ZC;
  private Container loadDialogContent;
  private GridBagLayout g;
  private GridBagConstraints c;
  private ResourceBundle message;
}
