package gnu.jtools.utils.storage;

import java.util.*;

public class SortingTools {

  public static void swap(Sortable s, int i, int j) {
    //System.out.println("Swap " + s.getValue(i) + " and " + s.getValue(j));
    Object swap = s.get(i);
    s.set(i, s.get(j));
    s.set(j, swap);
  }

/**
 * Adapted from "Numerical Recipes in C", Chapter 8.
 */

  public static void straightInsertion(Sortable s) {
    straightInsertion(s, 0);
  }
  public static void straightInsertion(Sortable s, int begin) {
    straightInsertion(s, begin, s.size());
  }
  public static void straightInsertion(Sortable s, int begin, int end) {
    Object tmp;
    int i;
    for(int j = begin + 1; j< end; j++) {//Pick out each element in turn.
      tmp = s.get(j);                 //The j element.
      s.remove(j);                    //Remove j.
      i = j - 1;
      while( i>0 && s.getComparator().compare(s.get(i), tmp) > 0) i++; //Look for the place to insert j.
      s.add(i + 1, tmp);                //Insert j.
    }
  }

  public static void shellsMethod(Sortable s) {
    shellsMethod(s, 0);
  }
  public static void shellsMethod(Sortable s, int begin) {
    shellsMethod(s, begin, s.size());
  }
  public static void shellsMethod(Sortable s, int begin, int end) {
    int i, j, inc;
    Object tmp;
    inc = begin + 1; //Determine the starting increment.
    do {
      inc *= 3;
      inc++;
    } while (inc < end);
    do { //Loop over the partial sorts.
      inc /= 3;
      for(i = inc + 1; i <= end; i++) { //Outer loop of straight insertion.
        tmp = s.get(i - 1);//The first element is 0 and not 1.
        j = i ;
        while(s.getComparator().compare(s.get(j - inc -1), tmp) > 0) { //Inner loop of straight insertion.
          j -= inc;
          if(j <= inc) break;
        }
        s.remove(i - 1); //remove the i element.
        s.add(j - 1, tmp); //Add the i element.
      }
    } while(inc > 1);
  }

//NB: This method is not already finished (need more debugging!)

  public static void quicksort(Sortable s) {
    quicksort(s, 0);
  }
  public static void quicksort(Sortable s, int begin) {
    quicksort(s, begin, s.size());
  }
  public static void quicksort(Sortable s, int begin, int end) {
    quicksort(s, begin, end, 7); //7 gives quite good results.
  }
  public static void quicksort(Sortable s, int begin, int end, int M) {
    int l = begin, k, j, i; //Lower limit included.
    int ir = end - 1; //Upper limit excluded. 'ir' is the last included element.
    Object a, tmp;
    int NSTACK = 50; //May be modified...
    int[] istack = new int[NSTACK];
    int   jstack = -1;
    for(;;) {
      if(ir - l < M) {//Insertion sort when subarray small enough.
System.out.println("SHELL");
        shellsMethod(s, l, ir + 1);
        if(jstack < 0) break;
        //Pop stack and begin a new round of partitioning:
        ir = istack[jstack--];
        l  = istack[jstack--];
      } else {
System.out.println("QUICK");
        //Choose median or left, center and right elements as partitioning element a.
        //Also rearrange so that a[l] <= a[l+1] <= a[ir].
        k = (l + ir) >> 1;
        swap(s, k, l + 1);
        if(s.getComparator().compare(s.get(l), s.get(ir)) > 0) swap(s, l, ir);
        if(s.getComparator().compare(s.get(l + 1), s.get(ir)) > 0) swap(s, l + 1, ir);
        if(s.getComparator().compare(s.get(l), s.get(l + 1)) > 0) swap(s, l, l + 1);
        i = l + 1; //Initialize pointers for partitioning.
        j = ir;
        a = s.get(l + 1); //Partitioning element.
        for(;;) {         //Beginning of innermost loop.
          do i++; while (i < end && s.getComparator().compare(s.get(i), a) < 0); //Scan up to find element > a.
          do j--; while (j >= begin && s.getComparator().compare(s.get(j), a) > 0); //Scan down to find element < a.
          if(j < i) break;//Pointers crossed. Partitioning complete.
          swap(s, i, j);  //Exchange elements.
        }                 //End of innermost loop.
        //Insert partitioning element:
        s.set(l + 1, s.get(j));
        s.set(j, a);
        jstack += 2;
        //Push pointers to larger subarray on stack, process smaller subarray immediately.
        if(jstack >= NSTACK) throw new ArrayIndexOutOfBoundsException();
        if(ir - i + 1 >= j - l) {
          istack[jstack] = ir;
          istack[jstack - 1] = i;
          ir = j - 1;
        } else {
          istack[jstack] = j - 1;
          istack[jstack - 1] = l;
          l = i;
        }
      }
    }
  }
}