/*
 * Bibliography package
 * Copyright (C) 2003  Julien Dutheil
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package gnu.jtools.utils.string;

public class TextTools {

  public static boolean isWhiteSpaceCharacter(char c) {
    return c == ' '
        || c == '\t'
        || c == '\n'
        || c == '\r'
        || c == '\f';
  }

  public static int getNextNonBlanckCharIndex(String s, int currentIndex) {
    int i;
    for (i = currentIndex + 1;
         i < s.length() && isWhiteSpaceCharacter(s.charAt(i)); i++) {
      ;
    }
    return i;
  }

  public static int getNextBlanckCharIndex(String s, int currentIndex) {
    int i;
    for (i = currentIndex + 1;
         i < s.length() && !isWhiteSpaceCharacter(s.charAt(i)); i++) {
      ;
    }
    return i;
  }

  public static char toUpperCase(char c) {
    String forbiddenChars = "&\"#'{([-|_\\^@)]=}$é*%èà";
    if (forbiddenChars.indexOf(c) > -1) {
      return c;
    }
    else {
      return Character.toUpperCase(c);
    }
  }

  public static String toUpper(String s) {
    String result = "";
    for (int i = 0; i < s.length(); i++) {
      result += Character.toUpperCase(s.charAt(i));
    }
    return result;
  }

  public static String toLower(String s) {
    String result = "";
    for (int i = 0; i < s.length(); i++) {
      result += Character.toLowerCase(s.charAt(i));
    }
    return result;
  }

  public static String normalizeCase(String s) {
    if (s == null) {
      return null; //Transmission of null objects.
    }
    StringBuffer sb = new StringBuffer(s.toLowerCase());
    for (int i = 0; i < sb.length(); i++) {
      if (i == 0) {
        sb.setCharAt(i, toUpperCase(sb.charAt(i))); //first char is set to upper case.
      }
      else {
        if (sb.charAt(i) == '.') {
          if(i > 0 && i < sb.length() - 2) {
            String t = sb.substring(i-1, i+3);
            if(t.equalsIgnoreCase("i.e.")) i = getNextNonBlanckCharIndex(s, i);
            if(t.equalsIgnoreCase("e.g.")) i = getNextNonBlanckCharIndex(s, i);
            else { // not an abbreviation (i.e., e.g., ...)
              i = getNextNonBlanckCharIndex(s, i);
              if (i < sb.length()) {
                sb.setCharAt(i, toUpperCase(sb.charAt(i))); //char following a dot set to upper case.
              }
            }
          }
        }
      }
    }
    return sb.toString();
  }

  public static int getNextWordFirstCharIndex(String s, int currentIndex) {
    char c = s.charAt(currentIndex);
    if (isWhiteSpaceCharacter(c)) {
      return getNextNonBlanckCharIndex(s, currentIndex);
    }
    else {
      return getNextNonBlanckCharIndex(s,
                                       getNextBlanckCharIndex(s, currentIndex));
    }
  }

  public static int indexOfFirstOf(String string, String charList) {
    for (int i = 0; i < string.length(); i++) {
      if (charList.indexOf(string.charAt(i)) > -1) {
        return i;
      }
    }
    return -1; //Char not found.
  }

  public static int indexOfLastOf(String string, String charList) {
    for (int i = string.length() - 1; i >= 0; i--) {
      if (charList.indexOf(string.charAt(i)) > -1) {
        return i;
      }
    }
    return -1; //Char not found.
  }

  public static int indexOfFirstNonBlankChar(String s) {
    int i;
    for (i = 0; (s.charAt(i) == ' '
                 || s.charAt(i) == '\t'
                 || s.charAt(i) == '\n'
                 || s.charAt(i) == '\r'
                 || s.charAt(i) == '\f'
                 ) && i < s.length(); i++) {}
    return i < s.length() ? i : -1;
  }

  public static String getFirstNonBlankChar(String s) {
    if(isVoid(s)) return "";
    int i;
    for (i = 0; (s.charAt(i) == ' '
                 || s.charAt(i) == '\t'
                 || s.charAt(i) == '\n'
                 || s.charAt(i) == '\r'
                 || s.charAt(i) == '\f'
                 ) && i < s.length(); i++) {}
    return i < s.length() ? "" + s.charAt(i) : "";
  }

  public static String setFirstCharsOfEachWordsToUpperCase(String s) {
    if (s == null) {
      return null; //Transmission of null objects.
    }
    StringBuffer sb = new StringBuffer(s.toLowerCase());
    for (int i = 0; i < sb.length(); i++) {
      if (i == 0) {
        sb.setCharAt(i, toUpperCase(sb.charAt(i))); //first char is set to upper case.
      }
      else {
        i = getNextWordFirstCharIndex(s, i);
        if (i < sb.length()) {
          sb.setCharAt(i, toUpperCase(sb.charAt(i))); //char following a dot set to upper case.
        }
      }
    }
    return sb.toString();
  }

  public static String removeBlanks(String s) {
    StringBuffer sb = new StringBuffer(s);
    for (int i = 0; i < sb.length(); i++) {
      if (sb.charAt(i) == ' '
          || sb.charAt(i) == '\t'
          || sb.charAt(i) == '\n'
          || sb.charAt(i) == '\r'
          || sb.charAt(i) == '\f'
          ) {
        sb.deleteCharAt(i); //Remove blanks
        i--;
      }
    }
    return sb.toString();
  }

  public static String removeFirstBlanks(String s) {
    StringBuffer sb = new StringBuffer(s);
    while (sb.length() > 0
           && (isWhiteSpaceCharacter(sb.charAt(0)))
           ) {
      sb.deleteCharAt(0);
    }
    return sb.toString();
  }

  public static String removeLastBlanks(String s) {
    StringBuffer sb = new StringBuffer(s);
    while (sb.length() > 0
           && isWhiteSpaceCharacter(sb.charAt(sb.length() - 1))
           ) {
      sb.deleteCharAt(sb.length() - 1);
    }
    return sb.toString();
  }

  public static String removeFirstAndLastBlanks(String s) {
    return removeFirstBlanks(removeLastBlanks(s));
  }

  public static String removeEOL(String s) {
    return removeChar(s, '\n');
  }

  public static int firstCharBefore(String s, int index) {
    int i;
    for (i = index - 1; i >= 0 && isWhiteSpaceCharacter(s.charAt(i)); i--) {
      ;
    }
    return i;
  }

  public static String normalizeLinebreaks(String s) {
    if (s == null) {
      return null; //redirection
    }
    StringBuffer sb = new StringBuffer(s);
    for (int i = 1; i < sb.length(); i++) {
      int fcb = firstCharBefore(s, i);
      if ( (sb.charAt(i) == '\n' || sb.charAt(i) == '\r' ||
            sb.charAt(i) == '\f')
          && (fcb == -1 || sb.charAt(fcb) != '.')) {
        //Remove this line break:
        sb.deleteCharAt(i);
        i--;
      }
    }
    return sb.toString();
  }

  public static String normalizeSpaces(String s) {
    if (s == null) {
      return null; //redirection
    }
    StringBuffer sb = new StringBuffer(removeLastBlanks(removeFirstBlanks(s)).replaceAll("\t"," "));
    for (int i = 1; i < sb.length(); i++) {
      if ( (sb.charAt(i) == ' '
            && (sb.charAt(i - 1) == ' '
                || sb.charAt(i - 1) == '\n'
                || sb.charAt(i - 1) == '\r'
                || sb.charAt(i - 1) == '\f'
                )
            )
          ) {
        //Remove this space:
        sb.deleteCharAt(i);
        i--;
      }
    }
    return sb.toString();
  }

  public static String normalizeSpacesAndLinebreaks(String s) {
    if (s == null) {
      return null; //redirection
    }
    return normalizeSpaces(normalizeLinebreaks(s));
  }

  public static String normalize(String s) {
    if (s == null) {
      return null; //redirection
    }
    return normalizeCase(normalizeSpaces(normalizeLinebreaks(s)));
  }

  public static String removeChar(String s, char c) {
    StringBuffer sb = new StringBuffer(s);
    for (int i = 0; i < sb.length(); i++) {
      if (sb.charAt(i) == c) {
        sb.deleteCharAt(i); //Remove char
        i--;
      }
    }
    return sb.toString();
  }

  public static boolean isVoid(String s) {
    if (s == null) {
      return true;
    }
    return removeBlanks(s).length() == 0;
  }

  public static String replace(String chain, String pattern, String replacement) {
    StringBuffer sb = new StringBuffer(chain);
    int index = sb.indexOf(pattern);
    while (index > -1) {
      sb.replace(index, index + pattern.length(), replacement);
      index = sb.indexOf(pattern, index + replacement.length());
    }
    return sb.toString();
  }

  /**
   * @brief Send a string of size 'newSize', which is a copy of 's' truncated or
   * filled with character 'fill' at the end.
   *
   * @param s       The string to parse.
   * @param newSize The new string size.
   * @return A string of size newsize which is a copy from the left of s.
   */
  public static String resizeRight(String s, int newSize, char fill) {
    if (s.length() > newSize) {
      return s.substring(0, newSize);
    }
    String t = s;
    for (int i = 0; i < newSize - s.length(); i++) {
      t += fill;
    }
    return t;
  }

  /**
   * @brief Send a string of size 'newSize', which is a copy of 's' truncated or
   * filled with character 'fill' at the beginning.
   *
   * @param s       The string to parse.
   * @param newSize The new string size.
   * @return A string of size newsize which is a copy from the right of s.
   */
  public static String resizeLeft(String s, int newSize, char fill) {
    if (s.length() > newSize) {
      return s.substring(s.length() - newSize);
    }
    String t = new String();
    for (int i = 0; i < newSize - s.length(); i++) {
      t += fill;
    }
    t += s;
    return t;
  }

  /**
   * @brief Split a string into parts of size 'n'.
   *
   * The last part may contain < n chars.
   *
   * @param s The string to parse.
   * @param n The number of tokens.
   * @return An array of strings with all tokens.
   */
  public static String[] split(String s, int n) {
    int r = (int)(s.length() / n);
    if(r*n < s.length()) r++; // This is to obtain the equivalent of ceil( length/n), which does not wort with Math.ceil()!!!
    String[] v = new String[r];
    String tmp = new String(s);
    int i = 0;
    while (tmp.length() > n) {
      v[i] = tmp.substring(0, n);
      i++;
      tmp = tmp.substring(n);
    }
    v[i] = tmp;
    return v;
  }

  /**
   * Create a string that is 'n' times the character 'c'.
   *
   * @param c char The character to use
   * @param n int  The length of the string;
   * @return String A string made of n characters c.
   */
  public static String rep(char c, int n) {
    String s = "";
    for(int i = 0; i < n; i++) s += c;
    return s;
  }

}
