package gnu.jtools.utils.xmltools;

import gnu.jtools.utils.storage.ReverseMap;
import gnu.jtools.utils.storage.SymetricMap;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.BorderLayout;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;
import java.util.*;
import java.io.*;

public class ParserOptionPanel extends JPanel {

  public ParserOptionPanel(boolean readOrWrite) {
    this.readOrWrite = readOrWrite;
    ttm = new TagTableModel();
    table = new JTable(ttm);

    availableTags = new DefaultListModel();
    list = new JList(availableTags);
    /*list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    list.addMouseListener(new MouseAdapter() {
      public void mousePressed(MouseEvent e) {
        if(e.getClickCount() == 2) {
          ttm.addRow((String)list.getSelectedValue(), (String)list.getSelectedValue(), true);
          table.updateUI();
        }
      }
    });*/
    list.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);

    initActions();
    initGUI();
  }

  private void initActions() {
    /*
    add = new AbstractAction("Add") {
      public void actionPerformed(ActionEvent e) {
        ttm.addRow("New Tag", "value", true);
        table.updateUI();
      }
    };

    remove = new AbstractAction("Remove") {
      public void actionPerformed(ActionEvent e) {
        int row = table.getSelectedRow();
        ttm.removeRow(row);
        table.updateUI();
      }
    };

    final ParserOptionPanel pop = this;
    set = new AbstractAction("Set") {
      public void actionPerformed(ActionEvent e) {
        int listIndex = list.getSelectedIndex();
        if(listIndex == -1) return; // no selection.
        int tableIndex = table.getSelectedRow();
        if(tableIndex == -1) return; // no selection.
        Object tag = list.getSelectedValue();
        table.setValueAt(tag, tableIndex, pop.readOrWrite ? 1 : 0);
        table.updateUI();
      }
    };*/

    final ParserOptionPanel pop = this;
    load = new AbstractAction("Load") {
      public void actionPerformed(ActionEvent e) {
        if(fileChooser == null) fileChooser = new JFileChooser();
          int i = fileChooser.showOpenDialog(pop);
          if(i != JFileChooser.CANCEL_OPTION) {
            try {
              loadTranslationTable(fileChooser.getSelectedFile());
            } catch(IOException ioe) {
              JOptionPane.showMessageDialog(
                pop,
                "Bad translation file.",
                "Error!",
                JOptionPane.ERROR_MESSAGE);
            }
          }
      }
    };

    addFromList = new AbstractAction("<<") {
      public void actionPerformed(ActionEvent e) {
        Object[] selectedTags = list.getSelectedValues();
        for(int i = 0; i < selectedTags.length; i++) {
          ttm.addRow((String)selectedTags[i], (String)selectedTags[i], true);
          table.updateUI();
          ((DefaultListModel)list.getModel()).removeElement(selectedTags[i]);
        }
      }
    };

    removeToList = new AbstractAction(">>") {
      public void actionPerformed(ActionEvent e) {
        int[] selectedRows = table.getSelectedRows();
        for(int i = selectedRows.length - 1; i >= 0; i--) {
          if(ttm.isRowEditable(selectedRows[i])) {
            ( (DefaultListModel) list.getModel()).addElement(ttm.getValueAt(selectedRows[i], 0));
            ttm.removeRow(selectedRows[i]);
            table.updateUI();
          }
        }
      }
    };
  }

  private void initGUI() {
    // Setting up layout:
    setLayout(new BorderLayout());

    //Right panel:
    JPanel rightPanel = new JPanel();
    rightPanel.setLayout(new GridBagLayout());
    GridBagConstraints gbc = new GridBagConstraints();
    gbc.fill = gbc.BOTH;
    gbc.gridx = 1;
    gbc.gridy = 0;
    gbc.gridwidth = 1;
    gbc.gridheight = 2;
    gbc.weightx = 100;
    gbc.weighty = 0;
    rightPanel.add(new JScrollPane(list), gbc);
    gbc.gridx = 0;
    gbc.gridy = 0;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.weightx = 0;
    gbc.weighty = 50;
    rightPanel.add(new JButton(addFromList), gbc);
    gbc.gridx = 0;
    gbc.gridy = 1;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.weightx = 0;
    gbc.weighty = 50;
    rightPanel.add(new JButton(removeToList), gbc);


    JSplitPane split = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
    split.setLeftComponent(new JScrollPane(table));
    split.setRightComponent(rightPanel);
    add(split, BorderLayout.CENTER);
    split.setDividerLocation(300);
    editBar = new JToolBar(JToolBar.HORIZONTAL);
    //editBar.add(add);
    //editBar.add(remove);
    editBar.add(load);
    //editBar.add(set);
    this.add(editBar, BorderLayout.PAGE_END);
  }

  public void setReadOrWrite(boolean readOrWrite) { this.readOrWrite = readOrWrite; }

  public String getTagValue(String tagName) {
    for(int i = 0; i < ttm.getRowCount(); i++) if(((String)ttm.getValueAt(i, 0)).equals(tagName)) return (String)ttm.getValueAt(i, 1);
    return null;
  }

  public String getTagName(int index) {
    return (String)ttm.getValueAt(index, 0);
  }

  public String getTagValue(int index) {
    return (String)ttm.getValueAt(index, 1);
  }

  public TableModel getTagTableModel() { return ttm; }

  public void setTagTableModel(TagTableModel ttm) { this.ttm = ttm; table.updateUI(); }

  public class TagTableModel extends AbstractTableModel {

    public TagTableModel() {
      editableRows = new ArrayList();
      data = new List[2];
      data[0] = new ArrayList();
      data[1] = new ArrayList();
    }

    public int getRowCount() { return data[0].size(); }

    public int getColumnCount() { return 2; }

    public String getColumnName(int column) { return columnNames[column]; }

    public Object getValueAt(int row, int column) {
      return data[column].get(row);
    }

    public void setValueAt(Object value, int row, int column) {
      data[column].set(row, value);
    }

    public void addRow(String tagName, String tagValue, boolean editable) {
      data[0].add(tagName);
      data[1].add(tagValue);
      editableRows.add(new Boolean(editable));
    }

    public void setRow(int row, String tagName, String tagValue) {
      this.setValueAt(tagValue, row, 1);
      if(((Boolean)editableRows.get(row)).booleanValue()) {
        this.setValueAt(tagName, row, 0);
      }
    }

    public void removeRow(int row) {
      if(row > -1 && row < this.getRowCount() && ((Boolean)editableRows.get(row)).booleanValue()) {
        data[0].remove(row);
        data[1].remove(row);
        editableRows.remove(row);
      }
    }

    public void removeEditableRows() {
      int n = getRowCount();
      for(int i = n - 1; i >= 0; i--) {
        if(isRowEditable(i)) removeRow(i);
      }
    }

    public boolean isRowEditable(int row) { return ((Boolean)editableRows.get(row)).booleanValue(); }

    public boolean isCellEditable(int row, int column) {
      if(readOrWrite) {
        return column == 0;
      } else {
        return column == 1;
      }
    }

    public String getTag(String attributeName) {
      for(int i = 0; i < getRowCount(); i++) {
        if(getValueAt(i, 0).equals(attributeName)) return (String)getValueAt(i, 1);
      }
      return null;
    }

    public int getTagIndex(String attributeName) {
      for(int i = 0; i < getRowCount(); i++) {
        if(getValueAt(i, 0).equals(attributeName)) return i;
      }
      return -1;
    }

    private List editableRows;
    private List[] data;
  }

  public ReverseMap getTagDescription() {
    ReverseMap result = new SymetricMap();
    for(int i = 0; i < ttm.getRowCount(); i++) result.put(ttm.getValueAt(i, 0), ttm.getValueAt(i, 1));
    //System.out.println("Tag description:");
    //System.out.println(result.toString());
    return result;
  }

  public void setAvailableTags(Set tags) {
    availableTags.removeAllElements();
    for(Iterator i = tags.iterator(); i.hasNext();) {
      availableTags.addElement(i.next());
    }
    list.updateUI();
  }

  public void resetTranslationTable() { ttm.removeEditableRows(); table.updateUI(); }

  public void loadTranslationTable(File file) throws IOException {
    BufferedReader in = new BufferedReader(new FileReader(file));
    String s = in.readLine();
    while(s != null) {
      StringTokenizer st = new StringTokenizer(s, "\t");
      String name = st.nextToken();
      String tag  = st.nextToken();
      int eTag    = ttm.getTagIndex(name);
      //if(eTag == -1) ttm.addRow(name, tag, true);
      //else ttm.setValueAt(tag, eTag, 1);
      if(eTag > -1) ttm.setValueAt(tag, eTag, 1); //Else ignored.
      s = in.readLine();
    }
    table.updateUI();
  }

  protected boolean readOrWrite;
  private JTable table;
  protected TagTableModel ttm;
  private JList list;
  protected DefaultListModel availableTags;
  private Action /*add, remove, set, */load;
  private Action addFromList, removeToList;
  private JToolBar editBar;
  private JFileChooser fileChooser;
  private static String[] columnNames = { "Attribute name: ", "Corresponding tag in file: " };
}
