package gnu.jtools.utils.xmltools;

import gnu.jtools.utils.file.FileTools;
import java.util.*;
import java.io.*;

public class XMLTools {

  public static class Tag {

    public Tag(String name, HashMap attributes, int beginningIndex, int endingIndex) {
      this.name = name;
      this.attributes = attributes;
      this.beginningIndex = beginningIndex;
      this.endingIndex = endingIndex;
    }

    public String getName()        { return name; }
    public HashMap getAttributes() { return attributes; }
    public int getBeginningIndex() { return beginningIndex; }
    public int getEndingIndex()    { return endingIndex; }

    private String name;
    private HashMap attributes;
    private int beginningIndex;
    private int endingIndex;
  }

  public static class TagNotFoundException extends Exception {}

/*  public static String removeSpaces(String s) {
    StringBuffer sb = new StringBuffer(s);
    for(int i = 0; i<sb.length(); i++) {
      if(sb.charAt(i)==' ') {
        sb.deleteCharAt(i);
        i--;
      }
    }
    return sb.toString();
  }*/

  public static String getTagName(String s) {
    StringTokenizer analyse = new StringTokenizer(s);
    return analyse.nextToken();
  }

  public static HashMap getTagAttributes(String s) {
    HashMap result = new HashMap();
    StringTokenizer analyse1 = new StringTokenizer(s), analyse2; //Blank caracters are the delimiters.
    int n1 = analyse1.countTokens(), n2;
    for(int i = 0; i < n1; i++) {
      analyse2 = new StringTokenizer(analyse1.nextToken(), "=");
      n2 = analyse2.countTokens();
      if(n2 == 2) result.put(analyse2.nextToken(), analyse2.nextToken());
    }
    return result;
  }

  public static Tag findFirstOpeningTag(String s) throws TagNotFoundException {
    return findFirstOpeningTag(s, 0);
  }
  public static Tag findFirstOpeningTag(String s, int from) throws TagNotFoundException {
    int i = from, j = 0;
    loop : while(i < s.length()) {
      i = s.indexOf('<', i);
      if (i == -1) break loop;
      if(s.charAt(i + 1) == '/') {//this is a closing tag!
        i = s.indexOf('>', i + 1);
        continue loop;
      } else if(s.charAt(i + 1) == '?') {//first line of description
        i = s.indexOf('>', i + 1);
        continue loop;
      } else if(s.charAt(i + 1) == '!') {//validation line. Will take this into account later...
        i = s.indexOf('>', i + 1);
        continue loop;
      } else {//this is an opening tag...
        j = s.indexOf('>', i + 1);
        break loop;
      }
    }
    if(i < s.length() && i >= 0 && j >= 0) {//Correct opening tag.
      String tag = s.substring(i + 1, j);
      return new Tag(getTagName(tag), getTagAttributes(tag), i + 1, j);
    } else throw new TagNotFoundException();
  }

  public static Tag findFirstClosingTag(Tag openingTag, String s) throws TagNotFoundException {
    return findFirstClosingTag(openingTag.getName(), s, openingTag.getEndingIndex() + 1);
  }
  public static Tag findFirstClosingTag(String tagName, String s) throws TagNotFoundException {
    return findFirstClosingTag(tagName, s, 0);
  }
  public static Tag findFirstClosingTag(String tagName, String s, int from) throws TagNotFoundException {
    int i = from, j = 0, count = 0;
    loop : while(i < s.length()) {
      i = s.indexOf('<', i);
      if (i == -1) break loop;
      if(s.charAt(i + 1) == '/') {//this is a closing tag!
        j = s.indexOf('>', i + 2);
        //Check if the closing tag has correct name:
        if(getTagName(s.substring(i + 2, j)).equals(tagName)) {//ok
          if(count == 0) break loop;
          else {
            count --;
            i = j + 1;
          }
        }
        else {//not ok...
          i = j;
          continue loop;
        }
      } else {//this is an opening tag...
        j = s.indexOf('>', i + 1);
        if(s.substring(i + 1, j).equals(tagName)) {
          //This is another element enclosed in this element.
          count++; //Must find the closing tag of this element first.
                   //We could use recursity and invoke the method again, but it would be slower, I think.
        }
        i = j;
        continue loop;
      }
    }
    if(i < s.length() && i > 0 && j > 0) {//Correct closing tag.
      String tag = s.substring(i + 1, j);
      return new Tag(getTagName(tag), getTagAttributes(tag), i + 1, j);
    } else throw new TagNotFoundException();
  }

  public static XMLDocument createNewDocument(String name, String content) {
    XMLDocument newDocument = new XMLDocument(name, content);
    int i = 0, j = 0;
    Tag currentOpeningTag, currentClosingTag;
    //Getting all children:
    try {
      while(i < content.length()) {
        currentOpeningTag = findFirstOpeningTag(content, i);
        i = currentOpeningTag.getEndingIndex() + 1;
        currentClosingTag = findFirstClosingTag(currentOpeningTag, content);
        j = currentClosingTag.getBeginningIndex() - 1;
        newDocument.addChild(createNewElement(currentOpeningTag.getName(), content.substring(i, j), currentOpeningTag.getAttributes(), newDocument));
        i = currentClosingTag.getEndingIndex() + 1;
      }
    } catch(TagNotFoundException e) {
      //s is a simple element, nothing is added. End of recursion.
    }
    return newDocument;
  }

  public static XMLElement createNewElement(String name, String content, HashMap attributes, XMLElement parent) {
    XMLElement newElement = new XMLElement(name, attributes, content, parent);
    int i = 0, j = 0;
    Tag currentOpeningTag, currentClosingTag;
    //Getting all children:
    try {
      while(i < content.length()) {
        currentOpeningTag = findFirstOpeningTag(content, i);
        i = currentOpeningTag.getEndingIndex() + 1;
        currentClosingTag = findFirstClosingTag(currentOpeningTag, content);
        j = currentClosingTag.getBeginningIndex() - 1;
        newElement.addChild(createNewElement(currentOpeningTag.getName(), content.substring(i, j), currentOpeningTag.getAttributes(), newElement));
        i = currentClosingTag.getEndingIndex() + 1;
      }
    } catch(TagNotFoundException e) {
      newElement.setElement(content);//s is a simple element, nothing is added. End of recursion.
    }
    return newElement;
  }

  public static XMLDocument readFile(File XMLFile) throws IOException {
    BufferedReader input = new BufferedReader (new FileReader(XMLFile)); //the file to be read.
    String content = "";
    String lineRead = input.readLine();
    do { content += lineRead; lineRead = input.readLine(); } while(lineRead != null);
    //System.out.println("Reading XML file " + XMLFile.getAbsolutePath());
    try {
      Tag t1 = findFirstOpeningTag(content);
      Tag t2 = findFirstClosingTag(t1, content);
      return createNewDocument(t1.getName(), content.substring(t1.getEndingIndex() + 2, t2.getBeginningIndex() - 1));
    } catch(TagNotFoundException e) { throw new IOException(); }
  }

  public static void writeDocument(XMLDocument doc, File outFile) throws IOException {
    PrintWriter pw = new PrintWriter(new FileWriter(outFile));
    doc.print(pw);
    pw.close();
  }

  public static javax.swing.filechooser.FileFilter XML_FILTER = new javax.swing.filechooser.FileFilter() {

    public boolean accept(File file) {
      if(file.isDirectory()) return true;
      String extension=FileTools.getExtension(file);
      if(extension!=null) {
        return (extension.equalsIgnoreCase("xml"));
      }
      return false;
    }

    // The description of this filter
    public String getDescription() {
      return "XML files (*.xml)";
    }
  };
}
