// [English version follows the french one]
//
// Copyright ou  ou Copr. Julien Dutheil, UMR 5171 "Gnome, populations,
// interactions, adaptation" CNRS, Universit Montpellier II, IFREMER
// (Septembre 2004)
//
// Julien.Dutheil@univ-montp2.fr
//
// Ce logiciel est un programme informatique servant  diter des arbres
// phylogntiques.
//
// Ce logiciel est rgi par la licence CeCILL soumise au droit franais et
// respectant les principes de diffusion des logiciels libres. Vous pouvez
// utiliser, modifier et/ou redistribuer ce programme sous les conditions
// de la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA
// sur le site "http://www.cecill.info".
//
// En contrepartie de l'accessibilit au code source et des droits de copie,
// de modification et de redistribution accords par cette licence, il n'est
// offert aux utilisateurs qu'une garantie limite.  Pour les mmes raisons,
// seule une responsabilit restreinte pse sur l'auteur du programme,  le
// titulaire des droits patrimoniaux et les concdants successifs.
//
// A cet gard  l'attention de l'utilisateur est attire sur les risques
// associs au chargement,   l'utilisation,   la modification et/ou au
// dveloppement et  la reproduction du logiciel par l'utilisateur tant
// donn sa spcificit de logiciel libre, qui peut le rendre complexe 
// manipuler et qui le rserve donc  des dveloppeurs et des professionnels
// avertis possdant  des  connaissances  informatiques approfondies.  Les
// utilisateurs sont donc invits  charger  et  tester  l'adquation  du
// logiciel  leurs besoins dans des conditions permettant d'assurer la
// scurit de leurs systmes et ou de leurs donnes et, plus gnralement,
//  l'utiliser et l'exploiter dans les mmes conditions de scurit.
//
// Le fait que vous puissiez accder  cet en-tte signifie que vous avez
// pris connaissance de la licence CeCILL, et que vous en avez accept les
// termes.
//
// -------------------------------------------------------------------------
//
//Copyright or  or Copr. Julien Dutheil, UMR 5171 "Gnome, populations,
// interactions, adaptation" CNRS, Universit Montpellier II, IFREMER
// (Septembre 2004)
//
// Julien.Dutheil@univ-montp2.fr
//
//
// This software is a computer program whose purpose is to edit
// phylogenetic trees.
//
// This software is governed by the CeCILL  license under French law and
// abiding by the rules of distribution of free software.  You can  use,
// modify and/ or redistribute the software under the terms of the CeCILL
// license as circulated by CEA, CNRS and INRIA at the following URL
// "http://www.cecill.info".
//
// As a counterpart to the access to the source code and  rights to copy,
// modify and redistribute granted by the license, users are provided only
// with a limited warranty  and the software's author,  the holder of the
// economic rights,  and the successive licensors  have only  limited
// liability.
//
// In this respect, the user's attention is drawn to the risks associated
// with loading,  using,  modifying and/or developing or reproducing the
// software by the user in light of its specific status of free software,
// that may mean  that it is complicated to manipulate,  and  that  also
// therefore means  that it is reserved for developers  and  experienced
// professionals having in-depth computer knowledge. Users are therefore
// encouraged to load and test the software's suitability as regards their
// requirements in conditions enabling the security of their systems and/or
// data to be ensured and,  more generally, to use and operate it in the
// same conditions as regards security.
//
// The fact that you are presently reading this means that you have had
// knowledge of the CeCILL license and that you accept its terms.
//
// -------------------------------------------------------------------------


/**
 * <p>Description : Baobab core classes.</p>
 * <p>Copyright : Copyright (c) Julien Dutheil</p>
 * @author Julien Dutheil
 */

package TreeEditor;

import java.awt.*;
import bio.dendogram.*;
import bio.dendogram.xml.*;
import java.util.*;

public class ColoringTools {

  public static class SingleTypeDescriptiveState implements ColoringMethod {

    public SingleTypeDescriptiveState(String type, Map associatedColors) {
      this.type = type;
      colors = associatedColors;
    }

    public Color getColor(XNode node) {
      Object value = node.getAttribute(type);
      Color c = (Color)colors.get(value);
      return c == null ? Color.black : c;
    }

    private String type;
    private Map colors;
  }



  public static class SingleTypeNumericalValue implements ColoringMethod {

    public SingleTypeNumericalValue(String type, Map associatedColors, int option) throws NumberFormatException {
      this.type = type;
      Map associations = new HashMap(associatedColors);
      if(associations.containsKey("default")) {
        defaultColor = (Color)associations.remove("default");
      } else {
        defaultColor = null;
      }
      String[] keys = (String[])associations.keySet().toArray(new String[0]);
      levels = new Double[keys.length];
      Map associatedColors2 = new HashMap();
      for(int i = 0; i < keys.length; i++) {
        levels[i] = new Double(keys[i]);
        associatedColors2.put(levels[i], associations.get(keys[i]));
      }
      Arrays.sort(levels);
      colors = new Color[levels.length];
      for(int i = 0; i < levels.length; i++) colors[i] = (Color)associatedColors2.get(levels[i]);
      selectedOption = option;
    }

    public Color getColor(XNode node) {
      double value;
      Color result = Color.black;
      if(node.getAttribute(type) == null && defaultColor != null) return defaultColor;
      try {
        value = Double.parseDouble(node.getAttribute(type).toString());
        double previousLimit = 0;
        boolean first = true;

        for(int i = 0; i < colors.length; i++) {
          double limit = levels[i].doubleValue();
          Color color = colors[i];
          switch(selectedOption) {
            case(0) : if(value == limit) result = color;
//            case(1) : if(value <= limit) result = color;
//            case(-1) : if(value > limit) result = color;
            case(2) : if(first) { if(value <= limit ) result = color; }
                      else      { if(value <= limit && value > previousLimit) result = color; }
          }
          previousLimit = limit;
          if(first) first = false;
        }
      } catch(Exception e) {
        result = Color.white;
      }
      return result;
    }

    private String type;
    private Double[] levels;
    private Color[] colors;
    private Color defaultColor;

    private int selectedOption;
    public static final int EXACT = 0;
    public static final int INTERVAL = 2;
    public static final int REPARTITION = 1;
    public static final int INVERSE_REPARTITION = -1;
  }

}
