// [English version follows the french one]
//
// Copyright ou  ou Copr. Julien Dutheil, UMR 5171 "Gnome, populations,
// interactions, adaptation" CNRS, Universit Montpellier II, IFREMER
// (Septembre 2004)
//
// Julien.Dutheil@univ-montp2.fr
//
// Ce logiciel est un programme informatique servant  diter des arbres
// phylogntiques.
//
// Ce logiciel est rgi par la licence CeCILL soumise au droit franais et
// respectant les principes de diffusion des logiciels libres. Vous pouvez
// utiliser, modifier et/ou redistribuer ce programme sous les conditions
// de la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA
// sur le site "http://www.cecill.info".
//
// En contrepartie de l'accessibilit au code source et des droits de copie,
// de modification et de redistribution accords par cette licence, il n'est
// offert aux utilisateurs qu'une garantie limite.  Pour les mmes raisons,
// seule une responsabilit restreinte pse sur l'auteur du programme,  le
// titulaire des droits patrimoniaux et les concdants successifs.
//
// A cet gard  l'attention de l'utilisateur est attire sur les risques
// associs au chargement,   l'utilisation,   la modification et/ou au
// dveloppement et  la reproduction du logiciel par l'utilisateur tant
// donn sa spcificit de logiciel libre, qui peut le rendre complexe 
// manipuler et qui le rserve donc  des dveloppeurs et des professionnels
// avertis possdant  des  connaissances  informatiques approfondies.  Les
// utilisateurs sont donc invits  charger  et  tester  l'adquation  du
// logiciel  leurs besoins dans des conditions permettant d'assurer la
// scurit de leurs systmes et ou de leurs donnes et, plus gnralement,
//  l'utiliser et l'exploiter dans les mmes conditions de scurit.
//
// Le fait que vous puissiez accder  cet en-tte signifie que vous avez
// pris connaissance de la licence CeCILL, et que vous en avez accept les
// termes.
//
// -------------------------------------------------------------------------
//
//Copyright or  or Copr. Julien Dutheil, UMR 5171 "Gnome, populations,
// interactions, adaptation" CNRS, Universit Montpellier II, IFREMER
// (Septembre 2004)
//
// Julien.Dutheil@univ-montp2.fr
//
//
// This software is a computer program whose purpose is to edit
// phylogenetic trees.
//
// This software is governed by the CeCILL  license under French law and
// abiding by the rules of distribution of free software.  You can  use,
// modify and/ or redistribute the software under the terms of the CeCILL
// license as circulated by CEA, CNRS and INRIA at the following URL
// "http://www.cecill.info".
//
// As a counterpart to the access to the source code and  rights to copy,
// modify and redistribute granted by the license, users are provided only
// with a limited warranty  and the software's author,  the holder of the
// economic rights,  and the successive licensors  have only  limited
// liability.
//
// In this respect, the user's attention is drawn to the risks associated
// with loading,  using,  modifying and/or developing or reproducing the
// software by the user in light of its specific status of free software,
// that may mean  that it is complicated to manipulate,  and  that  also
// therefore means  that it is reserved for developers  and  experienced
// professionals having in-depth computer knowledge. Users are therefore
// encouraged to load and test the software's suitability as regards their
// requirements in conditions enabling the security of their systems and/or
// data to be ensured and,  more generally, to use and operate it in the
// same conditions as regards security.
//
// The fact that you are presently reading this means that you have had
// knowledge of the CeCILL license and that you accept its terms.
//
// -------------------------------------------------------------------------


/**
 * <p>Description : Baobab core classes.</p>
 * <p>Copyright : Copyright (c) Julien Dutheil</p>
 * @author Julien Dutheil
 */

package TreeEditor;

import gnu.jtools.license.LicenseDisplayDialog;
import gnu.jtools.utils.OperatingSystem;
import gnu.jtools.utils.gui.GraphicsTools;
import gnu.jtools.utils.gui.ZoomControls;
import gnu.jtools.utils.gui.ZoomDialog;
import bio.dendogram.*;
import bio.dendogram.GUI.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import org.freehep.util.export.ExportDialog;

public class MainInterface extends JFrame {

  protected Action loadTrees, saveTree, exitAction;
  protected TreeIODialog treeIODialog;

  public MainInterface(Locale locale, OperatingSystem OS) {
    setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);

    mainInterface = this;

    addWindowListener( new WindowAdapter() {
      public void windowClosing (WindowEvent ev) {
        exitAction.actionPerformed(new ActionEvent(this, 1, "Closing window"));
      }
    } );

    //initialisation
    setLocale(locale);
    setOS(OS);
    trees = new StockTree();//The list of loaded trees.
    setTitle(Baobab.message.getString("MainInterface.Title"));

    //Default tree, not in the list.
    voidTree = new ColoredTree();
    try {
      voidTree.setRootNode(voidTree.createNode("root", false));
    } catch(NodeException ne) {} // Never thrown.
    voidTree.setName(Baobab.message.getString("voidTree"));
    currentTree = voidTree;
    navigators = new Navigator[this.NUMBER_MAX_OF_NAVIGATORS];
    treeCount = 1;

    //Dialog boxes.
     printDialog = new PrintDialog(this, Baobab.path);
    colorManager = new ColorManager(this);
           infos = new AboutDialog(this);

    //Setting up the GUI:
    initGUI();

    //Menus:
    initActions();
    initMenus();
    rebuildMenu();


    //pack();
    setSize(400, 600);
    splitPane.setDividerLocation(420);
  }

  private void initActions() {

    treeIODialog = new TreeIODialog(this, true, true);
    treeIODialog.setCurrentDirectory(Baobab.path);

    final MainInterface mi = this;

    // Load tree(s):
    loadTrees = new AbstractAction(Baobab.message.getString("MainInterface.TreeLoad")) {
      public void actionPerformed(ActionEvent e) {
        treeIODialog.setReadOrWrite(true);
        try {
          //Load a tree file.
          int t = treeIODialog.showOpenDialog(mi);
          if(t == treeIODialog.CANCEL_OPTION) return; // Operation canceled.
          Tree[] tree = treeIODialog.readTree();
          if(tree == null) return; // Operation cancelled //throw new NullPointerException();
          ColoredTree[] treesToBeAdded = new ColoredTree[tree.length];
          for(int i = 0; i < treesToBeAdded.length; i++) treesToBeAdded[i] = new ColoredTree(tree[i]);
          // Check if trees are correct:
          ArrayList ts = new ArrayList();
          for(int i = 0; i < treesToBeAdded.length; i++) { // Trees must have unique leaf name:
            if(!TreeTools.treeHasRepeatedLeavesNames(treesToBeAdded[i])) ts.add(treesToBeAdded[i]);
            else Baobab.error(mi, Baobab.message.getString("treeNotValid"));
          }
          treesToBeAdded = (ColoredTree[])ts.toArray(new ColoredTree[0]); // Keep only valid trees.
          if(treesToBeAdded.length == 0) {
            Baobab.error(mi, Baobab.message.getString("noTreeFound")); // No correct tree found!
          } else {
            //TreeNameInputDialog tnid = new TreeNameInputDialog(mi);
            for(int i = 0; i < treesToBeAdded.length; i++) {
              if(treesToBeAdded[i].getName() == null) treesToBeAdded[i].setName("Tree " + treeCount++);
              trees.addTreeToList(treesToBeAdded[i], treesToBeAdded[i].getName(), treeIODialog.getSelectedFormat());
              rebuildMenu();
              //String name = tnid.getName(treesToBeAdded[i]);
              //if(name != null) {
              //  treesToBeAdded[i].setName(name);
              //  trees.addTreeToList(treesToBeAdded[i], name);
              //}
            }
          }

          if(currentTree == voidTree) {
            setCurrentTree(trees.treeAt(0)); //Redraw the main interface too.
          } else redrawMainInterface();
        } catch(FileNotFoundException fnfe) {
          Baobab.error(mi, Baobab.message.getString("badPath"));
        } catch(Exception ex) {
          Baobab.error(mi, Baobab.message.getString("badFormat")+"\n\nError Type: "+ex.getClass()+"\nIn Class: "+ex.getMessage());
          ex.printStackTrace();
        }
      }
    };

    // Save current tree:
    saveTree = new AbstractAction(Baobab.message.getString("MainInterface.TreeSave")) {
      public void actionPerformed(ActionEvent e) {
        treeIODialog.setReadOrWrite(false);
        try {
          //Save a tree to a file.
          int i = trees.whichTreeIsIt(mi.getCurrentTree());
          String preferredFormat = trees.getFormatAt(i);
          treeIODialog.setSelectedFormat(preferredFormat);
          boolean success = false;
          while(!success) {
            int t = treeIODialog.showSaveDialog(mi);
            if (t == treeIODialog.CANCEL_OPTION)return; // Operation canceled.
            success = treeIODialog.writeTree(mi.getCurrentTree());
          }
          trees.setFormatAt(i, treeIODialog.getSelectedFormat());

        } catch(FileNotFoundException fnfe) {
          Baobab.error(mi, Baobab.message.getString("badPath"));
        } catch(Exception ex) {
          Baobab.error(mi, Baobab.message.getString("badFormat")+"\n\nError Type: "+ex.getClass()+"\nIn Class: "+ex.getMessage());
          ex.printStackTrace();
        }
      }
    };

    //Exit:
    exitAction = new AbstractAction(Baobab.message.getString("MainInterface.Exit")) {
      public void actionPerformed(ActionEvent e) {
        if(trees.size() == 0) System.exit(0);
        if (Baobab.warning(mainInterface, Baobab.message.getString("MainInterface.ExitWarning")) == 0)
            System.exit(0);
      }
    };

  }

  private void initMenus() {
    menu = new JMenuBar();
    /**
     * The tree menu:
     */
    treeMenu = new JMenu(Baobab.message.getString("MainInterface.Tree"));
    treeMenu.setMnemonic(Baobab.message.getString("MainInterface.TreeMnemo").toCharArray()[0]);

    treeLoadFromFile = new JMenuItem(loadTrees);
    treeLoadFromFile.setMnemonic(Baobab.message.getString("MainInterface.TreeLoadMnemo").toCharArray()[0]);
    treeLoadFromFile.setAccelerator(KeyStroke.getKeyStroke("F2"));

    treeCreateFromScratch = new JMenuItem(Baobab.message.getString("MainInterface.TreeCreate"));
    treeCreateFromScratch.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        // Ask for tree  name:
        String name = JOptionPane.showInputDialog(
            mainInterface,
            Baobab.message.getString("MainInterface.AskForTreeName"),
            Baobab.message.getString("question"),
            JOptionPane.QUESTION_MESSAGE);
        if(name == null) return; // Operation cancelled.
        ColoredNode node = new ColoredNode(0);
        node.setName("Root of tree " + name);
        trees.addTreeToList(new ColoredTree(name, node), name, "XML");
        setCurrentTree(trees.treeAt(0)); //Redraw the main interface too.
      }
    });
    treeCreateFromScratch.setMnemonic(Baobab.message.getString("MainInterface.TreeCreateMnemo").toCharArray()[0]);

    treeSaveCurrentTree = new JMenuItem(saveTree);
    treeSaveCurrentTree.setMnemonic(Baobab.message.getString("MainInterface.TreeSaveMnemo").toCharArray()[0]);
    treeSaveCurrentTree.setAccelerator(KeyStroke.getKeyStroke("F3"));

    treeRemoveCurrentTree = new JMenuItem(Baobab.message.getString("MainInterface.TreeRemove"));
    treeRemoveCurrentTree.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if(currentTree != voidTree) {
          if(getNumberOfNavigators() > 0) {
            Baobab.error(mainInterface, Baobab.message.getString("MainInterface.ShutNavigators"));
          } else {
            int i = trees.whichTreeIsIt(currentTree);
            trees.removeTreeFromList(i);
            treeTable.updateUI();
            if(trees.size() > 0) {
              setCurrentTree(trees.treeAt(0));
            } else {
              setCurrentTree(voidTree);
            }
            redrawAllCurrentNavigators();
            redrawMainInterface();
          }
        }
      }
    });
    treeRemoveCurrentTree.setMnemonic(Baobab.message.getString("MainInterface.TreeRemoveMnemo").toCharArray()[0]);
    treeRemoveCurrentTree.setAccelerator(KeyStroke.getKeyStroke("DELETE")); //ne fonctionne pas!

    treePrint = new JMenuItem(Baobab.message.getString("MainInterface.TreePrint"));
    treePrint.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
          printDialog.showUp();
      }
    });
    treePrint.setMnemonic(Baobab.message.getString("MainInterface.TreePrintMnemo").toCharArray()[0]);
    treePrint.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P, InputEvent.CTRL_MASK));

    treeSaveAsImage = new JMenuItem(Baobab.message.getString("MainInterface.TreeSaveAsImage"));
    treeSaveAsImage.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        ExportDialog export = new ExportDialog();
        export.showExportDialog(mainInterface, "title", getTreeDrawing(), "export");
      }
    });
    treeSaveAsImage.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_I, InputEvent.CTRL_MASK));

    exit = new JMenuItem(exitAction);
    exit.setMnemonic(Baobab.message.getString("MainInterface.ExitMnemo").toCharArray()[0]);
    exit.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E, InputEvent.CTRL_MASK));

    treeMenu.add(treeLoadFromFile);
    treeMenu.add(treeCreateFromScratch);
    treeMenu.add(treeSaveCurrentTree);
    treeMenu.add(treeRemoveCurrentTree);
    treeMenu.add(treeSaveAsImage);
    treeMenu.add(treePrint);
    treeMenu.add(exit);

    menu.add(treeMenu);

    /**
     * The edit menu:
     */
    editMenu = new JMenu(Baobab.message.getString("MainInterface.Edit"));
    editMenu.setMnemonic(Baobab.message.getString("MainInterface.EditMnemo").toCharArray()[0]);

    editConvertToTree = new JMenuItem(Baobab.message.getString("MainInterface.EditConvert"));
    editConvertToTree.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        currentTree.getRootNode().deleteLength();
        rebuildMenu();
        redrawAllCurrentNavigators();
        redrawMainInterface();
        editConvertToTree.setEnabled(false);
      }
    });
    editConvertToTree.setMnemonic(Baobab.message.getString("MainInterface.EditConvertMnemo").toCharArray()[0]);

    editAttributes = new JMenu(Baobab.message.getString("Navigator.EditSetAttributes"));
    editAttributes.setMnemonic(Baobab.message.getString("Navigator.EditSetAttributesMnemo").toCharArray()[0]);

    editRemoveLengths = new JMenuItem(Baobab.message.getString("MainInterface.EditRLengths"));
    editRemoveLengths.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        currentTree.deleteLengthsOfTree();
        rebuildMenu();
        redrawAllCurrentNavigators();
        redrawMainInterface();
      }
    });
    editRemoveLengths.setMnemonic(Baobab.message.getString("MainInterface.EditRLMnemo").toCharArray()[0]);

    editRemoveBootstraps = new JMenuItem(Baobab.message.getString("MainInterface.EditRBootstraps"));
    editRemoveBootstraps.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        currentTree.deleteBootstrapsOfTree();
        rebuildMenu();
        redrawAllCurrentNavigators();
        redrawMainInterface();
      }
    });
    editRemoveBootstraps.setMnemonic(Baobab.message.getString("MainInterface.EditRBMnemo").toCharArray()[0]);

    editCreateLengths = new JMenuItem(Baobab.message.getString("MainInterface.EditCLengths"));
    editCreateLengths.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        double value;
        try {
          value = Double.parseDouble(request(Baobab.message.getString("MainInterface.DefaultLength")));
          currentTree.setLengthsOfTreeExceptRoot(value);
          rebuildMenu();
          redrawAllCurrentNavigators();
          redrawMainInterface();
        } catch(NumberFormatException npe) {
          Baobab.error(mainInterface, Baobab.message.getString("badNumber"));
        } catch(NullPointerException npe)  {} //Cancel.
      }
    });
    editCreateLengths.setMnemonic(Baobab.message.getString("MainInterface.EditCLMnemo").toCharArray()[0]);
    editCreateBootstraps = new JMenuItem(Baobab.message.getString("MainInterface.EditCBootstraps"));
    editCreateBootstraps.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        double value;
        try {
          value = Double.parseDouble(request(Baobab.message.getString("MainInterface.DefaultBootstrap")));
          currentTree.setBootstrapsOfTree(value);
          rebuildMenu();
          redrawAllCurrentNavigators();
          redrawMainInterface();
        } catch(NumberFormatException npe) {
          Baobab.error(mainInterface, Baobab.message.getString("badNumber"));
        } catch(NullPointerException npe)  {} //Cancel.
      }
    });
    editCreateBootstraps.setMnemonic(Baobab.message.getString("MainInterface.EditCBMnemo").toCharArray()[0]);

    editAttributes.add(editRemoveLengths);
    editAttributes.add(editRemoveBootstraps);
    editAttributes.add(editCreateLengths);
    editAttributes.add(editCreateBootstraps);

    editUnroot = new JMenuItem(Baobab.message.getString("MainInterface.EditUnroot"));
    editUnroot.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if(TreeTools.unroot(currentTree)) {
          returnToRoot();
          redrawAllCurrentNavigators();
          redrawMainInterface();
        } else {
          JOptionPane.showMessageDialog(
              mainInterface,
              Baobab.message.getString("MainInterface.UnrootError"),
              Baobab.message.getString("warning"),
              JOptionPane.ERROR_MESSAGE);
        }
        editUnroot.setEnabled(false);
      }
    });
    editUnroot.setMnemonic(Baobab.message.getString("MainInterface.EditUnrootMnemo").toCharArray()[0]);

    editSample = new JMenuItem(Baobab.message.getString("MainInterface.EditSample"));
    editSample.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if(sampleOptionDialog == null) sampleOptionDialog = new SampleOptionDialog(main);
        sampleOptionDialog.showUp(getCurrentTree());
      }
    });
    editSample.setMnemonic(Baobab.message.getString("MainInterface.EditSampleMnemo").toCharArray()[0]);

    editConfig = new JMenuItem(Baobab.message.getString("MainInterface.Edit.Config"));
    editConfig.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if(fileEditor == null) {
          fileEditor = new FileEditor(mainInterface, Baobab.configFile);
          JOptionPane.showMessageDialog(
              fileEditor,
              Baobab.message.getString("MainInterface.Edit.Config.Warning"),
              Baobab.message.getString("warning"),
              JOptionPane.INFORMATION_MESSAGE);
        }
        fileEditor.setVisible(true);
      }
    });
    editConfig.setMnemonic(Baobab.message.getString("MainInterface.Edit.Config.Mnemo").toCharArray()[0]);

    editMenu.add(editConvertToTree);
    editMenu.add(editUnroot);
    editMenu.add(editAttributes);
    editMenu.add(editSample);
    editMenu.add(editConfig);

    menu.add(editMenu);

    /**
     * The 'preferences' menu.
     */
    preferences = new JMenu(Baobab.message.getString("MainInterface.Preferences"));
    preferences.setMnemonic(Baobab.message.getString(
        "MainInterface.PreferencesMnemo").toCharArray()[0]);

    allowMultifurcations = new JCheckBoxMenuItem(Baobab.message.getString(
        "MainInterface.Preferences.AllowMultifurcations"));
    allowMultifurcations.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        rebuildAllCurrentNavigatorMenus();
      }
    });
    allowMultifurcations.setMnemonic(Baobab.message.getString(
        "MainInterface.Preferences.AllowMultifurcationsMnemo").charAt(0));

    allowSingleNodes = new JCheckBoxMenuItem(Baobab.message.getString(
        "MainInterface.Preferences.AllowSingleNodes"));
    allowSingleNodes.setMnemonic(Baobab.message.getString(
        "MainInterface.Preferences.AllowSingleNodesMnemo").charAt(0));

//    allowMultifurcations.setMnemonic(Baobab.message.getString(
//        "MainInterface.Preferences.AllowMultifurcationsFMnemo").toCharArray()[0]);
//
//    allowValuesModifying = new JCheckBoxMenuItem(Baobab.message.getString(
//        "MainInterface.Preferences.ALlowValuesModification"));
//    allowValuesModifying.addActionListener(new ActionListener() {
//      public void actionPerformed(ActionEvent e) {
//        currentTree.setAttributeValueEditableToAllSubtrees(XNode.NODE_LENGTH,
//            allowValuesModifying.isSelected());
//        currentTree.setAttributeValueEditableToAllSubtrees(XNode.NODE_BOOTSTRAP,
//            allowValuesModifying.isSelected());
//        rebuildMenus();
//      }
//    });
//    allowValuesModifying.setMnemonic(Baobab.message.getString(
//        "MainInterface.Preferences.AllowValuesModificationsMnemo").toCharArray()[0]);

    setZoomParameters = new JMenuItem(Baobab.message.getString(
        "MainInterface.Preferences.SetZoomParameters"));
    setZoomParameters.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        zoomPar.setVisible(true);
      }
    });
    setZoomParameters.setMnemonic(Baobab.message.getString(
        "MainInterface.Preferences.SetZoomParametersMnemo").charAt(0));

    showPreferencesDialog = new JMenuItem(Baobab.message.getString(
        "MainInterface.Preferences.ShowDialog"));
    showPreferencesDialog.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        pod.setVisible(true);
      }
    });
    showPreferencesDialog.setMnemonic(Baobab.message.getString(
        "MainInterface.Preferences.ShowDialogMnemo").charAt(0));

    preferences.add(allowMultifurcations);
    preferences.add(allowSingleNodes);
//    preferences.add(allowValuesModifying);
    preferences.add(setZoomParameters);
    preferences.add(showPreferencesDialog);

    menu.add(preferences);

    /**
     * The draw menu:
     */
    drawMenu = new JMenu(Baobab.message.getString("MainInterface.DrawMenu"));
    drawMenu.setMnemonic(Baobab.message.getString("MainInterface.DrawMenuMnemo").charAt(0));

    turtleCheckBoxItem = new JCheckBoxMenuItem(Baobab.message.getString("MainInterface.DrawTurtle"), true);
    turtleCheckBoxItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        treeDrawing.setDrawingAlgorithm(0);
        redrawAllCurrentNavigators();
        redrawMainInterface();
      }
    });
    turtleCheckBoxItem.setMnemonic(Baobab.message.getString("MainInterface.DrawTurtleMnemo").charAt(0));

    hareCheckBoxItem = new JCheckBoxMenuItem(Baobab.message.getString("MainInterface.DrawHare"));
    hareCheckBoxItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        treeDrawing.setDrawingAlgorithm(1);
        redrawAllCurrentNavigators();
        redrawMainInterface();
      }
    });
    hareCheckBoxItem.setMnemonic(Baobab.message.getString("MainInterface.DrawHareMnemo").charAt(0));

    drawMenu.add(turtleCheckBoxItem);
    drawMenu.add(hareCheckBoxItem);

    ButtonGroup g = new ButtonGroup();
                g.add(turtleCheckBoxItem);
                g.add(hareCheckBoxItem);

    menu.add(drawMenu);

    /**
     * The color menu:
     */
    colorMenu = new JMenu(Baobab.message.getString("MainInterface.ColorMenu"));
    colorMenu.setMnemonic(Baobab.message.getString("MainInterface.ColorMenuMnemo").charAt(0));

    launchColorManagerItem = new JMenuItem(Baobab.message.getString("MainInterface.LaunchCMItem"));
    launchColorManagerItem.setMnemonic(Baobab.message.getString("MainInterface.LaunchCMItemMnemo").charAt(0));
    launchColorManagerItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        colorManager.setVisible(true);
        ColoringMethod cm = colorManager.getColoringMethod();
        treeDrawing.setColoringMethod(cm);
        treeDrawing.setDrawColors(false);
        try {
          navigators[0].getTreeDrawing().setColoringMethod(cm);
          navigators[1].getTreeDrawing().setColoringMethod(cm);
          navigators[2].getTreeDrawing().setColoringMethod(cm);
        } catch(NullPointerException npe) {}
        redrawMainInterface();
        redrawAllCurrentNavigators();
      }
    });

    colorMenu.add(launchColorManagerItem);

    restoreDefaultItem = new JMenuItem(Baobab.message.getString("MainInterface.RestoreItem"));
    restoreDefaultItem.setMnemonic(Baobab.message.getString("MainInterface.RestoreItemMnemo").charAt(0));
    restoreDefaultItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        treeDrawing.setDrawColors(true);
        try {
          navigators[0].restoreDefaultColoringMethod();
          navigators[1].restoreDefaultColoringMethod();
          navigators[2].restoreDefaultColoringMethod();
        } catch(NullPointerException npe) {}
        redrawMainInterface();
        redrawAllCurrentNavigators();
      }
    });

    colorMenu.add(restoreDefaultItem);

    menu.add(colorMenu);

    /**
     * The help menu:
     */
    helpMenu = new JMenu(Baobab.message.getString("MainInterface.Help"));
    helpMenu.setMnemonic(Baobab.message.getString("MainInterface.HelpMnemo").charAt(0));

    helpHelp = new JMenuItem(Baobab.message.getString("MainInterface.HelpHelp"));
    helpHelp.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        JOptionPane.showMessageDialog(
            mainInterface,
            Baobab.message.getString("MainInterface.HelpTxt"),
            Baobab.message.getString("MainInterface.Help"),
            JOptionPane.INFORMATION_MESSAGE);
      }
    });
    helpHelp.setMnemonic(Baobab.message.getString("MainInterface.HelpHelpMnemo").charAt(0));

    helpMenu.add(helpHelp);

    helpAbout = new JMenuItem(Baobab.message.getString("MainInterface.HelpAbout"));
    helpAbout.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        infos.setVisible(true);
      }
    });
    helpAbout.setMnemonic(Baobab.message.getString("MainInterface.HelpAboutMnemo").charAt(0));
    helpMenu.add(helpAbout);

    licenceMenu = new JMenu(Baobab.message.getString("MainInterface.Licence"));
    licenceMenu.setMnemonic(Baobab.message.getString("MainInterface.LicenceMnemo").charAt(0));

    licenceFrench = new JMenuItem(Baobab.message.getString("MainInterface.Licence.French"));
    licenceFrench.setMnemonic(Baobab.message.getString("MainInterface.Licence.FrenchMnemo").charAt(0));
    licenceFrench.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        frenchLicenceDialog.setVisible(true);
      }
    });
    licenceMenu.add(licenceFrench);

    licenceEnglish = new JMenuItem(Baobab.message.getString("MainInterface.Licence.English"));
    licenceEnglish.setMnemonic(Baobab.message.getString("MainInterface.Licence.EnglishMnemo").charAt(0));
    licenceEnglish.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        englishLicenceDialog.setVisible(true);
      }
    });
    licenceMenu.add(licenceEnglish);

    helpMenu.add(licenceMenu);

    menu.add(helpMenu);

    this.setJMenuBar(menu);
  }

  private void initGUI(){
    main = this;

    // Zoom toolbar:
    zoomPar = new ZoomDialog(this);

    //The bottom panel:
    treeTableModel = new TreeTableModel(trees);
    treeTable = new JTable(treeTableModel);
    treeTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    treeTable.setShowVerticalLines(false);
    treeTable.addMouseListener(new MouseAdapter() {
      public void mouseClicked(MouseEvent e) {
        if(e.getClickCount() == 2) {//Double-click
          if(getNumberOfNavigators() > 0) {//Il faut fermer les navigateurs
            Baobab.error(mainInterface, Baobab.message.getString("MainInterface.ShutNavigators"));
          } else {
            int index = treeTable.getSelectedRow();
            setCurrentTree(trees.treeAt(index));
            //Actualize treeDrawing:
            treeDrawing.setTree(currentTree);
            //Actualize printDialog:
            main.setTitle(Baobab.message.getString("MainInterface.Title") + trees.getNameAt(index) + ", " + currentTree.getNumberOfLeaves() + " leaves.");
          }
        }
      }

    });

    //The top panel:
    treeDrawing = new ColoredTreeDrawing(currentTree, getLocale(), OS);
    treeDrawing.setPreferredSize(600, 400);
    treeDrawing.setMargins(new Insets(15, 50, 15, 400)); // There may be long names!
    treeDrawing.setDrawBootstraps(false);
    treeDrawing.setDrawLengths(false);
    treeDrawing.setDrawBranchesNumbers(true);
    treeDrawing.setDrawClickableNodes(true);
    treeDrawing.setDrawColors(true);//par dfaut en fait...
    JScrollPane lift = new JScrollPane(treeDrawing);

    //The tool bar:
    ZoomControls zoomControls = new ZoomControls(treeDrawing, lift);
    zoomPar.addZoomControls(zoomControls);

    TreeDrawingControls tdControls = new TreeDrawingControls(treeDrawing);
    JToolBar navigatorToolBar = new JToolBar();
             navigatorToolBar.add(zoomControls.getZoomWidthMinus());
             navigatorToolBar.add(zoomControls.getZoomWidthPlus());
             navigatorToolBar.add(zoomControls.getZoomHeightMinus());
             navigatorToolBar.add(zoomControls.getZoomHeightPlus());
             navigatorToolBar.add(tdControls.drawLengthsButton);
             navigatorToolBar.add(tdControls.drawBootstrapsButton);
             navigatorToolBar.add(tdControls.drawIdsButton);
//             navigatorToolBar.add(tdControls.drawGroupsButton);
             navigatorToolBar.add(tdControls.drawClickableNodesButton);
             navigatorToolBar.add(tdControls.drawNumbersAtFirstBranchesButton);
             navigatorToolBar.add(tdControls.drawLeavesNamesButton);
             navigatorToolBar.setOrientation(JToolBar.VERTICAL);
    newNavigator = new JButton(Baobab.message.getString("MainInterface.NewNavigator"));
    newNavigator.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if(countNavigators <= NUMBER_MAX_OF_NAVIGATORS && currentTree != voidTree) {//cration d'un nouveau navigateur.
          boolean continu = true;
          for(int i = 0; i < colorStatus.length && continu; i++) {
            if(!colorStatus[i]) {
              navigators[countNavigators] = new Navigator(currentTree, colors[i], mainInterface);
              navigators[countNavigators].getTreeDrawing().setDrawingAlgorithm(treeDrawing.getDrawingAlgorithm());
              navigators[countNavigators].getTreeDrawing().setColoringMethod(treeDrawing.getColoringMethod());
              pod.actualizeTreeDrawing(navigators[countNavigators].getTreeDrawing());
              navigators[countNavigators].actualizeAttributesEditor();
              colorStatus[i] = true;//cette couleur est  prsent utilise et ne sera 
                                  //nouveau disponible qu'aprs fermeture de cette fenetre.
              countNavigators++;
              currentTree.setColorsOfTree(colors[i], i);
              redrawMainInterface();
              continu = false;
            }
          }
          if(continu) {//toutes les couleurs sont utilises.
            Baobab.error(mainInterface, Baobab.message.getString("MainInterface.AllNavigators")
                    + NUMBER_MAX_OF_NAVIGATORS
                    + Baobab.message.getString("MainInterface.CloseOneNavigator"));
          }
        }
      }
    });
    newNavigator.setMnemonic(Baobab.message.getString("MainInterface.NNMnemo").toCharArray()[0]);

    setCurrentTree = new JButton(Baobab.message.getString("MainInterface.setCurrentTree"));
    setCurrentTree.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        int index = treeTable.getSelectedRow();
        if(index < 0) return; // No selection.
        setCurrentTree(trees.treeAt(index));
        //Actualize treeDrawing:
        treeDrawing.setTree(currentTree);
        //Actualize printDialog:
        main.setTitle(Baobab.message.getString("MainInterface.Title") + trees.getNameAt(index) + ", " + currentTree.getNumberOfLeaves() + " leaves.");
      }
    });
    setCurrentTree.setMnemonic(Baobab.message.getString("MainInterface.setCurrentTreeMnemo").toCharArray()[0]);

    JToolBar tb = new JToolBar();
    tb.setFloatable(false);
    tb.add(setCurrentTree);
    tb.add(newNavigator);
    Box bottomPanel = Box.createVerticalBox();
        bottomPanel.add(tb);
        bottomPanel.add(new JScrollPane(treeTable));

    //Content panel:
    splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, lift, bottomPanel);

    //Setting uo layout:
    windowContent = getContentPane();
    windowContent.setLayout(new BorderLayout());
    windowContent.add(navigatorToolBar, BorderLayout.WEST);
    windowContent.add(splitPane, BorderLayout.CENTER);

    //Licence dialog boxes:
    frenchLicenceDialog = new JDialog(main, Baobab.message.getString("MainInterface.Licence.French"), true);
    frenchLicenceDialog.getContentPane().setLayout(new BorderLayout());
    JTextPane frenchLicenseTerms = new JTextPane();
    frenchLicenseTerms.setEditable(false);
    try {
      frenchLicenseTerms.setPage(LicenseDisplayDialog.class.getResource("Licence_CeCILL_V1-fr.txt"));
    } catch(IOException e) {
      System.err.println("Can't find french licence file.");
    }
    frenchLicenceDialog.getContentPane().add(new JScrollPane(frenchLicenseTerms), BorderLayout.CENTER);
    JButton frenchOk = new JButton(Baobab.message.getString("ok"));
    frenchOk.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        frenchLicenceDialog.setVisible(false);
      }
    });
    frenchLicenceDialog.getContentPane().add(frenchOk, BorderLayout.SOUTH);
    frenchLicenceDialog.pack();
    GraphicsTools.centerHorizontally(frenchLicenceDialog);
    GraphicsTools.adjust(frenchLicenceDialog, 30, 30);

    englishLicenceDialog = new JDialog(main, Baobab.message.getString("MainInterface.Licence.English"), true);
    englishLicenceDialog.getContentPane().setLayout(new BorderLayout());
    JTextPane englishLicenseTerms = new JTextPane();
    englishLicenseTerms.setEditable(false);
    try {
      englishLicenseTerms.setPage(LicenseDisplayDialog.class.getResource("Licence_CeCILL_V1-US.txt"));
    } catch(IOException e) {
      System.err.println("Can't find english licence file.");
    }
    englishLicenceDialog.getContentPane().add(new JScrollPane(englishLicenseTerms), BorderLayout.CENTER);
    JButton englishOk = new JButton(Baobab.message.getString("ok"));
    englishOk.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        englishLicenceDialog.setVisible(false);
      }
    });
    englishLicenceDialog.getContentPane().add(englishOk, BorderLayout.SOUTH);
    englishLicenceDialog.pack();
    GraphicsTools.centerHorizontally(englishLicenceDialog);
    GraphicsTools.adjust(englishLicenceDialog, 30, 30);
    pod = new PreferencesOptionDialog(this);
  }

  public void setOS(OperatingSystem OS) { this.OS = OS; }

  public OperatingSystem getOS() { return OS; }

  public ZoomDialog getZoomDialog() { return zoomPar; }

  public boolean areMultifurcationsAllowed() { return allowMultifurcations.isSelected(); }

  public boolean areSingleNodesAllowed() { return allowSingleNodes.isSelected(); }

  //public boolean areValueModificationsAllowed() { return allowValuesModifications.isSelected(); }

  public void setCurrentTree(ColoredTree tree) {
    currentTree = tree;
    treeDrawing.setTree(tree);
    redrawMainInterface();
    rebuildMenu();
  }

  public void rebuildMenu() {
    if(currentTree.getRootNode().hasLength()) editConvertToTree.setEnabled(true);
    else editConvertToTree.setEnabled(false);
    if(currentTree.getRootNode().getNumberOfSons() == 2
       && (!currentTree.getRootNode().getSon(0).isLeaf()
       || !currentTree.getRootNode().getSon(1).isLeaf())
       ) editUnroot.setEnabled(true);
    else editUnroot.setEnabled(false);
    if(TreeTools.hasAtLeastOneLength(currentTree)) editRemoveLengths.setEnabled(true);
    else                                           editRemoveLengths.setEnabled(false);
    if(TreeTools.hasAtLeastOneBootstrap(currentTree)) editRemoveBootstraps.setEnabled(true);
    else                                              editRemoveBootstraps.setEnabled(false);
    if(currentTree.hasLengthsIgnoreRoot()) editCreateLengths.setEnabled(false);
    else                                   editCreateLengths.setEnabled(true);
    if(currentTree.hasBootstraps() || currentTree.getRootNode().isLeaf()) editCreateBootstraps.setEnabled(false);
    else                                                                  editCreateBootstraps.setEnabled(true);
    if(currentTree == voidTree) {
      setTitle(Baobab.message.getString("MainInterface.Title"));
      treeSaveCurrentTree.setEnabled(false);
      treePrint.setEnabled(false);
      treeRemoveCurrentTree.setEnabled(false);
      editUnroot.setEnabled(false);
    } else {
      setTitle(Baobab.message.getString("MainInterface.Title") + trees.getNameAt(trees.whichTreeIsIt(currentTree)) + ", "+currentTree.getNumberOfLeaves() + " leaves.");
      treeSaveCurrentTree.setEnabled(true);
      treePrint.setEnabled(true);
      treeRemoveCurrentTree.setEnabled(true);
      editUnroot.setEnabled(true);
    }
    if(trees.size() > 1) setCurrentTree.setEnabled(true);
    else                 setCurrentTree.setEnabled(false);
  }

  public void redrawMainInterface() {
    treeTable.updateUI();
    treeDrawing.scale();
    treeDrawing.repaint();
  }

  public void shutOneNavigator(Color color) {
    countNavigators--;
    int n = getNavigatorNumber(color);
    colorStatus[n] = false;//this color is no free.
    navigators[n] = null;
    currentTree.setColorsOfTree(ColoredTreeDrawing.COLOR_DEFAULT, n);
    redrawMainInterface();
  }

  public void redrawAllCurrentNavigators() {
    for(int i = 0; i < navigators.length; i++) {
      try {
        navigators[i].getTreeDrawing().setDrawingAlgorithm(treeDrawing.getDrawingAlgorithm());
        navigators[i].redraw();
        navigators[i].actualizeAttributesEditor();
      } catch(NullPointerException e) {//interface non utilise renvoit une exception
      }
    }
  }

  public void rebuildAllCurrentNavigatorMenus() {
    for(int i = 0; i < navigators.length; i++) {
      try {
        navigators[i].rebuildMenus();
      } catch(NullPointerException e) {//interface non utilise renvoie une exception
      }
    }
  }

  public void returnToRoot() {
    this.treeDrawing.returnToRoot();
    for(int i = 0; i < navigators.length; i++) {
      try {
        navigators[i].getTreeDrawing().returnToRoot();
      } catch(NullPointerException e) {//Navigator non utilise renvoit une exception
      }
    }
  }

  public StockTree getList() { return trees; }

  public ColoredTree getCurrentTree() {
    return currentTree;
  }

  public ColoredTreeDrawing getTreeDrawing() { return treeDrawing; }

  public void addTreeToList(ColoredTree tree, String name, String preferredFormat) {
    tree.setName(name);
    trees.addTreeToList(tree, name, preferredFormat);
    //list.add(name + " (" + tree.getNumberOfLeaves() + ")");
    this.redrawMainInterface();
  }

  public ColoredTree removeTreeFromList(int index) {
    //list.remove(index);
    ColoredTree tree = trees.removeTreeFromList(index);
    treeTable.updateUI();
    return tree;
  }

  public StockTree getOtherTrees() {
    StockTree otherTrees=(StockTree)(trees.clone());//c'est une nouvelle liste.
    //aucune modification sur cette liste n'affectera la liste principale.
    if(currentTree!=voidTree) otherTrees.removeTreeFromList(otherTrees.whichTreeIsIt(currentTree));
    //on a enlev l'arbre courant.
    return otherTrees;//la liste est vide si l'arbre courant est voidTreee (car rciproquement)
  }

  public String getPreferredFormatOfCurrentTree() {
    return trees.getFormatAt(trees.whichTreeIsIt(currentTree));
  }

  public int getNavigatorNumber(Color color) {
    int result = 0;
    for(int i = 0; i < colors.length; i++) {
      if(colors[i].equals(color)) result = i;
    }
    return result;
  }

  public int getNumberOfNavigators() {
    return countNavigators;
  }

  public Navigator[] getNavigators() {
    Navigator[] nav = new Navigator[countNavigators];
    for(int i = 0; i < countNavigators; i++) {
      nav[i] = navigators[i];
    }
    return nav;
  }

  private String request(String text) {
    String result;
    result = JOptionPane.showInputDialog(this, text);
    return result;
  }


  //Data:
  private StockTree trees;
  private ColoredTree currentTree;

  //Menus:
  private MainInterface mainInterface;
  private JMenuBar menu;
  private JMenu treeMenu, editMenu, drawMenu, preferences, helpMenu, editAttributes, colorMenu, licenceMenu;
  private JMenuItem treeLoadFromFile, treeCreateFromScratch, treeSaveCurrentTree, treeRemoveCurrentTree, treePrint, treeSaveAsImage, exit;
  private JMenuItem editConvertToTree, editRemoveLengths, editRemoveBootstraps, editCreateLengths, editCreateBootstraps, editUnroot, editSample, editConfig, showPreferencesDialog;
  private JCheckBoxMenuItem turtleCheckBoxItem, hareCheckBoxItem;
  private JMenuItem launchColorManagerItem, restoreDefaultItem;
  private JCheckBoxMenuItem allowMultifurcations, allowSingleNodes, allowValuesModifications;
  private JMenuItem setZoomParameters;


  private JMenuItem helpAbout, helpHelp, licenceFrench, licenceEnglish;

  //Components:
  private JTable treeTable;
  private TreeTableModel treeTableModel;
  private ColoredTreeDrawing treeDrawing;
  private JSplitPane splitPane;
  private JButton newNavigator, setCurrentTree;
  private Container windowContent;
  private MainInterface main;
  private Navigator[] navigators;

  //Dialog boxes:
  private PrintDialog printDialog;
  private SampleOptionDialog sampleOptionDialog;
  private FileEditor fileEditor;
  private JDialog frenchLicenceDialog, englishLicenceDialog;
  private PreferencesOptionDialog pod;

  //Additional stuff:
  private OperatingSystem OS;
  private AboutDialog infos;
  private ColorManager colorManager;
  private ZoomDialog zoomPar;
  public final ColoredTree voidTree;
  private int countNavigators = 0;
  private final int NUMBER_MAX_OF_NAVIGATORS = 3;
  private final Color[] colors = {Color.red, Color.blue, Color.green};
  private boolean[] colorStatus = {false, false, false};//aucune couleur utilise.

  private int treeCount;
}
