// [English version follows the french one]
//
// Copyright ou  ou Copr. Julien Dutheil, UMR 5171 "Gnome, populations,
// interactions, adaptation" CNRS, Universit Montpellier II, IFREMER
// (Septembre 2004)
//
// Julien.Dutheil@univ-montp2.fr
//
// Ce logiciel est un programme informatique servant  diter des arbres
// phylogntiques.
//
// Ce logiciel est rgi par la licence CeCILL soumise au droit franais et
// respectant les principes de diffusion des logiciels libres. Vous pouvez
// utiliser, modifier et/ou redistribuer ce programme sous les conditions
// de la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA
// sur le site "http://www.cecill.info".
//
// En contrepartie de l'accessibilit au code source et des droits de copie,
// de modification et de redistribution accords par cette licence, il n'est
// offert aux utilisateurs qu'une garantie limite.  Pour les mmes raisons,
// seule une responsabilit restreinte pse sur l'auteur du programme,  le
// titulaire des droits patrimoniaux et les concdants successifs.
//
// A cet gard  l'attention de l'utilisateur est attire sur les risques
// associs au chargement,   l'utilisation,   la modification et/ou au
// dveloppement et  la reproduction du logiciel par l'utilisateur tant
// donn sa spcificit de logiciel libre, qui peut le rendre complexe 
// manipuler et qui le rserve donc  des dveloppeurs et des professionnels
// avertis possdant  des  connaissances  informatiques approfondies.  Les
// utilisateurs sont donc invits  charger  et  tester  l'adquation  du
// logiciel  leurs besoins dans des conditions permettant d'assurer la
// scurit de leurs systmes et ou de leurs donnes et, plus gnralement,
//  l'utiliser et l'exploiter dans les mmes conditions de scurit.
//
// Le fait que vous puissiez accder  cet en-tte signifie que vous avez
// pris connaissance de la licence CeCILL, et que vous en avez accept les
// termes.
//
// -------------------------------------------------------------------------
//
//Copyright or  or Copr. Julien Dutheil, UMR 5171 "Gnome, populations,
// interactions, adaptation" CNRS, Universit Montpellier II, IFREMER
// (Septembre 2004)
//
// Julien.Dutheil@univ-montp2.fr
//
//
// This software is a computer program whose purpose is to edit
// phylogenetic trees.
//
// This software is governed by the CeCILL  license under French law and
// abiding by the rules of distribution of free software.  You can  use,
// modify and/ or redistribute the software under the terms of the CeCILL
// license as circulated by CEA, CNRS and INRIA at the following URL
// "http://www.cecill.info".
//
// As a counterpart to the access to the source code and  rights to copy,
// modify and redistribute granted by the license, users are provided only
// with a limited warranty  and the software's author,  the holder of the
// economic rights,  and the successive licensors  have only  limited
// liability.
//
// In this respect, the user's attention is drawn to the risks associated
// with loading,  using,  modifying and/or developing or reproducing the
// software by the user in light of its specific status of free software,
// that may mean  that it is complicated to manipulate,  and  that  also
// therefore means  that it is reserved for developers  and  experienced
// professionals having in-depth computer knowledge. Users are therefore
// encouraged to load and test the software's suitability as regards their
// requirements in conditions enabling the security of their systems and/or
// data to be ensured and,  more generally, to use and operate it in the
// same conditions as regards security.
//
// The fact that you are presently reading this means that you have had
// knowledge of the CeCILL license and that you accept its terms.
//
// -------------------------------------------------------------------------


/**
 * <p>Description : Baobab core classes.</p>
 * <p>Copyright : Copyright (c) Julien Dutheil</p>
 * @author Julien Dutheil
 */

package TreeEditor;

import gnu.jtools.stats.csv.CSVIO;
import gnu.jtools.stats.table.Table;
import gnu.jtools.stats.table.TableRowNotFoundException;
import gnu.jtools.utils.OperatingSystem;
import gnu.jtools.utils.attributes.AttributeNotFoundException;
import gnu.jtools.utils.gui.GraphicsTools;
import gnu.jtools.utils.gui.ZoomControls;
import gnu.jtools.utils.storage.ReverseMap;
import gnu.jtools.utils.string.TextTools;
import bio.dendogram.*;
import bio.dendogram.xml.*;
import bio.dendogram.GUI.*;
import bio.dendogram.datation.*;
import bio.dendogram.statistics.*;
import bio.sequences.*;
import bio.sequences.io.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

public class Navigator
    extends JFrame {

  protected SequenceIODialog sequenceIODialog;

  public Navigator(ColoredTree root, Color selection, MainInterface mi) {

    this.mi = mi;
    this.COLOR_SELECTED = selection;
    addWindowListener(new Fermeture());

    //TreeDrawing frame bounds:
    width = this.WINDOW_WIDTH + 30;
    height = this.WINDOW_HEIGHT - 200;
    setLocale(mi.getLocale());
    setOS(mi.getOS());

    //parametres de la fenetre
    //setBounds(330, 0, this.WINDOW_WIDTH, this.WINDOW_HEIGHT);
    setSize(this.WINDOW_WIDTH, this.WINDOW_HEIGHT);
    GraphicsTools.center(this);
    setTitle(Baobab.message.getString("Navigator.Title"));
    setDefaultCloseOperation(DISPOSE_ON_CLOSE);

    //Setting up the GUI:
    initGUI(root);
    initActions();
    initToolBars();
    initLayout();
    initMenus();
    rebuildMenus();

    setVisible(true);
    split.setDividerLocation(1.00);
  }

  private void initActions() {
    final Navigator navigator = this;
    sequenceIODialog = new SequenceIODialog(this, true, true,
                                            new DefaultAlphabet());
    actions = new EditionTools(mi, this).createEditionActions(treeDrawing);
    actions.put("Init date",
                new AbstractAction(Baobab.message.getString(
        "Navigator.Attributes.Date.Init")) {
      public void actionPerformed(ActionEvent e) {
        DatationTools.initDates(getCurrentNode());
        actualizeAttributesEditor();
      }
    });

    actions.put("Compute lengths",
                new AbstractAction(Baobab.message.getString(
        "Navigator.Attributes.Date.Compute")) {
      public void actionPerformed(ActionEvent e) {
        if (!DatationTools.hasDates(getCurrentNode())) {
          JOptionPane.showMessageDialog(
              navigator,
              Baobab.message.getString("errorDateNotFound"),
              Baobab.message.getString("error"),
              JOptionPane.ERROR_MESSAGE
              );
          return;
        }
        if (!DatationTools.isChronological(getCurrentNode())) {
          JOptionPane.showMessageDialog(
              navigator,
              Baobab.message.getString("errorChronology"),
              Baobab.message.getString("error"),
              JOptionPane.ERROR_MESSAGE
              );
          return;
        }
        DatationTools.computeBranchLengths(getCurrentNode());
        mi.redrawMainInterface();
        mi.redrawAllCurrentNavigators();
      }
    });

    actions.put("Load attributes from table",
                new AbstractAction(Baobab.message.
                                   getString("Navigator.Attributes.Load.Table")) {
      public void actionPerformed(ActionEvent e) {
        JFileChooser jfc = new JFileChooser(Baobab.path);
        jfc.setFileFilter(CSVIO.CSV_FILTER);
        if (jfc.showOpenDialog(navigator) == JFileChooser.CANCEL_OPTION) {
          return; // Operation cancelled.
        }

        CSVIO csvio = new CSVIO("NA");
        csvio.setNumberPolicy(CSVIO.ALL_STRINGS);
        if(csvio.showOptionDialog(navigator) == JOptionPane.CANCEL_OPTION) return; //operation cancelled
        try {
          Table table = csvio.readTable(jfc.getSelectedFile(), true, true);
          SpreadSheetDialog ssd = new SpreadSheetDialog(navigator, table);
          ssd.setVisible(true);
          if(TATD == null) TATD = new TableAttributesTranslationDialog(navigator);
          TATD.setReadOrWrite(true);
          TATD.setAttributeNames(table.getColumnNames());
          TATD.setVisible(true);
          ReverseMap tagTranslation = TATD.getTagTranslation();
          XTreeTools.loadAttributesFromTable(getCurrentNode(), table, tagTranslation, true);
          // Update attributes editor:
          actualizeAttributesEditor();
        }
        catch (IOException ioe) {
          // Error while loading file.
          Baobab.error(navigator, Baobab.message.getString("badFormat") + "\n" + ioe.getMessage());
          //ioe.printStackTrace();
        }
        catch (TableRowNotFoundException trnfe) {
          // Error while parsing table.
          Baobab.error(navigator, Baobab.message.getString("errorRowNotFound") + " (" + trnfe.getRowName() + ")");
        }
      }
    });

    actions.put("Write attributes to table",
                new AbstractAction(Baobab.message.
                                   getString("Navigator.Attributes.Write.Table")) {
      public void actionPerformed(ActionEvent e) {
        JFileChooser jfc = new JFileChooser(Baobab.path);
        jfc.setFileFilter(CSVIO.CSV_FILTER);
        if (jfc.showSaveDialog(navigator) == JFileChooser.CANCEL_OPTION) {
          return; // Operation cancelled.
        }

        CSVIO csvio = new CSVIO("NA");
        csvio.setNumberPolicy(CSVIO.ALL_DOUBLES);
        if(csvio.showOptionDialog(navigator) == JOptionPane.CANCEL_OPTION) return; //operation cancelled
        try {
          //Must select columns to build:
          Set tags = XTreeTools.getAvailableTags(getCurrentNode(), false);
          if(TATD == null) TATD = new TableAttributesTranslationDialog(navigator);
          TATD.setReadOrWrite(false);
          TATD.setAttributeNames((String[])tags.toArray(new String[0]));
          TATD.setVisible(true);
          ReverseMap tagTranslation = TATD.getTagTranslation();
          //Now create the table:
          Table table = new Table(tagTranslation.size());
          String[] rowNames = (String[])TreeTools.listOfLeavesNames(getCurrentNode()).toArray(new String[0]);
          table.createRows(rowNames.length);
          table.setRowNames(rowNames);
          Set rows = tagTranslation.entrySet();
          int count = 0;
          for(Iterator i = rows.iterator(); i.hasNext();) {
            table.setColumnName(count, ((Map.Entry)i.next()).getValue().toString());
            count++;
          }
          XTreeTools.writeAttributesToTable(getCurrentNode(), table, tagTranslation, true);
          SpreadSheetDialog ssd = new SpreadSheetDialog(navigator, table);
          ssd.setVisible(true);
          PrintWriter pw = new PrintWriter(new FileWriter(jfc.getSelectedFile()));
          csvio.writeTable(table, pw);
          pw.close();
        }
        catch (IOException ioe) {
          // Error while loading file.
          Baobab.error(navigator, Baobab.message.getString("badFormat") + "\n" + ioe.getMessage());
          //ioe.printStackTrace();
        }
        catch (TableRowNotFoundException trnfe) {
          // Error while parsing table.
          Baobab.error(navigator,Baobab.message.getString("errorRowNotFound") + " (" + trnfe.getRowName() + ")");
        }
        catch (Exception ex) {
          // Must not happen!
          Baobab.error(navigator, Baobab.message.getString("Error.unexpected"));
          ex.printStackTrace();
        }
      }
    });

    /*
         actions.put("TFSI on leaves",
                new AbstractAction(Baobab.message.getString(
        "Navigator.CATFSIOnLeaves")) {
      public void actionPerformed(ActionEvent e) {
        // Get the attribute name to consider:
        //String attName = JOptionPane.showInputDialog(
        //    navigator,
        //    Baobab.message.getString("Navigator.CATFSIGetAttributeName"),
        //    Baobab.message.getString("question"),
        //    JOptionPane.QUESTION_MESSAGE
        //    );
        // Now apply the test:
        if (TFSIDialog == null) {
          TFSIDialog = new TFSIDialog(navigator, true);
        }
        TFSIDialog.setTree(navigator.getTree());
        TFSIDialog.setVisible(true);
      }
         });*/

    actions.put("Load sequences",
                new AbstractAction(Baobab.message.getString(
        "Navigator.Attributes.Sequences.Load")) {
      public void actionPerformed(ActionEvent e) {
        sequenceIODialog.setReadOrWrite(true);
        int t = sequenceIODialog.showOpenDialog(navigator);
        if (t == sequenceIODialog.CANCEL_OPTION) {
          return; // Operation canceled.
        }
        try {
          SequenceContainer sc = sequenceIODialog.readSequenceFile();
          XTreeTools.loadSequences(navigator.getCurrentNode(), sc);
          navigator.actualizeAttributesEditor();
        }
        catch (SequenceNotFoundException snfe) {
          Baobab.error(navigator,
                       Baobab.message.getString("Error.SequenceNotFound") +
                       ": " + snfe.getSequenceId());
        }
        catch (IOException ioe) {
          Baobab.error(navigator,
                       Baobab.message.getString("Error.io.in") + ": " +
                       ioe.getMessage());
        }
      }
    });

    actions.put("Write sequences",
                new AbstractAction(Baobab.message.getString(
        "Navigator.Attributes.Sequences.Write")) {
      public void actionPerformed(ActionEvent e) {
        sequenceIODialog.setReadOrWrite(false);
        int t = sequenceIODialog.showSaveDialog(navigator);
        if (t == sequenceIODialog.CANCEL) {
          return; // Operation canceled.
        }
        try {
          SequenceContainer sc = XTreeTools.getSequenceContainer(navigator.
              getCurrentNode(), new DefaultAlphabet());
          sequenceIODialog.writeSequenceFile(sc);
        }
        catch (BadCharException bce) {
          Baobab.error(navigator,
                       Baobab.message.getString("Error.Sequence.badChar") +
                       ": " + bce.getMessage());
        }
        catch (Exception ex) {
          ex.printStackTrace();
          Baobab.error(navigator,
                       Baobab.message.getString("Error.unexpected") + ": " +
                       ex.getMessage());
        }
      }
    });

    actions.put("Compute ancestral states -- parsimony",
                new AbstractAction(Baobab.message.getString(
        "Navigator.ComparativeAnalysis.AncestralStates.Discrete.Parsimony")) {
      public void actionPerformed(ActionEvent e) {
        String varName = chooseAttributeName();
        try {
          if(varName != null) {
            Map nodesStates = new HashMap();
            XTreeTools.computeAncestralStatesUsingParsimonyFirstPass(navigator.
                getCurrentNode(), varName, nodesStates); //Otherwise operation cancelled
            XTreeTools.computeAncestralStatesUsingParsimonySecondPassInteractive(navigator.getCurrentNode(), varName, nodesStates, navigator);
            getTree().setColorsOfTree(COLOR_DEFAULT,
                                      mi.getNavigatorNumber(COLOR_SELECTED));
            ColoredTree.setColorsOfAllSubtrees(getCurrentNode(), COLOR_SELECTED,
                                               mi.getNavigatorNumber(COLOR_SELECTED));
            rebuildMenus();
            actualizeAttributesEditor();
            mi.redrawMainInterface();
            mi.redrawAllCurrentNavigators();
          }
        }
        catch (AttributeNotFoundException anfe) {
          Baobab.error(navigator,
                       Baobab.message.getString("Error.Attribute.NotFound") +
                       ": " + anfe.getAttributeName() + ".");
        }
        catch (NodeException ne) {
          Baobab.error(navigator,
                       Baobab.message.getString("Error.Node.OnlyOneSon") +
                       ": " + ne.getNode().getName() + "(Id=" + ne.getNode().getId() + ").");
        }
      }
    });

    actions.put("Parsimony score knowing ancestral states",
                new AbstractAction(Baobab.message.getString(
        "Navigator.ComparativeAnalysis.AncestralStates.Discrete.ParsimonyScore")) {
      public void actionPerformed(ActionEvent e) {
        String varName = chooseAttributeName();
        try {
          if(varName != null && !TextTools.isVoid(varName)) {
            int score = XTreeTools.getParsimonyScoreKnowingAncestralStates(navigator.getCurrentNode(), varName); //Otherwise operation cancelled
            Baobab.message(navigator, Baobab.message.getString("Navigator.ComparativeAnalysis.AncestralStates.Discrete.ParsimonyScoreResult") + " " + score);
          }
        }
        catch (AttributeNotFoundException anfe) {
          Baobab.error(navigator,
                       Baobab.message.getString("Error.Attribute.NotFound") +
                       ": " + anfe.getAttributeName() + ".");
        }
      }
    });

    actions.put("Compute ancestral states -- non ambiguous",
                new AbstractAction(Baobab.message.getString(
        "Navigator.ComparativeAnalysis.AncestralStates.Discrete.NonAmbiguous")) {
      public void actionPerformed(ActionEvent e) {
        String varName = chooseAttributeName();
        try {
          if(varName != null) {
            XTreeTools.computeNonAmbiguousAncestralStates(navigator.getCurrentNode(), varName); //Otherwise operation cancelled
            ColoredTree.setColorsOfAllSubtrees(getCurrentNode(), COLOR_SELECTED,
                                               mi.getNavigatorNumber(COLOR_SELECTED));
            rebuildMenus();
            actualizeAttributesEditor();
            mi.redrawMainInterface();
            mi.redrawAllCurrentNavigators();
          }
        }
        catch (AttributeNotFoundException anfe) {
          Baobab.error(navigator,
                       Baobab.message.getString("Error.Attribute.NotFound") +
                       ": " + anfe.getAttributeName() + ".");
        }
        catch (NodeException ne) {
          Baobab.error(navigator,
                       Baobab.message.getString("Error.Node.OnlyOneSon") +
                       ": " + ne.getNode().getName() + "(Id=" + ne.getNode().getId() + ").");
        }
      }
    });

    actions.put("Compute ancestral states -- brownian",
                new AbstractAction(Baobab.message.getString(
        "Navigator.ComparativeAnalysis.AncestralStates.Continuous.Brownian")) {
      public void actionPerformed(ActionEvent e) {
        String varName = chooseAttributeName();
        try {
          if(varName != null) PIC.ancestor(navigator.getCurrentNode(), varName);//Otherwise operation cancelled
        }
        catch (AttributeNotFoundException anfe) {
          Baobab.error(navigator,
                       Baobab.message.getString("Error.Attribute.NotFound") +
                       ": " + anfe.getAttributeName() + ".");
        }
        getTree().setColorsOfTree(COLOR_DEFAULT,
                                  mi.getNavigatorNumber(COLOR_SELECTED));
        ColoredTree.setColorsOfAllSubtrees(getCurrentNode(), COLOR_SELECTED,
                                           mi.getNavigatorNumber(COLOR_SELECTED));
        rebuildMenus();
        actualizeAttributesEditor();
        mi.redrawMainInterface();
        mi.redrawAllCurrentNavigators();
      }
    });

    actions.put("Compute ancestral states -- weighted brownian",
                new AbstractAction(Baobab.message.getString(
        "Navigator.ComparativeAnalysis.AncestralStates.Continuous.BrownianWeighted")) {
      public void actionPerformed(ActionEvent e) {
        String varName = chooseAttributeName();
        try {
          if(varName != null) PIC.ancestorWeighted(navigator.getCurrentNode(), varName); //Otherwise operation cancelled
        }
        catch (AttributeNotFoundException anfe) {
          Baobab.error(navigator,
                       Baobab.message.getString("Error.Attribute.NotFound") +
                       ": " + anfe.getAttributeName() + ".");
        }
        catch (NodeException ne) {
          Baobab.error(navigator,
                       Baobab.message.getString("Error.Node.NoLength") + ":" +
                       ne.getNode().getName() + "(" + ne.getNode().getId() +
                       ").");
        }
        getTree().setColorsOfTree(COLOR_DEFAULT,
                                  mi.getNavigatorNumber(COLOR_SELECTED));
        ColoredTree.setColorsOfAllSubtrees(getCurrentNode(), COLOR_SELECTED,
                                           mi.getNavigatorNumber(COLOR_SELECTED));
        rebuildMenus();
        actualizeAttributesEditor();
        mi.redrawMainInterface();
        mi.redrawAllCurrentNavigators();
      }
    });

    actions.put("Compute branch lengths -- Grafen (1989)",
                new AbstractAction(Baobab.message.
                                   getString(
        "Navigator.ComparativeAnalysis.BranchLengths.Grafen")) {
      public void actionPerformed(ActionEvent e) {
        try {
          String powerS = JOptionPane.showInputDialog(
              navigator,
              Baobab.message.getString(
              "Navigator.ComparativeAnalysis.BranchLengths.Grafen.Power"),
              Baobab.message.getString("Question"),
              JOptionPane.QUESTION_MESSAGE);
          if(powerS != null) {//Otherwise operation cancelled
            double power = Double.parseDouble(powerS);
            if (power <= 0) throw new NumberFormatException("Negative power.");
            PhylogeneticRegression.computeBranchLengths(navigator.
                getCurrentNode(), power,
                (double) navigator.getTree().getNumberOfLeaves());
            navigator.getTree().treeIsModified("Branch lengths changed");
            mi.rebuildMenu();
          }
        } catch(NumberFormatException nfe) {
          Baobab.error(navigator,
                       Baobab.message.getString("Navigator.ComparativeAnalysis.BranchLengths.Grafen.PowerError"));
        }
      }
    });

    /****************************************************************************/

  }

  private void initGUI(ColoredTree root) {
    treeDrawing = new ColoredTreeDrawing(root, getLocale(), getOS());
    treeDrawing.addActionListener(new TreeDrawingListener());
    treeDrawing.setPreferredSize(
      Math.max(this.TREE_MINIMAL_WIDTH, width),
      Math.max(this.TREE_MINIMAL_HEIGHT, height)
      );
    treeDrawing.setPreferredSize(600, 400);
    treeDrawing.setMargins(new Insets(15, 50, 15, 400)); // There may be long names!
    treeDrawing.setDrawColors(false);
    treeDrawing.setDrawClickableNodes(true);
    treeDrawing.setDrawBranchesNumbers(true);
    defaultColoringMethod = new ColoringMethod() {
      public Color getColor(XNode node) {
        return Color.black;
      }
    };
    treeDrawing.setColoringMethod(defaultColoringMethod);
    treeDrawing.scale();
    lift = new JScrollPane(treeDrawing);

    infos = new JTextArea(Baobab.message.getString("Navigator.InfosRoot"));
    infos.setLineWrap(true);
    infos.setWrapStyleWord(true);
    infos.setForeground(this.COLOR_SELECTED);
    infos.setEditable(false);
    infos.setBorder(BorderFactory.createTitledBorder(BorderFactory.
        createLineBorder(COLOR_SELECTED),
        Baobab.message.getString("Navigator.Note")));

    JPanel mainPanel = new JPanel();
    mainPanel.setLayout(new BorderLayout());
    mainPanel.add(lift, BorderLayout.CENTER);
    mainPanel.add(infos, BorderLayout.SOUTH);

    attributesEditor = new TreeAttributesEditor(treeDrawing);
    attributesEditor.addTableModelListener(new TableModelListener() {
      public void tableChanged(TableModelEvent e) {
        getCurrentNode().setAttributes(attributesEditor.getAttributesTableModel().getAttributes());
        mi.redrawMainInterface();
        mi.redrawAllCurrentNavigators();
      }
    });

    /**
     * And then the navigator remote:
     */
    navigatorRemote = new NavigatorRemote(treeDrawing);
    ActionMap remoteActions = navigatorRemote.createActionMap();
    remoteToolBar = new JToolBar("Remote", JToolBar.VERTICAL);
    remoteToolBar.add(remoteActions.get(NavigatorRemote.UP));
    remoteToolBar.add(remoteActions.get(NavigatorRemote.DOWN));
    remoteToolBar.add(remoteActions.get(NavigatorRemote.BACK));

    split = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, mainPanel, attributesEditor);
    split.setOneTouchExpandable(true);

    sequencesEditor = new SequencesEditor(this);
  }

  private void initToolBars() {
    // Zoom toolbar:
    ZoomControls zoomControls = new ZoomControls(treeDrawing, lift);
    //zoomPar = new ZoomDialog(this);
    //zoomPar.addZoomControls(zoomControls);
    mi.getZoomDialog().addZoomControls(zoomControls);

    TreeDrawingControls tdControls = new TreeDrawingControls(treeDrawing);

    // Navigator toolbar:
    navigatorToolBar = new JToolBar();
    navigatorToolBar.add(zoomControls.getZoomWidthMinus());
    navigatorToolBar.add(zoomControls.getZoomWidthPlus());
    navigatorToolBar.add(zoomControls.getZoomHeightMinus());
    navigatorToolBar.add(zoomControls.getZoomHeightPlus());
    navigatorToolBar.add(tdControls.drawLengthsButton);
    navigatorToolBar.add(tdControls.drawBootstrapsButton);
    navigatorToolBar.add(tdControls.drawIdsButton);
 //   navigatorToolBar.add(tdControls.drawGroupsButton);
    navigatorToolBar.add(tdControls.drawClickableNodesButton);
    navigatorToolBar.add(tdControls.drawNumbersAtFirstBranchesButton);
    navigatorToolBar.add(tdControls.drawLeavesNamesButton);

    // Edition toolbar:
    editionToolBar = new JToolBar("Edition tools", JToolBar.VERTICAL);
    editionToolBar.add(actions.get(EditionTools.NEW_BRANCH));
    editionToolBar.add(actions.get(EditionTools.COPY_BRANCH));
    editionToolBar.add(actions.get(EditionTools.CUT_BRANCH));
    editionToolBar.add(actions.get(EditionTools.INSERT_BRANCH));
    editionToolBar.add(actions.get(EditionTools.DELETE_BRANCH));
    editionToolBar.add(actions.get(EditionTools.SWAP_BRANCHES));
    editionToolBar.add(actions.get(EditionTools.REROOT_TREE));
    editionToolBar.add(actions.get(EditionTools.SET_NAME));
  }

  private void initLayout() {
    windowContent = getContentPane();
    windowContent.setLayout(new BorderLayout());
    windowContent.add(navigatorToolBar, BorderLayout.NORTH);
    windowContent.add(editionToolBar, BorderLayout.WEST);
    windowContent.add(remoteToolBar, BorderLayout.EAST);
    windowContent.add(split, BorderLayout.CENTER);
  }

  /****************************************************************************/

  private void initMenus() {

    menu = new JMenuBar();

    /**
     * The 'edit' menu:
     */
    setName = new JMenuItem(actions.get(EditionTools.SET_NAME));
    setName.setMnemonic(Baobab.message.getString(
        "Navigator.Edit.SetNameMnemo").charAt(0));

    deleteName = new JMenuItem(actions.get(EditionTools.DELETE_NAME));
    deleteName.setMnemonic(Baobab.message.getString(
        "Navigator.Edit.DeleteNameMnemo").charAt(0));

    edit = new JMenu(Baobab.message.getString("Navigator.Edit"));
    edit.setMnemonic(Baobab.message.getString("Navigator.EditMnemo").charAt(0));

    cut = new JMenuItem(actions.get(EditionTools.CUT_BRANCH));
    cut.setMnemonic(Baobab.message.getString("Navigator.EditCutMnemo").
                    toCharArray()[0]);
    cut.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X,
                                              InputEvent.CTRL_MASK));

    del = new JMenuItem(actions.get(EditionTools.DELETE_BRANCH));
    del.setMnemonic(Baobab.message.getString("Navigator.EditDelMnemo").
                    toCharArray()[0]);
    del.setAccelerator(KeyStroke.getKeyStroke("DELETE"));

    copy = new JMenuItem(actions.get(EditionTools.COPY_BRANCH));
    copy.setMnemonic(Baobab.message.getString("Navigator.EditCopyMnemo").
                     toCharArray()[0]);
    copy.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C,
                                               InputEvent.CTRL_MASK));

    addBranch = new JMenuItem(actions.get(EditionTools.ADD_BRANCH));
    addBranch.setMnemonic(Baobab.message.getString(
        "Navigator.EditAddBranchMnemo").toCharArray()[0]);
    addBranch.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_B,
        InputEvent.CTRL_MASK));

    addNode = new JMenuItem(actions.get(EditionTools.INSERT_BRANCH));
    addNode.setMnemonic(Baobab.message.getString("Navigator.EditAddNodeMnemo").
                        toCharArray()[0]);
    addNode.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V,
                                                  InputEvent.CTRL_MASK));

    setAttributes = new JMenu(Baobab.message.getString(
        "Navigator.EditSetAttributes"));
    setAttributes.setMnemonic(Baobab.message.getString(
        "Navigator.EditSetAttributesMnemo").toCharArray()[0]);

    deleteLength = new JMenuItem(actions.get(EditionTools.DELETE_LENGTH));
    deleteLength.setMnemonic(Baobab.message.getString(
        "Navigator.EditDeleteLengthMnemo").toCharArray()[0]);

    deleteBootstrap = new JMenuItem(actions.get(EditionTools.DELETE_BOOTSTRAP));
    deleteBootstrap.setMnemonic(Baobab.message.getString(
        "Navigator.EditDeleteBootstrapMnemo").toCharArray()[0]);

    setLength = new JMenuItem(actions.get(EditionTools.SET_LENGTH));
    setLength.setMnemonic(Baobab.message.getString(
        "Navigator.EditSetLengthMnemo").toCharArray()[0]);

    setBootstrap = new JMenuItem(actions.get(EditionTools.SET_BOOTSTRAP));
    setBootstrap.setMnemonic(Baobab.message.getString(
        "Navigator.EditSetBootstrapMnemo").toCharArray()[0]);

    setAttributes.add(deleteLength);
    setAttributes.add(deleteBootstrap);
    setAttributes.add(setLength);
    setAttributes.add(setBootstrap);

    newBranch = new JMenuItem(actions.get(EditionTools.NEW_BRANCH));
    newBranch.setMnemonic(Baobab.message.getString(
        "Navigator.Edit.NewBranchMnemo").toCharArray()[0]);

    newMultBranch = new JMenuItem(actions.get(EditionTools.NEW_MULT_BRANCH));
    newMultBranch.setMnemonic(Baobab.message.getString(
        "Navigator.Edit.NewMultBranchMnemo").toCharArray()[0]);

    newSons = new JMenuItem(actions.get(EditionTools.NEW_SONS));
    newSons.setMnemonic(Baobab.message.getString("Navigator.Edit.NewSons.Mnemo").charAt(0));

    unresolve = new JMenuItem(actions.get(EditionTools.UNRESOLVE));
    unresolve.setMnemonic(Baobab.message.getString("EditionTools.UnresolveMnemo").charAt(0));

    deepUnresolve = new JMenuItem(actions.get(EditionTools.DEEPUNRESOLVE));
    deepUnresolve.setMnemonic(Baobab.message.getString("EditionTools.DeepUnresolveMnemo").charAt(0));

    leavesEdition = new JMenu(Baobab.message.getString("Navigator.Edit.Leaves"));
    leavesEdition.setMnemonic(Baobab.message.getString(
        "Navigator.Edit.LeavesMnemo").toCharArray()[0]);

    truncNames = new JMenuItem(actions.get(EditionTools.TRUNC_LEAVES_NAMES));
    truncNames.setMnemonic(Baobab.message.getString(
        "EditionTools.TruncLeavesNamesMnemo").charAt(0));

    leavesEdition.add(truncNames);

    removeBlanksFromNames = new JMenuItem(actions.get(EditionTools.
        REMOVE_BLANKS_LEAVES_NAMES));
    removeBlanksFromNames.setMnemonic(Baobab.message.getString(
        "EditionTools.RemoveBlanksFromLeavesNamesMnemo").charAt(0));

    leavesEdition.add(removeBlanksFromNames);

    correctNames = new JMenuItem(actions.get(EditionTools.
        CORRECT_NAMES));
    correctNames.setMnemonic(Baobab.message.getString(
        "EditionTools.CorrectNamesMnemo").charAt(0));

    leavesEdition.add(correctNames);

    translateNames = new JMenuItem(actions.get(EditionTools.
        TRANSLATE_NAMES));
    translateNames.setMnemonic(Baobab.message.getString(
        "EditionTools.TranslateNamesMnemo").charAt(0));

    leavesEdition.add(translateNames);

    reverseNames = new JMenuItem(actions.get(EditionTools.
        REVERSE_NAMES));
    reverseNames.setMnemonic(Baobab.message.getString(
        "EditionTools.ReverseNamesMnemo").charAt(0));

    leavesEdition.add(reverseNames);

    edit.add(setName);
    edit.add(deleteName);
    edit.add(cut);
    edit.add(del);
    edit.add(copy);
    edit.add(addBranch);
    edit.add(addNode);
    edit.add(setAttributes);
    edit.add(newBranch);
    edit.add(newMultBranch);
    edit.add(newSons);
    edit.add(unresolve);
    edit.add(deepUnresolve);
    edit.add(leavesEdition);

    menu.add(edit);

    /**
     * The 'display' menu.
     */
    display = new JMenu(Baobab.message.getString("Navigator.Display"));
    display.setMnemonic(Baobab.message.getString("Navigator.DisplayMnemo").
                        toCharArray()[0]);

    swap = new JMenuItem(actions.get(EditionTools.SWAP_BRANCHES));
    swap.setMnemonic(Baobab.message.getString("Navigator.DisplaySwapMnemo").
                     toCharArray()[0]);

    reRoot = new JMenuItem(actions.get(EditionTools.REROOT_TREE));
    reRoot.setMnemonic(Baobab.message.getString("Navigator.DisplayReRootMnemo").
                       toCharArray()[0]);

    display.add(swap);
    display.add(reRoot);

    menu.add(display);

    /**
     * The 'group' menu:
     */
//    group = new JMenu(Baobab.message.getString("Navigator.Group"));
//    group.setMnemonic(Baobab.message.getString("Navigator.GroupMnemo").
//                      toCharArray()[0]);

//    createGroup = new JMenuItem(actions.get(EditionTools.SET_GROUP));
//    createGroup.setMnemonic(Baobab.message.getString(
//        "Navigator.GroupCreateMnemo").toCharArray()[0]);

//    renameLeaf = new JMenuItem(actions.get(EditionTools.SET_NAME));
//    renameLeaf.setMnemonic(Baobab.message.getString(
//        "Navigator.GroupRenameMnemo").toCharArray()[0]);

//    deleteGroup = new JMenuItem(actions.get(EditionTools.DELETE_GROUP));
//    deleteGroup.setMnemonic(Baobab.message.getString(
//        "Navigator.GroupDeleteMnemo").toCharArray()[0]);
//    deleteGroup.setEnabled(false);

//    group.add(createGroup);
//    group.add(deleteGroup);

//    menu.add(group);

    /**
     * The 'attributes' menu:
     */

    attributes = new JMenu(Baobab.message.getString("Navigator.Attributes"));
    attributes.setMnemonic(Baobab.message.getString("Navigator.AttributesMnemo").
                           toCharArray()[0]);

    showAttributesEditor = new JCheckBoxMenuItem(Baobab.message.getString(
        "Navigator.ShowAttributesEditor"), false);
    showAttributesEditor.setMnemonic(Baobab.message.getString(
        "Navigator.ShowAttributesEditorMnemo").charAt(0));
    showAttributesEditor.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        //attributesEditor.setVisible(showAttributesEditor.isSelected());
        if (showAttributesEditor.isSelected()) {
          split.setDividerLocation(0.60);
        }
        else {
          split.setDividerLocation(1.00);
        }
      }
    });

    attributes.add(showAttributesEditor);

    date = new JMenu(Baobab.message.getString("Navigator.Attributes.Date"));
    date.setMnemonic(Baobab.message.getString("Navigator.Attributes.DateMnemo").
                     toCharArray()[0]);

    initDate = new JMenuItem(actions.get("Init date"));
    initDate.setMnemonic(Baobab.message.getString(
        "Navigator.Attributes.Date.InitMnemo").charAt(0));
    date.add(initDate);

    computeLengths = new JMenuItem(actions.get("Compute lengths"));
    computeLengths.setMnemonic(Baobab.message.getString(
        "Navigator.Attributes.Date.ComputeMnemo").charAt(0));
    date.add(computeLengths);

    attributes.add(date);

    loadAttributes = new JMenu(Baobab.message.getString(
        "Navigator.Attributes.Load"));
    loadAttributes.setMnemonic(Baobab.message.getString(
        "Navigator.Attributes.LoadMnemo").charAt(0));

    loadAttributesFromTable = new JMenuItem(actions.get(
        "Load attributes from table"));
    loadAttributesFromTable.setMnemonic(Baobab.message.getString(
        "Navigator.Attributes.Load.TableMnemo").charAt(0));
    loadAttributes.add(loadAttributesFromTable);

    attributes.add(loadAttributes);

    writeAttributes = new JMenu(Baobab.message.getString(
        "Navigator.Attributes.Write"));
    writeAttributes.setMnemonic(Baobab.message.getString(
        "Navigator.Attributes.WriteMnemo").charAt(0));

    writeAttributesToTable = new JMenuItem(actions.get(
        "Write attributes to table"));
    writeAttributesToTable.setMnemonic(Baobab.message.getString(
        "Navigator.Attributes.Write.TableMnemo").charAt(0));
    writeAttributes.add(writeAttributesToTable);

    attributes.add(writeAttributes);

    sequencesAttributes = new JMenu(Baobab.message.getString(
        "Navigator.Attributes.Sequences"));
    sequencesAttributes.setMnemonic(Baobab.message.getString(
        "Navigator.Attributes.SequencesMnemo").charAt(0));

    loadSequences = new JMenuItem(actions.get(
        "Load sequences"));
    loadSequences.setMnemonic(Baobab.message.getString(
        "Navigator.Attributes.Sequences.LoadMnemo").charAt(0));
    sequencesAttributes.add(loadSequences);

    writeSequences = new JMenuItem(actions.get(
        "Write sequences"));
    writeSequences.setMnemonic(Baobab.message.getString(
        "Navigator.Attributes.Sequences.WriteMnemo").charAt(0));
    sequencesAttributes.add(writeSequences);

    final Navigator navigator = this;
    editSequences = new JMenuItem(new AbstractAction(Baobab.message.getString("Navigator.Attributes.Sequences.Edit")) {
      public void actionPerformed(ActionEvent e) {
        try {
          SequenceContainer seqCont = XTreeTools.getSequenceContainer(getCurrentNode(), new DefaultAlphabet());
          if(seqCont == null || seqCont.getNumberOfSequences() == 0) {
            Baobab.message(navigator, Baobab.message.getString("Navigator.Attributes.Sequences.Error.NoSequence"));
          } else {
            sequencesEditor.getEditor().setSequences(seqCont);
            sequencesEditor.setVisible(true);
          }
        } catch(Exception ex) {
          ex.printStackTrace();
        }
      }
    });
    editSequences.setMnemonic(Baobab.message.getString(
        "Navigator.Attributes.Sequences.EditMnemo").charAt(0));
    sequencesAttributes.add(editSequences);

    attributes.add(sequencesAttributes);

    menu.add(attributes);

    /**
     * The 'Comparative Analysis' menu:
     */
    comparativeAnalysis = new JMenu(Baobab.message.getString("Navigator.CA"));
    comparativeAnalysis.setMnemonic(Baobab.message.getString(
        "Navigator.CAMnemo").charAt(0));

    ancestralStates = new JMenu(Baobab.message.getString(
      "Navigator.ComparativeAnalysis.AncestralStates"));
    ancestralStates.setMnemonic(Baobab.message.getString(
      "Navigator.ComparativeAnalysis.AncestralStatesMnemo").charAt(0));

    ancestralStatesContinuous = new JMenu(Baobab.message.getString(
      "Navigator.ComparativeAnalysis.AncestralStates.Continuous"));
    ancestralStatesContinuous.setMnemonic(Baobab.message.getString(
      "Navigator.ComparativeAnalysis.AncestralStates.ContinuousMnemo").charAt(0));

    computeAncestorsBrownian = new JMenuItem(actions.get("Compute ancestral states -- brownian"));
    computeAncestorsBrownian.setMnemonic(Baobab.message.getString(
      "Navigator.ComparativeAnalysis.AncestralStates.Continuous.BrownianMnemo").charAt(0));
    ancestralStatesContinuous.add(computeAncestorsBrownian);

    computeAncestorsBrownianWeighted = new JMenuItem(actions.get("Compute ancestral states -- weighted brownian"));
    computeAncestorsBrownianWeighted.setMnemonic(Baobab.message.getString(
      "Navigator.ComparativeAnalysis.AncestralStates.Continuous.BrownianWeightedMnemo").charAt(0));
    ancestralStatesContinuous.add(computeAncestorsBrownianWeighted);

    ancestralStates.add(ancestralStatesContinuous);

    ancestralStatesDiscrete = new JMenu(Baobab.message.getString(
      "Navigator.ComparativeAnalysis.AncestralStates.Discrete"));
    ancestralStatesDiscrete.setMnemonic(Baobab.message.getString(
      "Navigator.ComparativeAnalysis.AncestralStates.DiscreteMnemo").charAt(0));

    computeAncestorsParsimony = new JMenuItem(actions.get("Compute ancestral states -- parsimony"));
    computeAncestorsParsimony.setMnemonic(Baobab.message.getString(
    "Navigator.ComparativeAnalysis.AncestralStates.Discrete.ParsimonyMnemo").charAt(0));

    ancestralStatesDiscrete.add(computeAncestorsParsimony);

    getParsimonyScore = new JMenuItem(actions.get("Parsimony score knowing ancestral states"));
    getParsimonyScore.setMnemonic(Baobab.message.getString(
      "Navigator.ComparativeAnalysis.AncestralStates.Discrete.ParsimonyScoreMnemo").charAt(0));

    ancestralStatesDiscrete.add(getParsimonyScore);

    computeNonAmbiguousAncestors = new JMenuItem(actions.get("Compute ancestral states -- non ambiguous"));
    computeNonAmbiguousAncestors.setMnemonic(Baobab.message.getString(
      "Navigator.ComparativeAnalysis.AncestralStates.Discrete.NonAmbiguousMnemo").charAt(0));

    ancestralStatesDiscrete.add(computeNonAmbiguousAncestors);

    ancestralStates.add(ancestralStatesDiscrete);
    comparativeAnalysis.add(ancestralStates);


    branchLengths = new JMenu(Baobab.message.getString(
      "Navigator.ComparativeAnalysis.BranchLengths"));
    branchLengths.setMnemonic(Baobab.message.getString("Navigator.ComparativeAnalysis.BranchLengthsMnemo").charAt(0));

    computeBranchLengthsGrafen = new JMenuItem(actions.get("Compute branch lengths -- Grafen (1989)"));
    computeBranchLengthsGrafen.setMnemonic(Baobab.message.getString("Navigator.ComparativeAnalysis.BranchLengths.GrafenMnemo").charAt(0));

    branchLengths.add(computeBranchLengthsGrafen);

    comparativeAnalysis.add(branchLengths);

    menu.add(comparativeAnalysis);

    this.setJMenuBar(menu);
  }

  /****************************************************************************/

  public void setOS(OperatingSystem OS) {
    this.OS = OS;
  }

  public OperatingSystem getOS() {
    return OS;
  }

  /**
   * Subclasses:
   */
  class TreeDrawingListener
      implements ActionListener {

    public void actionPerformed(ActionEvent e) {
      //System.out.println("Tree is modified.");
      getTree().setColorsOfTree(COLOR_DEFAULT,
                                mi.getNavigatorNumber(COLOR_SELECTED));
      ColoredTree.setColorsOfAllSubtrees(getCurrentNode(), COLOR_SELECTED,
                                         mi.getNavigatorNumber(COLOR_SELECTED));
      rebuildMenus();
      actualizeAttributesEditor();
      mi.redrawMainInterface();
      mi.redrawAllCurrentNavigators();
    }
  }

  /****************************************************************************/

  class Fermeture
      extends WindowAdapter {
    public void windowClosing(WindowEvent ev) {
      mi.shutOneNavigator(COLOR_SELECTED);
    }
  }

  /****************************************************************************/

  public void rebuildMenus() {
    String[] contenu;
    ColoredTree tree = (ColoredTree) treeDrawing.getTree();
    ColoredNode currentNode = (ColoredNode) treeDrawing.getCurrentNode();

    //Groups:
    deleteName.setEnabled(!currentNode.isLeaf() && treeDrawing.isGroup(currentNode));

    //Adding inner nodes:
    if (mi.getOtherTrees().size() == 0) {
      addNode.setEnabled(false);
    }
    else {
      addNode.setEnabled(true);
    }

    //Adding branches:
    if (mi.areMultifurcationsAllowed()) {
      if (currentNode.isLeaf() || mi.getOtherTrees().size() == 0) {
        addBranch.setEnabled(false);
      }
      else {
        addBranch.setEnabled(true);
      }
      if(TreeTools.isRoot(currentNode)) newMultBranch.setEnabled(false);
      else                              newMultBranch.setEnabled(true);
    }
    else {
      addBranch.setEnabled(false);
      newMultBranch.setEnabled(false);
    }

    //Branch lengths and bootstraps:
    //if (allowValuesModifying.isSelected()) {
    //  if (currentNode.hasLength()) {
    //    deleteLength.setEnabled(true);
    //  }
    //  else {
    //    deleteLength.setEnabled(false);
    //  }
    //  if (currentNode.hasBootstrap()) {
    //    deleteBootstrap.setEnabled(true);
    //  }
    //  else {
    //    deleteBootstrap.setEnabled(false);
    //  }
    //  setLength.setEnabled(true);
    //  setBootstrap.setEnabled(true);
    //}
    //else {
    //  deleteLength.setEnabled(false);
    //  deleteBootstrap.setEnabled(false);
    //  setLength.setEnabled(false);
    //  setBootstrap.setEnabled(false);
    //}

    if (currentNode == tree.getRootNode()) {
      reRoot.setEnabled(false);
    }
    else {
      reRoot.setEnabled(true);
    }

    //Node name:
    deleteName.setEnabled(!currentNode.isLeaf() && currentNode.hasName());

    //Update infos pane
    infos.setText("");
    if (currentNode == tree.getRootNode()) {
      infos.setText(Baobab.message.getString("Navigator.InfosRoot"));
    }
    if (currentNode.isLeaf()) {
      infos.setText(Baobab.message.getString("Navigator.InfosLeaf"));
    }
    if (currentNode.hasName() && !currentNode.isLeaf()) {
      infos.setText(Baobab.message.getString("Navigator.InfosGroup1")
                    + currentNode.getName() + ", "
                    + currentNode.getNumberOfLeaves() + " "
                    + Baobab.message.getString("Navigator.InfosGroup2"));
    }
    infos.append(Baobab.message.getString("Navigator.InfosSubtree"));
  }

  /****************************************************************************/

  public void actualizeAttributesEditor() {
    //AttributesEditor.AttributesTableModel atm = new AttributesEditor.
    //    AttributesTableModel(getCurrentNode().getAttributes());
    //attributesEditor.setAttributesTableModel(atm);
    //attributesEditor.updateUI();
    attributesEditor.actualizeEditor();
  }

  /****************************************************************************/

  public void redraw() {
    treeDrawing.scale();
    treeDrawing.repaint();
    actualizeAttributesEditor();
    windowContent.validate();
  }

  /****************************************************************************/

  public void restoreDefaultColoringMethod() {
    getTreeDrawing().setColoringMethod(this.defaultColoringMethod);
  }

  /****************************************************************************/

  //Zoom control:
  public int getDrawingWidth() {
    return width;
  }

  public int getDrawingHeight() {
    return height;
  }

  public void setDrawingWidth(int width) {
    this.width = width;
  }

  public void setDrawingHeight(int height) {
    this.height = height;
  }

  //rcupration des rfrences des donnes:
  public TreePath getTreePath() {
    return getTree().getTreePath(getCurrentNode());
  }

  public ColoredTree getTree() {
    return (ColoredTree) treeDrawing.getTree();
  }

  public ColoredNode getCurrentNode() {
    return (ColoredNode) treeDrawing.getCurrentNode();
  }

  public MainInterface getMainInterface() {
    return mi;
  }

  public ColoredTreeDrawing getTreeDrawing() {
    return treeDrawing;
  }

  public String chooseAttributeName() {
    String[] attNames = (String[])XTreeTools.getAvailableTags(getTree(), false).toArray(new String[0]);
    Object c = JOptionPane.showInputDialog(this, Baobab.message.getString("Navigator.AttributeName"), Baobab.message.getString("Question"), JOptionPane.QUESTION_MESSAGE, null, attNames, attNames[0]);
    if (c == null) return null;
    return (String)c;
  }

  /****************************************************************************/

  private MainInterface mi;
  private OperatingSystem OS;
  private ColoredTreeDrawing treeDrawing;
  private JScrollPane lift;

  //Menu components:
  private JMenuBar menu;

  private JMenu edit, setAttributes, leavesEdition;
  private JMenuItem setName, deleteName, cut, del, copy, addBranch, addNode, deleteLength,
      deleteBootstrap,
      setLength, setBootstrap, newBranch, newMultBranch, truncNames, correctNames,
      removeBlanksFromNames, translateNames, reverseNames, newSons, unresolve, deepUnresolve;

  private JMenu display;
  private JMenuItem swap, reRoot;

 // private JMenu group;
 // private JMenuItem createGroup, renameLeaf, deleteGroup;
  private JCheckBoxMenuItem showAttributesEditor;

  private JMenu attributes;
  private JMenu date;
  private JMenuItem initDate, computeLengths;
  private JMenu loadAttributes;
  private JMenuItem loadAttributesFromTable;
  private JMenu writeAttributes;
  private JMenuItem writeAttributesToTable;
  private JMenu sequencesAttributes;
  private JMenuItem loadSequences, writeSequences, editSequences;

  private JMenu comparativeAnalysis;
  private JMenu ancestralStates, branchLengths, ancestralStatesDiscrete, ancestralStatesContinuous;
  private JMenuItem computeAncestorsBrownian, computeAncestorsBrownianWeighted, computeAncestorsParsimony, getParsimonyScore, computeNonAmbiguousAncestors, computeBranchLengthsGrafen;

  //Tool bars:
  private JToolBar editionToolBar, remoteToolBar, navigatorToolBar;

  //Other stuff:
  private NavigatorRemote navigatorRemote;
  private ActionMap actions;
  private JSplitPane split;
  private JTextArea infos;
  private TreeAttributesEditor attributesEditor;
  private TableAttributesTranslationDialog TATD;
  private SequencesEditor sequencesEditor;

//  private String[] contenu;
  private Container windowContent;
  private int width, height;

//  private ZoomControls ZC;
  private final Color COLOR_DEFAULT = ColoredTreeDrawing.COLOR_DEFAULT;
  public final Color COLOR_SELECTED;
  private final int TREE_MINIMAL_WIDTH = 300; //pixels
  private final int TREE_MINIMAL_HEIGHT = 100; //pixels
  private final int WINDOW_WIDTH = 600; //pixels
  private final int WINDOW_HEIGHT = 600; //pixels
  private ColoringMethod defaultColoringMethod;
}
