// [English version follows the french one]
//
// Copyright ou  ou Copr. Julien Dutheil, UMR 5171 "Gnome, populations,
// interactions, adaptation" CNRS, Universit Montpellier II, IFREMER
// (Septembre 2004)
//
// Julien.Dutheil@univ-montp2.fr
//
// Ce logiciel est un programme informatique servant  diter des arbres
// phylogntiques.
//
// Ce logiciel est rgi par la licence CeCILL soumise au droit franais et
// respectant les principes de diffusion des logiciels libres. Vous pouvez
// utiliser, modifier et/ou redistribuer ce programme sous les conditions
// de la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA
// sur le site "http://www.cecill.info".
//
// En contrepartie de l'accessibilit au code source et des droits de copie,
// de modification et de redistribution accords par cette licence, il n'est
// offert aux utilisateurs qu'une garantie limite.  Pour les mmes raisons,
// seule une responsabilit restreinte pse sur l'auteur du programme,  le
// titulaire des droits patrimoniaux et les concdants successifs.
//
// A cet gard  l'attention de l'utilisateur est attire sur les risques
// associs au chargement,   l'utilisation,   la modification et/ou au
// dveloppement et  la reproduction du logiciel par l'utilisateur tant
// donn sa spcificit de logiciel libre, qui peut le rendre complexe 
// manipuler et qui le rserve donc  des dveloppeurs et des professionnels
// avertis possdant  des  connaissances  informatiques approfondies.  Les
// utilisateurs sont donc invits  charger  et  tester  l'adquation  du
// logiciel  leurs besoins dans des conditions permettant d'assurer la
// scurit de leurs systmes et ou de leurs donnes et, plus gnralement,
//  l'utiliser et l'exploiter dans les mmes conditions de scurit.
//
// Le fait que vous puissiez accder  cet en-tte signifie que vous avez
// pris connaissance de la licence CeCILL, et que vous en avez accept les
// termes.
//
// -------------------------------------------------------------------------
//
//Copyright or  or Copr. Julien Dutheil, UMR 5171 "Gnome, populations,
// interactions, adaptation" CNRS, Universit Montpellier II, IFREMER
// (Septembre 2004)
//
// Julien.Dutheil@univ-montp2.fr
//
//
// This software is a computer program whose purpose is to edit
// phylogenetic trees.
//
// This software is governed by the CeCILL  license under French law and
// abiding by the rules of distribution of free software.  You can  use,
// modify and/ or redistribute the software under the terms of the CeCILL
// license as circulated by CEA, CNRS and INRIA at the following URL
// "http://www.cecill.info".
//
// As a counterpart to the access to the source code and  rights to copy,
// modify and redistribute granted by the license, users are provided only
// with a limited warranty  and the software's author,  the holder of the
// economic rights,  and the successive licensors  have only  limited
// liability.
//
// In this respect, the user's attention is drawn to the risks associated
// with loading,  using,  modifying and/or developing or reproducing the
// software by the user in light of its specific status of free software,
// that may mean  that it is complicated to manipulate,  and  that  also
// therefore means  that it is reserved for developers  and  experienced
// professionals having in-depth computer knowledge. Users are therefore
// encouraged to load and test the software's suitability as regards their
// requirements in conditions enabling the security of their systems and/or
// data to be ensured and,  more generally, to use and operate it in the
// same conditions as regards security.
//
// The fact that you are presently reading this means that you have had
// knowledge of the CeCILL license and that you accept its terms.
//
// -------------------------------------------------------------------------

/**
 * <p>Description : Baobab core classes.</p>
 * <p>Copyright : Copyright (c) Julien Dutheil</p>
 * @author Julien Dutheil
 */

package TreeEditor;

import gnu.jtools.utils.gui.GraphicsTools;
import bio.dendogram.*;
import bio.dendogram.edition.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class SampleOptionDialog extends JDialog {

  public SampleOptionDialog(MainInterface mi) {
    super(mi, "Sample tree", true);
    this.mi = mi;
    initGUI();
    initListeners();
  }

  private void initGUI() {
    Container cp = getContentPane();
    cp.setLayout(new GridLayout(4,2));

    cp.add(new JLabel("Tree:"));
    treeName = new JTextField(15);
    treeName.setEditable(false);
    cp.add(treeName);

    cp.add(new JLabel("Number of leaves to keep:"));
    numberOfLeavesToKeep = new JTextField(5);
    cp.add(numberOfLeavesToKeep);

    cp.add(new JLabel("Sample name:"));
    newTreeName = new JTextField(15);
    cp.add(newTreeName);

    ok = new JButton("Ok");
    cp.add(ok);
    cancel = new JButton("Cancel");
    cp.add(cancel);

    pack();
  }

  private void initListeners() {
    ok.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        //Must check parameters first:
        if(mi.getList().whoseNameIsIt(newTreeName.getText()) > -1) {
           //This tree name already exists.
           errorMsg("Tree name already used.");
           return;
        }
        int nltk = 0;
        try {
          nltk = Integer.parseInt(numberOfLeavesToKeep.getText());
        } catch(NumberFormatException ex) {
          errorMsg("Number of leaves to keep must be a positive integer.");
          return;
        }
        if(nltk >= treeToSample.getNumberOfLeaves()) {
          errorMsg("Number of leaves to keep must be <= number of leaves in the tree.");
          return;
        }
        if(nltk == 0) {
          return;
        }
        ColoredTree sample = (ColoredTree)treeToSample.clone();
        //System.out.println(((ColoredNode)sample.getLeaves().get(0)).getNumberOfAttributes());
        bio.dendogram.edition.EditionTools.sample(sample, nltk);
        mi.addTreeToList(sample, newTreeName.getText(), mi.getPreferredFormatOfCurrentTree());
        setVisible(false);
      }
    });

    cancel.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        setVisible(false);
      }
    });
  }

  public void errorMsg(String message) {
    JOptionPane.showMessageDialog(this, message, "Error! Bad option.", JOptionPane.ERROR_MESSAGE);
  }

  public void showUp(ColoredTree treeToSample) {
    this.treeToSample = treeToSample;
       treeName.setText(treeToSample.getName() + "(" + treeToSample.getNumberOfLeaves() + " leaves)");
    newTreeName.setText(treeToSample.getName() + "_sampled");
    GraphicsTools.center(this);
    setVisible(true);
  }

  private MainInterface mi;
  private ColoredTree treeToSample;
  private JTextField treeName, numberOfLeavesToKeep, newTreeName;
  private JCheckBox sampleGroups;
  private JButton ok, cancel;
}
